-----------------------------------------------------------------------------
-- |
-- Module      :  Camila.DTInvariants
-- Copyright   :  (c) Alexandra Mendes, Joao Ferreira
-- License     :  GPL
--
-- Maintainer  :  Joao Ferreira, Alexandra Mendes
-- Stability   :  experimental
-- Portability :  experimental
--
-- Datatype invariants. 
-- Inspired by the work of Claudia Necco.
-- 
-----------------------------------------------------------------------------

module Camila.DTInvariants (CData(..),Undefined) where

import Camila.Error
import Control.Monad.Error

-----------------------------------------------------------------------------

-- | Class of data types with an invariant. Minimal implementation defines 'inv'.
class CData a where
    -- | Invariant as boolean function.
    inv :: a -> Bool
    inv a = True
    -- | Computation that checks the invariant.
    inv' :: a -> CamilaError a
    inv' a = if (inv a) then return a else fail "Invariant violation"

instance (CData a, CData b) => CData (a,b) where
    inv (a,b) = (inv a) && (inv b)

instance (CData a, CData b) => CData (Either a b) where
    inv (Left a) = inv a
    inv (Right b) = inv b

instance CData () where
  inv _ = True

data Undefined = Undefined
instance Eq Undefined where
  _ == _ = undefined
instance Show Undefined where
  show _ = undefined

instance CData Undefined where
  inv _ = False

-----------------------------------------------------------------------------
