{-# OPTIONS -fglasgow-exts  -fallow-overlapping-instances #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Camila.Error
-- Copyright   :  (c) Alexandra Mendes, João Ferreira
-- License     :  GPL
--
-- Maintainer  :  João Ferreira, Alexandra Mendes
-- Stability   :  experimental
-- Portability :  experimental
--
-- Module that defines Error structure. It uses Error Monad.
-- 
-----------------------------------------------------------------------------

module Camila.Error (CamilaError',reason,CamilaError,runError) where

import Control.Monad.Error
import Data.Set

-----------------------------------------------------------------------------

-- | This is the type of our Camila error. 
data CamilaError' = Err { reason::String } 

-- | We make it an instance of the Error class
instance Error CamilaError' where
  noMsg    = Err "Default Error"
  strMsg s = Err s

-- | We use Either CamilaError' which represents failure using Left CamilaError'
-- or a successful result of type a using Right a.
type CamilaError = Either CamilaError'

instance Show CamilaError' where
   showsPrec _ e = showString $ "Error: " ++ reason e

-----------------------------------------------------------------------------

-- | Class RunError defines function runError
-- which returns the boolean value from the
-- type argument.
class RunError a where
   runError :: a -> Bool
   runError _ = True

instance RunError Bool where
   runError = id

instance RunError (CamilaError Bool) where
   runError (Right True) = True
   runError (Right False) = False
   runError (Left _) = False -- Should we define this case?

-----------------------------------------------------------------------------
