{-# OPTIONS -fglasgow-exts #-}
-----------------------------------------------------------------------------
-- |
-- Stability   :  experimental
-- Portability :  experimental
--
-- The Folder example. Uses a state with algebraic stacks.
--
-----------------------------------------------------------------------------

module Camila.Examples.Folder where

import Camila.Prelude hiding (filter)
import Camila.InterpState
import Data.Dynamic
import Camila.Examples.StackAlg
import Control.Monad.State

-----------------------------------------------------------------------------
-- * Types

-- Instance Variables
data FolderIVars = FolderIVars { 
  leftPages :: Stack, 
  rightPages :: Stack 
 } deriving Show
instance CData FolderIVars where
  inv = (uncurry (&&)) . (inv >< inv) . (leftPages /\ rightPages)

-- | Initial state of the folder object. 
initialFolder :: FolderIVars
initialFolder = FolderIVars { 
  leftPages = Stack [], 
  rightPages = Stack $ filter odd [1..4] }

initialFolder' :: (CamilaMonad m, MonadState FolderIVars m) => m ()
initialFolder' = do
  let lft = Stack []
  let rgt = Stack $ filter odd [1..4]
  pre (inv lft)
  pre (inv rgt)
  putInv $ FolderIVars { 
    leftPages = lft, 
    rightPages = rgt
   }

-- We have to define FolderIVars as instance of AltData.Typeable to
-- allow the interpreter to make dynamic values
instance Typeable FolderIVars where
    typeOf _ = mkTyConApp (mkTyCon "FolderIVars") []

-----------------------------------------------------------------------------
-- * Operations

-- | Flip one page forward. 
--   Pre-condition: There should be at least one page on the right stack.
--   When we reach the end of the book, we can not read another page.
opread :: (CamilaMonad m, MonadState FolderIVars m) =>  m ()
opread = do 
  preState ((/=[]) . theStack . rightPages)
  lpages <- gets leftPages
  rpages <- gets rightPages
  page <- top rpages
  rpages2 <- pop rpages
  lpages2 <- push page lpages
  modifyInv (\s -> s { leftPages = lpages2, rightPages = rpages2 })


-- | Flip back one page.
--   Precondition: There should be at least one page on the left stack.
opback :: (CamilaMonad m, MonadState FolderIVars m) =>  m ()
opback = do 
  preState ((/=[]) . theStack . leftPages)
  lpages <- gets leftPages
  rpages <- gets rightPages
  page <- top lpages
  lpages2 <- pop lpages
  rpages2 <- push page rpages
  modifyInv (\s -> s { leftPages = lpages2, rightPages = rpages2 })

-- | Retrieve the current page.
--   Pre-condition: There should be at least one page on the right stack.
oppage :: (CamilaMonad m, MonadState FolderIVars m) => m Int
oppage = do
  preState ((/=[]) . theStack . rightPages)
  rpages <- gets rightPages
  top rpages

-----------------------------------------------------------------------------
-- * Tests

folderProgram :: (MonadState FolderIVars m, CamilaMonad m, MonadIO m) => m Int
folderProgram = do
  initialFolder'
  opback
  p <- oppage
  liftIO $ putStrLn ("Seeing: " ++ show p)
  return p

testIO :: IO Int
testIO = evalStateT folderProgram $ undefined

folderProgram' :: M Int
folderProgram' = folderProgram

type M = CamilaT Warn (StateT FolderIVars IO)
type M' = StateT FolderIVars (CamilaT Warn IO)

testWarn :: IO Int
testWarn = evalStateT (runCamilaT folderProgram') undefined

testWarn' :: IO Int
testWarn' = runCamilaT $ evalStateT (folderProgram :: M' Int) undefined




-----------------------------------------------------------------------------
