{-# OPTIONS -fglasgow-exts #-}
-----------------------------------------------------------------------------
-- |
-- Stability   :  experimental
-- Portability :  experimental
--
-- The Stack Object example.
--
-----------------------------------------------------------------------------

module Camila.Examples.StackObj where

import Camila.Prelude hiding (empty)
import Camila.Examples.StackAlg
import AltData.Typeable

-----------------------------------------------------------------------------

-- * Types

-- | The Stack Object instance variables.
data StackIVars = StackIVars { stateStack :: Stack } deriving Show

-- The only constraint of the object is that the instance variable
-- respects the contraint of its type. 
instance CData StackIVars where
  inv s = inv $ stateStack s

-- | Initial state.
initialStack :: CamilaMonad m => m StackIVars
initialStack = do
  s <- initStack
  returnInv $ StackIVars { stateStack = s }

-- We have to define StackIVars as instance of AltData.Typeable,
-- to allow the interpreter to make dynamic values
instance Typeable StackIVars where
    typeOf _ = mkTyConApp (mkTyCon "StackIVars") []

-----------------------------------------------------------------------------
-- * Operations

-- | Clear the stack.
opclear :: (MonadState StackIVars m, CamilaMonad m) => m ()
opclear = do
  s <- initStack
  modifyInv (\state -> state { stateStack = s })

-- | Push a number onto the stack.
oppush :: (MonadState StackIVars m, CamilaMonad m) => Int -> m ()
oppush a = do 
  pre (odd a)
  s <- gets stateStack
  s' <- push a s
  modifyInv (\state -> state { stateStack = s' })

-- | Pop a number from the stack.
oppop :: (MonadState StackIVars m, CamilaMonad m) => m Int
oppop = do 
  s <- gets stateStack
  n <- top s
  s' <- pop s
  modifyInv (\state -> state { stateStack = s' })
  post (odd n)
  return n

optop :: (MonadState StackIVars m, CamilaMonad m) => m Int
optop = do
  s <- gets stateStack
  pre $ not $ empty s
  n <- top s
  post (odd n)
  return n

-----------------------------------------------------------------------------
