-----------------------------------------------------------------------------

module Camila.Experiment where

-- Attempt at:
--
--  * Simplifying pre-condition checking
--  * Supporting post-condition checking
--  * Allowing toggling of checking mode by choosing different monads
--  * Support for more specification discipine

-----------------------------------------------------------------------------

import Camila.Monad
import Camila.MonadT
import Camila.Prelude
import Control.Monad.Identity
import IO

-----------------------------------------------------------------------------
-- * More discipline

type Condition a = a -> Bool
type Pre a = Condition a
type Post a = Condition a
type Fun a b = a -> b
type Op s a b = (a,s) -> (b,s)    -- Arrow?

funDef :: (CamilaMonad m, CData b)
       => Fun a b
       -> Pre a
       -> Post b
       -> (a -> m b)
funDef f cpre cpost a = do
  pre $ cpre a
  b <- returnInv $ f a
  post $ cpost b
  return b

opDef :: (CamilaMonad m, CData b, CData s, MonadState s m)
       => Op s a b
       -> Pre (a,s)
       -> Post (b,s)
       -> (a -> m b)
opDef op cpre cpost a = do
  s <- get
  pre $ cpre (a,s)
  (b,s') <- returnInv $ op (a,s)
  post $ cpost (b,s')
  put s'
  return b

funDefM :: (CamilaMonad m, CData b)
       => (a -> m b)
       -> Pre a
       -> Post b
       -> (a -> m b)
funDefM f cpre cpost a = do
  pre $ cpre a
  b <- f a
  returnInv b
  post $ cpost b
  return b


opDefM :: (CamilaMonad m, CData b, CData s, MonadState s m)
       => ((a,s) -> m (b,s))
       -> Pre (a,s)
       -> Post (b,s)
       -> (a -> m b)
opDefM op cpre cpost a = do
  s <- get
  pre $ cpre (a,s)
  (b,s') <- op (a,s)
  returnInv b
  returnInv s'
  post $ cpost (b,s')
  put s'
  return b

-----------------------------------------------------------------------------
