-----------------------------------------------------------------------------
-- |
-- Module      :  Camila.InterpState
-- Copyright   :  (c) Alexandra Silva
-- License     :  GPL
--
-- Maintainer  :  Alexandra Silva
-- Stability   :  experimental
-- Portability :  experimental
--
-- Module that defines Interpreter State structure. It uses State Monad.
-- 
-----------------------------------------------------------------------------

module Camila.InterpState
		(
         CamilaState(CamilaState), CamilaElement(CamilaElement),
		 camilaInitial,
		 initialSt,
         HLookup,
		 hLookup, 
         store, 
         Store, add
) where

import System.IO.Unsafe
import Data.Typeable

-----------------------------------------------------------------------------


-- | The type of the Camila interpreter state. It holds a map from
--   class references to classes.
data CamilaState = CamilaState { 
				 store :: Store
         		       } 

type Store = [CamilaElement]
data CamilaElement = forall b.(Typeable b,Show b) => CamilaElement (String,b) -- instance of Show is just for test purposes

initialSt :: Store
initialSt = []





{- AddToStore -- type sensible -- Failed attempt

class AddToStore l b where
    add :: l -> b -> l

instance AddToStore Store CamilaElement where
	add [] c = [c]
	add  (a@(CamilaElement (y,b)):rs) e@(CamilaElement (x,v)) | x==y = add' x b v rs
                                                                  | otherwise = a : (add rs e)

class (Show b, Show v)=>AddToStore' v b l where
	add' :: String -> b -> v -> l -> l

instance (Show b, Show v)=>AddToStore' v b Store where
	add' x b v rs = (CamilaElement (x,b) : add rs (CamilaElement (x,v)))

instance Show v => AddToStore' v v Store where
	add' x b v rs = (CamilaElement (x,v) : rs)
-}

class AddToStore l b where
    add :: l -> b -> l

instance AddToStore Store CamilaElement where
        add [] c = [c]
        add  (a@(CamilaElement (y,b)):rs) e@(CamilaElement (x,v)) | (typeOf b == typeOf v) && (x==y) = (CamilaElement (x,v) : rs)
								  | (x==y) = (CamilaElement (x,b) : add rs (CamilaElement (x,v)))
                                                                  | otherwise = a : (add rs e)

class HLookup l b where
  hLookup :: String -> l -> Maybe b

{-
-- Lookup generic for HList

instance HLookup HNil b where
   hLookup id HNil = Nothing

instance HLookup l b  => HLookup (HCons (String,b) l) b where
   hLookup x (HCons (a,b) l) | (x == a) = Just b
                             | otherwise = hLookup x l

instance HLookup l b  => HLookup (HCons (String,b') l) b where
   hLookup x (HCons (a,b) l) = hLookup x l
-}

-- | Definition of Lookup for Store
instance Typeable b => HLookup Store b where
	hLookup _ [] = Nothing
	hLookup x (a@(CamilaElement (y,b)):rs) | x==y = case (cast b) of
								Nothing -> hLookup x rs
								Just k -> Just k
	                                       | otherwise = hLookup x rs

-- | The initial state of the interpreter.
camilaInitial :: CamilaState
camilaInitial = CamilaState { store = initialSt}


-----------------------------------------------------------------------------

instance Show (CamilaState) where
	showsPrec _ s = let st = store s
                        in shows $ st

instance Show CamilaElement where
        showsPrec _ (CamilaElement (a,b)) = shows (a,b)


{-----------------------------------------------------------------------------}

