
-----------------------------------------------------------------------------
-- |
-- Module      :  Interpreter
-- Copyright   :  (c) Alexandra Silva 
-- License     :  GPL
--
-- Maintainer  :  Alexandra Silva
-- Stability   :  experimental
-- Portability :  experimental
--
-- 
-----------------------------------------------------------------------------


module Camila.Interpreter 
(
state,
cset,
ccreate,
apply,
look,
runCamilaT',
(.=),
(.>)
)
where

import Camila.MonadT
import Camila.InterpState
import Data.IORef
import Data.Dynamic
import System.IO
import System.IO.Unsafe
import HListPrelude hiding (apply)

-- | State of the interpreter

state :: IORef (CamilaState)
state = unsafePerformIO $ newIORef $ initial

-- | Initial State of the Interpreter

initial :: CamilaState
initial = camilaInitial


-- | Mode of evaluation for runCamila

mode :: IORef (CamilaT mo m a -> CamilaT mo m a)
mode = unsafePerformIO $ newIORef $ id



-- | runCamila from Camila.MonadT running with the defined mode
runCamilaT' :: CamilaT mode m a -> m a 
runCamilaT' =  runCamilaT.(unsafePerformIO $ readIORef mode)
		

-- | Setting of the mode for runCamilaT

cset m = 
     case m of 
	"freeFall" -> writeIORef mode freeFall
	"warn" -> writeIORef mode warn
	"fail" -> writeIORef mode fatal
--	"errorMode" -> writeIORef mode errorMode 
	_ -> putStr "mode not defined (the possible modes are: freeFall, warn, fatal and errorMode)"

-- | Checking if a variable of type b with a certain label is already defined and what value it has
-- | It is a generic lookup for the Interpreter store

(.>) :: (Typeable b) => String -> Maybe b
(.>)=look

look id = (hLookup id (store $ unsafePerformIO $ readIORef state)) 

-- | Function Application to a defined variable 

(.$) :: (Typeable b) => (b -> a) -> String -> Maybe a
(.$) = apply
apply f id = do x <- look id
	        return (f x)

-- | Definition of a variable value
 
(.=) :: (Typeable b, Show b) => String -> b -> IO ()
(.=) = ccreate


ccreate id h = modifyIORef state (\s -> (s { store = add (store s) (CamilaElement (id,h)) }))
--	where 
--          add c [] = [c]
---          add (CamilaElement (a,b)) ((CamilaElement (id,h)) : rs) | a == id =  ((CamilaElement (id,b)) : rs)
--                                                                  | otherwise = let l = add (CamilaElement (a,b)) rs
 --                                                                               in  ((CamilaElement (id,h)) : l)




