-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Unit tests for Camila.Monad.
--
-----------------------------------------------------------------------------

module Camila.MonadHUnit where

import HUnit
import Debug.HUnitExtras

import Control.Monad.Identity
import Camila.Monad
import Pointless.Combinators (One)
import Camila.DTInvariants

-----------------------------------------------------------------------------

main = runTestsTT [
   labeledTest "Maybe"  $ do
     assertEqual "pre" Nothing $ pre False
     assertEqual "pre" (Just ()) $ pre True
     assertEqual "post" Nothing $ post False
     assertEqual "post" (Just ()) $ post True
     assertEqual "return" Nothing $ returnInv $ (undefined::Undefined)
     assertEqual "return" (Just ()) $ returnInv ()
  ,
   labeledTest "Either"  $ do
     assertEqual "pre" (Left PreConditionViolation) $ pre False
     assertEqual "pre" (Right () :: Either CamilaViolation ()) $ pre True
     assertEqual "post" (Left PostConditionViolation) $ post False
     assertEqual "post" (Right () :: Either CamilaViolation ()) $ (post True)
     assertEqual "return" (Left InvariantViolation) $ 
                          returnInv $ (undefined::Undefined)
     assertEqual "return" (Right () :: Either CamilaViolation ()) $ 
                          returnInv ()
  ,
   labeledTest "Identity" $ do
     assertEqual "pre" () $ runIdentity $ pre False
     assertEqual "pre" () $ runIdentity $ pre True
     assertEqual "post" () $ runIdentity $ post False
     assertEqual "post" () $ runIdentity $ post True
     assertSuccess "undetected violation" $ 
         return $ runIdentity $ returnInv $ (undefined::Undefined)
     assertEqual "return" () $ runIdentity $ returnInv ()
 ]

---------------------------------------------------------------------------
