-----------------------------------------------------------------------------
-- |
-- Module      :  Camila.PartialFunc
-- Copyright   :  (c) Alexandra Mendes, João Ferreira
-- License     :  GPL
--
-- Maintainer  :  João Ferreira, Alexandra Mendes
-- Stability   :  experimental
-- Portability :  experimental
--
-- Partial functions (pre-conditions are seen as local invariants)
-- 
-----------------------------------------------------------------------------


module Camila.PartialFunc (Partial,pre,pre') where

import Camila.Error
import Control.Monad.Error

-----------------------------------------------------------------------------

class Partial a where
    pre :: a -> Bool
    pre a = True
    pre' :: a -> CamilaError a
    pre' a = if (pre a) then return a else fail "Pre-condition violated!"

instance (Partial a, Partial b) => Partial (a,b) where
    pre(a,b) = (pre a) && (pre b)

instance (Partial a, Partial b) => Partial (Either a b) where
    pre (Left a) = pre a
    pre (Right b) = pre b

-----------------------------------------------------------------------------
