-----------------------------------------------------------------------------
-- |
-- Module      :  Camila.Prelude
-- Copyright   :  (c) Alexandra Mendes, Joo Ferreira
-- License     :  GPL
--
-- Maintainer  :  Joo Ferreira, Alexandra Mendes
-- Stability   :  experimental
-- Portability :  experimental
--
-- CAMILA Prelude: base library that supports the new Haskell implementation
-- of CAMILA.
--
-----------------------------------------------------------------------------

module Camila.Prelude (
-- * FiniteMaps' basic functions export by Data.FiniteMapExtras
--   domFM,             -- | VDM: dom m
--   rngFM,             -- | VDM: rng m
--   munion,            -- | VDM: munion m1 m2
--   (+++), override,   -- | VDM: m1 ++ m2
--   merge,             -- | VDM: merge ms
--   (<:),              -- | VDM: s <: m 
--   (<-:),             -- | VDM: s <-: m 
--   (>:),              -- | VDM: m :> s
--   (>-:),             -- | VDM: m :-> s
--   compFM,            -- | VDM: m1 comp m2
--   (***),             -- | VDM: m ** n 
--   inverse,           -- | VDM: inverse m 
--   inverse2,          -- | VDM: inverse m 
--   m,                 -- | VDM: m(d) 
--   injective,
--   mkr,fmToSet,
--   setOfKeysFM,
--   setOfEltsFM,
--   readFile_FM,
--   interact_FM,

-- * Sets' basic functions exported by Data.SetExtras
--   filterSet,
--   dunion,
--   readFile_Set,
--   interact_Set,

-- * Modules that can be used in Camila.Prelude

-- Datatypes modules
module Data.FiniteMapExtras,
module Data.Set,
module Data.SetExtras,

-- Camila modules
module Camila.Monad,
module Camila.MonadT,
module Camila.DTInvariants,
--module Camila.PartialFunc,
module Camila.Error,

module Camila.Interpreter,
module Camila.InterpState,
-- State Monad
module Control.Monad.State,

-- Error Monad
module Control.Monad.Error,

-- Pointless modules
module Pointless.Combinators,
--module Pointless.Combinators.Uncurried hiding (comp),
module Pointless.Functors,
module Pointless.Isomorphisms,
module Pointless.RecursionPatterns

) where


-- Datatypes modules
import Data.FiniteMapExtras
import Data.Set hiding (split)
import Data.SetExtras

-- Camila modules
import Camila.Monad
import Camila.MonadT
import Camila.Interpreter
import Camila.InterpState
import Camila.DTInvariants
--import Camila.PartialFunc
import Camila.Error
import Control.Monad.Error(
   Error(..),
   MonadError(..),
   ErrorT(..),
   mapErrorT)

-- State Monad
import Control.Monad.State hiding (fix)

-- Pointless modules
import Pointless.Combinators
import Pointless.Combinators.Uncurried()
import Pointless.Functors
import Pointless.Isomorphisms
import Pointless.RecursionPatterns

-----------------------------------------------------------------------------

