-----------------------------------------------------------------------------
-- |
-- Module      :  Data.MapExtrasQuickCheck
-- Copyright   :  (c) Joo Ferreira, Alexandra Mendes
-- License     :  GPL
--
-- Maintainer  :  Joo Ferreira, Alexandra Mendes
-- Stability   :  experimental
-- Portability :  experimental
--
-- Properties of Data.MapExtras implementation
-- (based on <http://www.di.uminho.pt/~jno/ps/mfp.ps.gz>, page 389)
-----------------------------------------------------------------------------


module Data.FiniteMapExtrasQuickCheck where

import Camila.Prelude
import qualified Data.Set as Set
import qualified Data.Map as Map
import Data.Set(Set)
import Data.Map(Map)
import Data.Relation.SetOfPairs as SP
import Test.QuickCheck
import Maybe

-----------------------------------------------------------------------------


instance (Ord a, Ord b, Arbitrary a, Arbitrary b) => Arbitrary (Map a b) where
  arbitrary = do xs <- arbitrary
                 return (Map.fromList xs)


instance (Ord a, Arbitrary a) => Arbitrary (Set a) where
  arbitrary = do xs <- arbitrary
                 return (Set.fromList xs)


prop_MUnion f g = (munion f g) == (munion g f)
 where types = (f::Map Int Int, g::Map Int Int)

prop_Override f g = 
   Set.null ( intersection (Map.keysSet f) (Map.keysSet g)) ==>
	override f g == override g f
 where types = (f::Map Int Int, g::Map Int Int)

prop_Munion1 f = (munionRel f f) == mkr f
 where types = f::Map Int Int

prop_Override1 f = f +++ f == f
 where types = f::Map Int Int

prop_A35 f g h = 
	override f (override g h) == override (override f g) h
 where types = (f::Map Int Int, g::Map Int Int, h::Map Int Int)

prop_A36 f = 
	Just f == munion f Map.empty
 where types = f::Map Int Int

prop_A37 f = 
	Just f == munion Map.empty f
 where types = f::Map Int Int

prop_A38 f = 	
	f == override f Map.empty
 where types = f::Map Int Int

prop_A39 f = 	
	f == override Map.empty f
 where types = f::Map Int Int

prop_A40 f g =
	SP.dom (munionRel f g) == Set.union (Map.keysSet f) (Map.keysSet g)
 where types = (f::Map Int Int,g::Map Int Int)

prop_A41 f g =
	Map.keysSet (override f g) == union (Map.keysSet f) (Map.keysSet g)
 where types = (f::Map Int Int, g::Map Int Int)

prop_A43 f g =
	SP.rng (munionRel f g) == union (Set.fromList $ Map.elems f) (Set.fromList $ Map.elems g)
 where types = (f::Map Int Int, g::Map Int Int)

prop_A45 f g s =
	s <-: (override f g) == override (s <-: f) (s <-: g)
 where types = (f::Map Int Int, g::Map Int Int, s::Set Int)

prop_A46 f s r =
	r <-: (s <-: f)== (union s r) <-: f
 where types = (f::Map Int Int, r::Set Int, s::Set Int)

prop_48 f s r =
	mkr ((union s r) <: f) == munionRel (s <: f) (r <: f)
 where types = (f::Map Int Int, r::Set Int, s::Set Int)

prop_49 f s r = 
	s <: (r <: f) == r <: (s <: f)
 where types = (f::Map Int Int, r::Set Int, s::Set Int)

prop_50 f r =
	r <-: (r <: f) == empty <: f
 where types = (f::Map Int Int, r::Set Int)

prop_51 f s =
	(Map.fromList.(Set.toList)) (munionRel (s <-: f) (s <: f)) == f
 where types = (f::Map Int Int, s::Set Int)

prop_52 f s r =
	s <: (r <-: f) == (difference s r) <: f
 where types = (f::Map Int Int, r::Set Int, s::Set Int)

prop_53 f s =
	Set.null (intersection s (Map.keysSet f)) ==> 
		s <-: f == f
 where types = (f::Map Int Int, s::Set Int)

prop_54 f s =
	Map.keysSet (s <-: f) == difference (Map.keysSet f) s
 where types = (f::Map Int Int, s::Set Int)

prop_55 f s =
	Map.keysSet (s <: f) == intersection (Map.keysSet f) s
 where types = (f::Map Int Int, s::Set Int)

prop_57 f g =
	mkr (override f g) == munionRel ((Map.keysSet g) <-: f) g
 where types = (f::Map Int Int, g::Map Int Int)

prop_58 f =
	(Map.keysSet f) <: f == f
 where types = (f::Map Int Int)

prop_59 f g s =
	s <: (override f g) == override (s <: f) (s <: g)
 where types = (f::Map Int Int, g::Map Int Int, s::Set Int)

prop_60 f s =
	Just f == munion (s <-: f) (s <: f)
 where types = (f::Map Int Int,s::Set Int)

prop_61 f =
	(Map.keysSet f) <-: f == Map.empty
 where types = (f::Map Int Int)

prop_62 f s r = 
	mkr ((intersection s r) <-: f) == munionRel (s <-: f) (r <-: f)
 where types = (f::Map Int Int, s::Set Int, r::Set Int)
