{-# OPTIONS -fglasgow-exts -fallow-overlapping-instances -fallow-undecidable-instances #-}
-----------------------------------------------------------------------------
-- Maintainer  :  Alexandra and Joost Visser
--
-- Example from the paper 
--
-----------------------------------------------------------------------------

module Data.HDB.Example where

import Data.HListExtras
import Data.HDB.Representation
import Data.HDB.Advanced
import Data.HDB.Read
import Data.HDB.Write
import Data.HDB.SQL
import Data.HDB.RelationalAlgebra
import Data.HDB.FunctionalDependencies
import qualified Data.Map as Map
import Prelude hiding (null)
import Text.XML.HaXml.Xml2Haskell hiding (Default)
import System
import Control.Monad

-- * Example 

-- | Setup of attributes and table identifiers

data People ; people = undefined :: People -- identifier for table people
instance Show (People) where show _ = "People"
data Cities ; cities = undefined :: Cities -- identifier for table cities
instance Show (Cities) where show _ = "Cities"

data ID;    atID    = attribute :: Attribute Int (People,ID)
instance Show (Attribute Int (a,ID)) where show _ = "ID"
data NAME;  atName  = attribute :: Attribute String (People,NAME)
instance Show (Attribute String (a,NAME)) where show _ = "NAME"
data CITY;  atCity = (Default "Amsterdam") :: DefaultableAttribute String (People, CITY)
data CITY2;
instance Show (DefaultableAttribute String (a,CITY)) where show _ = "CITY"
instance ShowLabel (DefaultableAttribute String (a,CITY)) where showLabel _ = "CITY"
instance Show (Attribute String (a,CITY2)) where show _ = "CITY2"
instance ShowLabel (Attribute String (a,CITY2)) where showLabel _ = "CITY2"
data COUNTRY;  atCountry  = (Default "Afganistan") :: DefaultableAttribute String (People, COUNTRY)
--data COUNTRY;  atCountry  = attribute :: Attribute String (People, COUNTRY)
instance Show (DefaultableAttribute String (a,COUNTRY)) where show _ = "COUNTRY"
data AGE;  atAge = nullableattribute :: NullableAttribute Int (People, AGE)
instance Show (Attribute Int (a,AGE)) where show _ = "AGE"

atCity'     = attribute :: Attribute String (Cities, CITY)
atCity''     = attribute :: Attribute String (Cities, CITY2)
atCity'''     = attribute :: Attribute String (People', CITY2)
atAge'     = attribute :: Attribute Int (People', AGE)
atName'     = attribute :: Attribute String (People', NAME)
atID''     = attribute :: Attribute Int (People', ID)
atCountry'  :: DefaultableAttribute String (Cities, COUNTRY)
atCountry'  = (Default "Afghanistan") 
atID'       = attribute :: Attribute Int (Cities,ID)

-- | setup of headers 

null = Nothing


myHeader = ( atID.*.HNil , atName.*.atAge.*.atCity.*.HNil )
myHeader2 = ( atCity' .*.HNil, atCountry' .*.HNil )
myHeader3 = ( atID'.*.HNil, atCountry' .*.HNil )
myHeader4 = ( atID.*. HNil,atCountry' .*.HNil )
myHeader5 = ( atCity'.*. HNil,atCity'' .*. atCountry' .*.HNil )
myHeader6 = ( atID''.*.HNil , atName'.*.atAge'.*.atCity'''.*.HNil )
myheader = (atID .*. HNil, atName.*.atAge.*.atCity'.*.atCountry'.*.HNil)
-- | setup of tables

table = Table myheader Map.empty
peopleT = Table myHeader (
  Map.insert ( 1.*.HNil )( "Ralf".*. null .*."Seattle".*.HNil ) $
  Map.insert ( 6.*.HNil )( "Oleg".*.(Just 17).*."Seattle".*.HNil ) $
  Map.insert ( 5.*.HNil )( "Dorothy".*.(Just 42).*."Oz".*.HNil ) $
  Map.empty )

citiesT = Table myHeader2 (
  Map.insert  ( "Seattle".*.HNil ) ( "USA".*.HNil ) $
  Map.insert ( "Braga".*.HNil ) ( "Portugal".*.HNil ) $
  Map.insert ( "Oz".*.HNil ) ( "Oz".*.HNil ) $
  Map.empty )

citiesT2 = Table myHeader3 (
  Map.insert (1.*. HNil) ( "USA".*.HNil ) $
  Map.insert (2.*. HNil) ( "Portugal".*.HNil ) $
  Map.empty )

citiesT3 = Table myHeader4 Map.empty
-- with self ref
citiesT4 = Table myHeader5 (
  Map.insert ("Seattle".*. HNil) ( "Seattle" .*. "USA".*.HNil ) $
  Map.insert ("Braga".*. HNil) ( "Braga" .*. "Portugal".*.HNil ) $
  Map.empty )


peopleT4 = Table myHeader6 Map.empty 

-- | setup of databases (well and not well-formed)
 
myrdb = Record $
  cities .=. (citiesT,Record HNil) .*. 
  people .=. (peopleT,peopleFKS) .*. HNil
        
peopleFKS = Record $ (atCity .*. HNil) .=. cities .=. (atCity'.*.HNil) .*. HNil

-- not well formed: table citiesT2 does not have attribute city as key
mybadrdb = (cities .=. citiesT2 .=. HNil) .*. (people .=. peopleT .=. peopleFKS) .*. emptyRecord

-- not well formed: table citiesT3 has repeated attribute atID from table peopleT 
mybadrdb2 = (cities .=. citiesT3 .=. HNil) .*. (people .=. peopleT .=. peopleFKS) .*. emptyRecord
 

twfds1 = createTable' myHeader2 ((uncurry FD myHeader2) .*. HNil)
twfds2 = createTable' myHeader ((uncurry FD myHeader) .*. HNil)
myOn () = (\r -> ((atCity' .=. (r.!.atCity)) .*. HNil))

-- | Data Manipulation operations

-- The where clause

isOzSenior :: (
  HasField (NullableAttribute Int (People,AGE)) r (Maybe Int),
  HasField (DefaultableAttribute String (People,CITY)) r String
 ) => r -> Bool
isOzSenior = \r -> (maybe False (>65) (r.!.atAge)) && (r.!.atCity)== "Oz"

-- The delete statement

{-
DELETE FROM table_name
  [ WHERE expression ]

Delete
from People
where isOzSenior

*Data.HDB.AdvancedHUnit Data.HDB.Example> (delete peopleT isOzSenior)
Table (HCons ID HNil,HCons NAME (HCons AGE (HCons CITY HNil))) {HCons 123 HNil:=HCons "Ralf" (HCons 23 (HCons "Seattle" HNil)),HCons 678 HNil:=HCons "Oleg" (HCons 17 (HCons "Seattle" HNil))}
-}

-- Update statement

{-
UPDATE table_name
  SET col_name1 = expression1, col_name2 = expression2, ....
  [ WHERE expression ]

Update peopleT
    set AGE = "20"
    where CITY = "Oz"


*Data.HDB.AdvancedHUnit Data.HDB.Example> update peopleT (Record $ (atAge .=. (20::Int)) .*. HNil)  (\r -> (r.!.atCity)==(Just "Oz"))
Table (HCons ID HNil,HCons NAME (HCons AGE (HCons CITY HNil))) {HCons 123 HNil:=HCons "Ralf" (HCons 23 (HCons (Just "Seattle") HNil)),HCons 504 HNil:=HCons "Dorothy" (HCons 20 (HCons (Just "Oz") HNil)),HCons 678 HNil:=HCons "Oleg" (HCons 17 (HCons (Just "Seattle") HNil))}
-}

-- Insert statement 

{-
INSERT INTO table_name [ ( col_name1, col_name2, .... ) ]
  VALUES ( expression1_1, expression1_2, .... ),
         ( expression2_1, expression2_2, .... ), .... 


-- first example: all values specified : attributes of same type have to be specified in right order

Insert into peopleT
        Values ("Joost" .*. (Just "Braga") .*. (1::Int) .*. (30::Int).*.HNil)
*Data.HDB.NormalFormsHUnit Data.HDB.Example> insertValues peopleT HNil ("Joost".*.(Just "Braga").*.(1::Int).*.(30::Int).*.HNil)
Table (HCons ID HNil,HCons NAME (HCons AGE (HCons CITY HNil))) {HCons 1 HNil:=HCons "Joost" (HCons 30 (HCons (Just "Braga") HNil)),HCons 123 HNil:=HCons "Ralf" (HCons 23 (HCons (Just "Seattle") HNil)),HCons 504 HNil:=HCons "Dorothy" (HCons 92 (HCons (Just "Oz") HNil)),HCons 678 HNil:=HCons "Oleg" (HCons 17 (HCons (Just "Seattle") HNil))}

-- second example: all values specified : attributes of same type reordered base on collumn specification

Insert into peopleT (atCity, atName, atAge, atId)
        Values ((Just "Braga") .*. "Joost" .*. (30::Int) .*. (1::Int).*.HNil)

*Data.HDB.Example> insertValues peopleT (atCity.*.atName.*.atAge.*.atID.*.HNil) ((Just "Braga").*."Joost".*.(30::Int).*.(1::Int).*.HNil)
Table (HCons ID HNil,HCons NAME (HCons AGE (HCons CITY HNil))) {HCons 1 HNil:=HCons "Joost" (HCons 30 (HCons (Just "Braga") HNil)),HCons 123 HNil:=HCons "Ralf" (HCons 23 (HCons (Just "Seattle") HNil)),HCons 504 HNil:=HCons "Dorothy" (HCons 92 (HCons (Just "Oz") HNil)),HCons 678 HNil:=HCons "Oleg" (HCons 17 (HCons (Just "Seattle") HNil))}


-- third example: nullable and default in action

Insert into peopleT
        Values ((1::Int) .*.HNil)
*Data.HDB.Example> insertValues peopleT  HNil ((1::Int) .*.HNil)
Table (HCons ID HNil,HCons NAME (HCons AGE (HCons CITY HNil))) {HCons 1 HNil:=HCons "" (HCons 0 (HCons Nothing HNil)),HCons 123 HNil:=HCons "Ralf" (HCons 23 (HCons (Just "Seattle") HNil)),HCons 504 HNil:=HCons "Dorothy" (HCons 92 (HCons (Just "Oz") HNil)),HCons 678 HNil:=HCons "Oleg" (HCons 17 (HCons (Just "Seattle") HNil))}

INSERT INTO table_name [ ( col_name1, col_name2, .... ) ]
  SELECT ... 
...
-}


-- * Database transformation


myrdb' = Record $
  cities .=. (citiesT, Record HNil) .*. 
  people .=. (peopleT2,peopleFKS2) .*. HNil

myrdb'' = Record $
  cities .=. (citiesT,Record HNil) .*. 
  people .=. (peopleT3,peopleFKS2) .*. HNil

-- rdb with self references 
myrdb''' = Record $
  cities .=. (citiesT4,citiesFK) .*. 
  people .=. (peopleT2,peopleFKS2) .*. 
  people' .=. (peopleT4,peopleFKS3) .*. HNil

myrdb1 = Record $
    newT .=. (table, Record HNil) .*. HNil

peopleFKS2 = Record $ (atCity2 .*. HNil) .=. cities .=. (atCity'.*.HNil) .*. HNil
peopleFKS3 = Record $ (atCity''' .*. HNil) .=. cities .=. (atCity'.*.HNil) .*. HNil
citiesFK = Record $ (atCity'' .*. HNil) .=. cities .=. (atCity'.*.HNil) .*. HNil

data Newt
newT = undefined :: Newt
instance ShowLabel Newt where
    showLabel _ = "newT"
instance Show Newt where
    show _ = "newT"
data People'
people' = undefined :: People'
instance ShowLabel People' where
    showLabel _ = "people'"
instance Show People' where
    show _ = "people'"

instance ShowLabel People where
    showLabel _ = "people"
instance ShowLabel Cities where
    showLabel _ = "cities"
instance ShowLabel (Attribute a (b,CITY)) where
    showLabel _ = "city"
instance ShowLabel HNil where
    showLabel _ = "HNil"
instance (ShowLabel e, ShowLabel l) => ShowLabel (HCons e l) where
    showLabel (HCons e l) = (showLabel e) ++ ".*." ++ (showLabel l)

peopleT2 = Table (atID .*. HNil, atName .*. atAge' .*. atCity2 .*.HNil)(
           Map.insert ( 1.*.HNil )( "Ralf".*.19.*."Seattle".*.HNil ) $
           Map.insert ( 6.*.HNil )( "Oleg".*.17.*."Seattle".*.HNil ) $
           Map.insert ( 5.*.HNil )( "Dorothy".*.42.*."Oz".*.HNil ) $
           Map.empty
           )

atCity2 = undefined :: Attribute String (People, CITY)
atCountry2 = undefined :: Attribute String (People, COUNTRY)

peopleT3 = Table (atID .*. atID' .*. HNil, atCity2 .*. atName .*. HNil)
           (Map.insert ((1::Int).*.(2::Int).*.HNil) ( "Braga".*."Xana".*.HNil) Map.empty)



testLineUP1 = do
  let h = (atCity2.*.HNil,atCountry2.*.HNil)
  let vs = "Braga" .*. "Portugal" .*. HNil
  putStrLn $ show $ split vs h 
                

testLineUP2 = do
  let h = (atCity2.*.HNil,atCountry.*.HNil)
  let vs = "Braga" .*. "Portugal" .*. HNil
  putStrLn $ show $ split vs h 

testComposeByName = do
    let (rdb',fds) =  composeByName myrdb' (atCity2 .*. HNil) people cities newT
    putStrLn $ show $ rdb'
    putStrLn $ show $ fds

isOzJunior :: (
  HasField (NullableAttribute Int (People,AGE)) r (Maybe Int),
  HasField (DefaultableAttribute String (People,CITY)) r String
 ) => r -> Bool
isOzJunior = \r -> (maybe False (<65) (r.!.atAge)) && (r.!.atCity)== "Oz"

testNested = do
    let t = insertResultList 
               (atCity'.*.HNil) 
               (select''' True (atCity.*.HNil) (peopleT.*.HNil) isOzJunior HNil) 
               citiesT
    unless ((=="Afghanistan")  (hHead $ (tuples t) Map.! ("Oz".*.HNil))) (putStrLn "Fail")

testInnerJoin = do
    let t = (peopleT `sqlInnerJoin` citiesT) (\r -> ((atCity' .=. (r.!.atCity)).*.HNil))
    putStrLn $ show t


--testInnerJoin' = do
  --  let t = ((addFDs peopleT) `join'` (addFDs citiesT)) (\r -> ((atCity' .=. (r.!.atCity)).*.HNil))
    --putStrLn $ show t
             

data AVG; atAVG = undefined :: Attribute t n -> Attribute Float (AVG,n)
data COUNT; atCOUNT = undefined :: n -> Attribute Int (COUNT,n)

myAFs :: (
        Real v,
        HasField (Attribute Int (People', AGE)) r v
    ) => (HCons (Attribute Float (AVG, (People', AGE)), AF r v Float) (HCons (Attribute Int (COUNT, ()), AF r () Int) HNil))
myAFs =  (atAVG atAge')  .=. AF avg (\r -> r .!. atAge') .*.
         (atCOUNT ())   .=. AF length (const ()) .*.  HNil

------------------------------------------------------------------
-- * Example from report


ids = cities .*. people .*. HNil
myFK = (atCity2 .*. HNil, (cities, atCity'.*.HNil))
fks =  emptyRecord .*. (Record $ myFK .*. HNil) .*. HNil
fds = HNil .*. HNil .*. HNil

myH = (atID .*. HNil,atName .*. atAge .*. atCity2 .*. HNil)
yourH = (atCity' .*.HNil, atCountry .*. HNil)

h = yourH .*. myH .*. HNil

-- do DB db <- readRDB "teste.gnumeric" h fds ids fks; putStrLn $ show db
