{-# OPTIONS -fglasgow-exts -fallow-overlapping-instances -fallow-undecidable-instances #-}
-----------------------------------------------------------------------------
-- |
-- Maintainer  :  Alexandra Silva, Joost Visser
-- Stability   :  experimental
-- Portability :  portable
--
-- Operations for migration of databases.
--
-----------------------------------------------------------------------------

module Data.HDB.Migration where

-- import Data.HDB.Advanced
import Data.HDB.FunctionalDependencies
import Data.HDB.SQL
import Data.HDB.Representation

import Data.Map as Map
import Data.HListExtras

-- * On the level of individual tables.

-- | Add a column.
addColumn :: (
  HeaderFor (a,b) k v,
  HeaderFor (a, HCons c b) k (HCons d v)
 ) => c -> (k -> v -> d) -> Table (a,b) k v
   -> Table (a, HCons c b) k (HCons d v)
addColumn c f (Table (a,b) mp)
  = Table (a,HCons c b) (Map.mapWithKey (\k v -> HCons (f k v) v) mp)

-- | Remove a column.
rmColumn :: (
  HeaderFor (a,b) k v,
  HFind c b n,
  HDeleteAtHNat n b b',
  HDeleteAtHNat n v v',
  HeaderFor (a,b') k v'
 ) => c -> Table (a,b) k v -> Table (a,b') k v'
rmColumn c (Table (a,b) mp)
 = Table (a,b') mp'
   where
     n = hFind c b
     b' = hDeleteAtHNat n b
     mp' = Map.map (hDeleteAtHNat n) mp


