{-# OPTIONS -fallow-overlapping-instances #-}
{-# OPTIONS -fallow-undecidable-instances #-}
-----------------------------------------------------------------------------
-- |
-- Maintainer  :  xana@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Checking properties.
--
-----------------------------------------------------------------------------

module Data.HDB.RelationalAlgebraQuickCheck where


import Data.HDB.RelationalAlgebra
import Data.HDB.Representation
import Data.HListExtras
import GhcSyntax 
import HListPrelude 
import FakePrelude 
import qualified Data.Map as Map
import Data.Map (Map)
import qualified Data.Set as Set
import Test.QuickCheck

-----------------------------------------------------------------------------

instance Arbitrary Char where
    arbitrary = do (x::Int) <- arbitrary
                   let (y::Int) = x `mod` (fromEnum (maxBound::Char))
                   return ((toEnum y)::Char)

instance Arbitrary (Attribute v n) where
    arbitrary = do return (attribute :: Attribute v n)

instance Arbitrary HZero where
    arbitrary = return hZero

instance (HNat n, Arbitrary n) => Arbitrary (HSucc n) where
    arbitrary = do n <- arbitrary
                   return (hSucc n)

instance Arbitrary HNil where
    arbitrary = return HNil

instance (Arbitrary e, Arbitrary l) => Arbitrary (HCons e l) where
    arbitrary = do e <- arbitrary
                   l <- arbitrary
                   return (HCons e l)

instance (HeaderFor h ks vs, Arbitrary h, Arbitrary ks, Arbitrary vs)
      => Arbitrary (Table h ks vs) where
  arbitrary = do h <- arbitrary
                 ks <- arbitrary
                 vs <- arbitrary
                 return (Table h (Map.fromList (zip ks vs)))


-- Maier pag 15
-- X_1 \subseteq X_2 \subseteq X_3 ... => pi_(X_1) (pi_(X_2) .... (r))= pi_(X_1) r
prop_project1 a1 a2 a3 t 
  = classify (Map.size (tuples t) == 0) "empty" $
    (not (booleanValue $ containedEq a1 a2) || not (booleanValue $ containedEq a1 a3)) ||
    ((project a1 t) == (project a1 $ resultSet2table $ project a2 (resultSet2table $ project a3 t)))
    where 
        types = (t :: Table ((Attribute Int Zero) :*: HNil, (Attribute Int One) :*: (Attribute Bool Two) :*: (Attribute String Three) :*: HNil) (Int :*: HNil) (Int :*: Bool :*: String :*: HNil), a1 :: (Attribute Int Zero) :*: HNil, a2 :: (Attribute Int Zero) :*: (Attribute Int One) :*: (Attribute Bool Two) :*: HNil, a3:: (Attribute Int Zero) :*: (Attribute Int One) :*: (Attribute Bool Two) :*: (Attribute String Three) :*: HNil)
     


{-
prop_Union_self t 
  = classify (Map.size (tuples t) == 0) "empty" $
    ((DB.union t t) == t)
    where 
        types =[t:: Table (HCons (Attribute Bool HZero HZero) (HCons (Attribute Bool (HSucc HZero) HZero) (HCons (Attribute Bool (HSucc (HSucc HZero)) HZero) HNil)), HNil) (HCons Bool (HCons Bool (HCons Bool HNil))) HNil]

-- | DB.union of two tables t1 and t2, where t1 contains t2 equals t2
prop_Union t t' 
  = let ts =  tuples t
        ts'=  tuples t'
        isEmpty = (Map.size ts == 0) || (Map.size ts' == 0)
        t1 = Map.keysSet ts
        t2 = Map.keysSet ts'
    in
    classify isEmpty "empty" $
    ((not (t2 `Set.isSubsetOf` t1)) || ((DB.union t t') == t))
    where 
      types =[t :: Table (
            HCons (Attribute Bool HZero HZero)
                  (HCons (Attribute Bool (HSucc HZero) HZero)
                         (HCons (Attribute Bool (HSucc (HSucc HZero)) HZero) HNil)), HNil)
           (HCons Bool (HCons Bool (HCons Bool HNil))) HNil, t']

prop_intersection_self t 
  = classify (Map.size ( (tuples t)) == 0) "empty" $
    ((DB.intersectionT t t) == t)
    where 
      types = [t :: Table (
            HCons (Attribute Bool HZero HZero)
                  (HCons (Attribute Bool (HSucc HZero) HZero)
                         (HCons (Attribute Bool (HSucc (HSucc HZero)) HZero) HNil)), HNil)
           (HCons Bool (HCons Bool (HCons Bool HNil))) HNil ]

-- | intersection of two tables t1 and t2, where t1 contains t2 equals t2
prop_intersection t t' 
  = let ts =  tuples t
        ts'=  tuples t'
        isEmpty = Map.size ts == 0 ||Map.size ts' == 0
        t1 = Map.keysSet ts
        t2 = Map.keysSet ts'
    in
    classify isEmpty "empty" $
    ((not (t1 `Set.isSubsetOf` t2)) || ((DB.intersectionT t t') == t))
    where 
      types =[t :: Table (
            HCons (Attribute Bool HZero HZero)
                  (HCons (Attribute Bool (HSucc HZero) HZero)
                         (HCons (Attribute Bool (HSucc (HSucc HZero)) HZero) HNil)), HNil)
           (HCons Bool (HCons Bool (HCons Bool HNil))) HNil , t']

-- | intersection commutes with DB.union

prop_union t t' t''
  = let isEmpty = Map.size ( tuples t) == 0 ||Map.size ( tuples t') == 0 ||Map.size ( tuples t'') == 0
    in
    classify isEmpty "empty" $
    (isEmpty) || DB.union t (DB.intersectionT t' t'') == (DB.union t t') `DB.intersectionT` (DB.union t t'')
    where 
      types =[t :: Table (
            HCons (Attribute Bool HZero HZero)
                  (HCons (Attribute Bool (HSucc HZero) HZero)
                         (HCons (Attribute Bool (HSucc (HSucc HZero)) HZero) HNil)), HNil)
           (HCons Bool (HCons Bool (HCons Bool HNil))) HNil, t', t'']

-- | DB.union commutes with intersection

prop_unionintersect t t' t''
  = let isEmpty = (Map.size ( tuples t) == 0) || (Map.size ( tuples t') == 0) 
                    || (Map.size ( tuples t'') == 0)
    in
    classify isEmpty "empty" $
    DB.intersectionT t (DB.union t' t'') == (DB.intersectionT t t') `DB.union` (DB.intersectionT t t'')
    where 
      types =[t :: Table (
            HCons (Attribute Bool HZero HZero)
                  (HCons (Attribute Bool (HSucc HZero) HZero)
                         (HCons (Attribute Bool (HSucc (HSucc HZero)) HZero) HNil)), HNil)
           (HCons Bool (HCons Bool (HCons Bool HNil))) HNil, t', t'']


prop_filter t = classify (Map.size ((tuples t)) == 0) "empty" $
    ((DB.filter p t) == t)
    where 
      types = [t :: Table (
            HCons (Attribute Bool HZero HZero)
                  (HCons (Attribute Bool (HSucc HZero) HZero)
                         (HCons (Attribute Bool (HSucc (HSucc HZero)) HZero) HNil)), HNil)
           (HCons Bool (HCons Bool (HCons Bool HNil))) HNil] 
      p::(HCons Bool (HCons Bool (HCons Bool HNil)), HNil)->Bool 
      p = (const True)

prop_project t = classify (Map.size ((tuples t)) == 0) "empty" $
    ((DB.project (schema t) t) == t)
    where 
      types = [t :: Table (
            HCons (Attribute Bool HZero HZero)
                  (HCons (Attribute Bool (HSucc HZero) HZero)
                         (HCons (Attribute Bool (HSucc (HSucc HZero)) HZero) HNil)), HNil)
           (HCons Bool (HCons Bool (HCons Bool HNil))) HNil] 

-- | Properties for tables with information in the keys
prop_Union_self' t 
  = classify (Map.size (tuples t) == 0) "empty" $
    ((DB.union t t) == t)
    where 
        types =[t :: Table (HCons (Attribute Int HZero HZero) HNil,
            HCons (Attribute String HZero HZero)
                  (HCons (Attribute String (HSucc HZero) HZero)
                         (HCons (Attribute String (HSucc (HSucc HZero)) HZero) HNil)))
           (HCons Int HNil)
           (HCons [Char] (HCons [Char] (HCons [Char] HNil)))]

-- | DB.union of two tables t1 and t2, where t1 contains t2 equals t2
prop_Union' t t' 
  = let ts =  tuples t
        ts'=  tuples t'
        isEmpty = (Map.size ts == 0) || (Map.size ts' == 0)
        t1 = Map.keysSet ts
        t2 = Map.keysSet ts'
    in
    classify isEmpty "empty" $
    ((not (t2 `Set.isSubsetOf` t1)) || ((DB.union t t') == t))
    where 
      types =[t :: Table (HCons (Attribute Int HZero HZero) HNil,
            HCons (Attribute String HZero HZero)
                  (HCons (Attribute String (HSucc HZero) HZero)
                         (HCons (Attribute String (HSucc (HSucc HZero)) HZero) HNil)))
           (HCons Int HNil)
           (HCons [Char] (HCons [Char] (HCons [Char] HNil))), t']

prop_intersection_self' t 
  = classify (Map.size ( (tuples t)) == 0) "empty" $
    ((DB.intersectionT t t) == t)
    where 
      types = [ t :: Table (HCons (Attribute Int HZero HZero) HNil,
            HCons (Attribute String HZero HZero)
                  (HCons (Attribute String (HSucc HZero) HZero)
                         (HCons (Attribute String (HSucc (HSucc HZero)) HZero) HNil)))
           (HCons Int HNil)
           (HCons [Char] (HCons [Char] (HCons [Char] HNil)))]

-- | intersection of two tables t1 and t2, where t1 contains t2 equals t2
prop_intersection' t t' 
  = let ts =  tuples t
        ts'=  tuples t'
        isEmpty = Map.size ts == 0 ||Map.size ts' == 0
        t1 = Map.keysSet ts
        t2 = Map.keysSet ts'
    in
    classify isEmpty "empty" $
    ((not (t1 `Set.isSubsetOf` t2)) || ((DB.intersectionT t t') == t))
    where 
      types =[t :: Table (HCons (Attribute Int HZero HZero) HNil,
            HCons (Attribute String HZero HZero)
                  (HCons (Attribute String (HSucc HZero) HZero)
                         (HCons (Attribute String (HSucc (HSucc HZero)) HZero) HNil)))
           (HCons Int HNil)
           (HCons [Char] (HCons [Char] (HCons [Char] HNil))), t']

-- | intersection commutes with DB.union

prop_intersectunion' t t' t''
  = let isEmpty = Map.size ( tuples t) == 0 ||Map.size ( tuples t') == 0 ||Map.size ( tuples t'') == 0
    in
    classify isEmpty "empty" $
    (isEmpty) || DB.union t (DB.intersectionT t' t'') == (DB.union t t') `DB.intersectionT` (DB.union t t'')
    where 
      types =[t :: Table (HCons (Attribute Int HZero HZero) HNil,
            HCons (Attribute String HZero HZero)
                  (HCons (Attribute String (HSucc HZero) HZero)
                         (HCons (Attribute String (HSucc (HSucc HZero)) HZero) HNil)))
           (HCons Int HNil)
           (HCons [Char] (HCons [Char] (HCons [Char] HNil))), t', t'']

-- | DB.union commutes with intersection

prop_unionintersect' t t' t''
  = let isEmpty = (Map.size ( tuples t) == 0) || (Map.size ( tuples t') == 0) 
                    || (Map.size ( tuples t'') == 0)
    in
    classify isEmpty "empty" $
    DB.intersectionT t (DB.union t' t'') == (DB.intersectionT t t') `DB.union` (DB.intersectionT t t'')
    where 
      types =[t :: Table (HCons (Attribute Int HZero HZero) HNil,
            HCons (Attribute String HZero HZero)
                  (HCons (Attribute String (HSucc HZero) HZero)
                         (HCons (Attribute String (HSucc (HSucc HZero)) HZero) HNil)))
           (HCons Int HNil)
           (HCons [Char] (HCons [Char] (HCons [Char] HNil))), t', t'']


prop_filter' t = classify (Map.size ((tuples t)) == 0) "empty" $
    ((DB.filter p t) == t)
    where 
      types = [t :: Table (HCons (Attribute Int HZero HZero) HNil,
            HCons (Attribute String HZero HZero)
                  (HCons (Attribute String (HSucc HZero) HZero)
                         (HCons (Attribute String (HSucc (HSucc HZero)) HZero) HNil)))
           (HCons Int HNil)
           (HCons [Char] (HCons [Char] (HCons [Char] HNil)))]
      p::(HCons Int HNil,HCons [Char] (HCons [Char] (HCons [Char] HNil))) -> Bool
      p = (const True)

prop_project' t = classify (Map.size ((tuples t)) == 0) "empty" $
    ((DB.project' (schema t) t) == t)
    where 
      types = [t :: Table (HCons (Attribute Int HZero HZero) HNil,
            HCons (Attribute String HZero HZero)
                  (HCons (Attribute String (HSucc HZero) HZero)
                         (HCons (Attribute String (HSucc (HSucc HZero)) HZero) HNil)))
           (HCons Int HNil)
           (HCons [Char] (HCons [Char] (HCons [Char] HNil)))]

prop_join t t' 
  = let ts =  tuples t
        ts'=  tuples t'
        isEmpty = Map.size ts == 0 ||Map.size ts' == 0
        t1 = Map.keysSet ts
        t2 = Map.keysSet ts'
        at1 = (hHead (fst $ schema t)) :: (Attribute Int HZero HZero)
        at2 = (hHead (fst $ schema t')) :: (Attribute Int HZero HZero)
    in
    classify isEmpty "empty" $
    ((DB.joinT at1 at2 (==) t t') == (DB.joinT' at1 at2 (==) t t'))
    where 
      types =[t :: Table (
            HCons (Attribute Int HZero HZero)
                  (HCons (Attribute Bool (HSucc HZero) HZero)
                         (HCons (Attribute Bool (HSucc (HSucc HZero)) HZero) HNil)), HNil)
           (HCons Int (HCons Bool (HCons Bool HNil))) HNil , t']
-}
