-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Universidade do Minho, 2005
-- License     :  LGPL
-- 
-- Maintainer  :  Joo Saraiva - jas@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Representing Haskell-based Context-free Grammars in GraphViz
--
-----------------------------------------------------------------------------

module Language.ContextFree.Cfg2GraphViz 
       ( cfg2dotGraph 
       ) where

------------------------------------------------------------------------------

import List
import Language.ContextFree.Cfg

cfg2dotGraph :: (Eq t, Eq nt, Show t, Show nt) => Cfg t nt -> String	
cfg2dotGraph gr
  = let pr     = map ((\x -> (head x, tail x)).snd) (prods gr)
        header = "digraph G {" ++ "\n" ++ "rankdir=LR;\n"
        footer = "}"
        aux (h,t) 
          = let edge  = nodeName1 ++ " -> " ++ nodeName2 ++ ";\n"
                color = if (h == root gr) 
                         then ",fontcolor=green" 
                         else "" 
                node1 = nodeName1 ++ " [label =" ++ nodeName1 ++ ",shape=box" ++ color ++"];\n"
                node2 = nodeName2 ++ " [label =" ++ nodeName2 ++ ",shape=box,fontcolor=white,color=black,style=filled];\n"
                nodeName1 = quote $ sy2NodeName h
                nodeName2 = quote $ rhs2NodeName t
			    
                edges = concatMap rhs_nt t
                rhs_nt (T _) = ""
                rhs_nt s     = nodeName2 ++ " -> " ++ quote (sy2NodeName s)
            in edge ++ node1 ++ node2 ++ edges
    in header ++ (concat.reverse) (map aux pr) ++ footer  

sy2NodeName (T sy) = show sy
sy2NodeName sy     = (tail.init.showSymb) sy
rhs2NodeName rhs   = "::=  "++ (concat (intersperse ("  ") (map sy2NodeName rhs)))++" "
			    


quote s = "\""++s++"\""	

cfg2dotList :: (Eq t, Eq nt, Show t, Show nt) => Cfg t nt -> String	
cfg2dotList gr
  = let pr    = map ((\x -> (head x, tail x)).snd) (prods gr)
        named = zip pr ["node" ++ show i | i <- [1..]]
        header = "digraph G {" ++ "\n" ++ "rankdir=LR;\n"
        footer = "}"
        aux ((h,t),n) = let edge  = n ++ "h -> " ++ n ++ "t;\n"
                            color = if (h == root gr) 
                                     then ",fontcolor=green" 
                                     else "" 
                            node1 = n ++ "h [label =\"" ++ (tail.init.showSymb) h ++ "\",shape=plaintext" ++ color ++"];\n"
                            node2 = n ++ "t [label =\"" ++ concat (intersperse ("   ") (map faux t)) ++"\",shape=plaintext];\n"
                            faux (T sy) = show sy
                            faux sy     = (tail.init.showSymb) sy
                        in edge ++ node1 ++ node2
	in header ++ (concat.reverse) (map aux named) ++ footer  
	      
