
\section{Incremental generalised Acceptance Function in Haskell}

This module uses GHC library module Memo and the function memo that
memoises a function call (it uses a hash table to implement the memo
table).

\begin{code}
module Language.ContextFree.MemoInc_G_LR ( inc_glraccept
      		    , inc_glrLazy
      		    , inc_glrStrict
		    , glr_inc
		    , memo
                    ) where


import Char
import List
import Language.HaLex.Dfa
import Language.ContextFree.Stack
import Language.ContextFree.Cfg
import Language.ContextFree.LR_0
import Language.ContextFree.SLR
import Language.ContextFree.G_SLR


-- import LALR


--import IOExts
--import Text.XML.HaXml.Types
--import Text.XML.HaXml.Pretty (element)
import Memo (memo)



\end{code}

\begin{code}


inc_glraccept g inp =  glr_inc lookupTT_g lookupAT_g [s] inp 
  where eg@(Cfg t nt r p)   = expand_cfg g      -- The expanded grammar
        dfa@(Dfa v q s z d) = ecfg2LR_0Dfa eg   -- The LR_0 DFA

        at                  = e_gslr_at  eg      -- The Action Table

        lookupAT_g          = glookupAT t at     -- The lookup fun. for the AT
        lookupTT_g          = d                 -- The lookup fun. for the TT



glr_inc l_tt l_at s [] = [ res
                             | ac <- acs
                             , res <- glr' l_tt l_at s [] ac
                             ]
   where a = top s 
         acs       = l_at a Dollar

glr_inc l_tt l_at s inp@(h:t) = [ res
                            | ac <- acs
                            , res <- memo (glr' l_tt l_at) s inp ac  
                            ]
  where  a = top s 
         acs = l_at a (T h)




glr' l_tt l_at sk inp (Reduce pr)  = glr_inc l_tt l_at sk'' inp
  where sk'  = popN (sizeProd pr) sk        -- reducing the stack   
        st   = top sk'                      -- acessing the state on its top 
        st'  = l_tt st (lhs_prod pr)        -- moving to a new state
        sk'' = push st' sk'                 -- pushing the new state


glr' l_tt l_at sk (h:t) (Shift st) = glr_inc l_tt l_at sk' t
  where sk'  = push st sk
	  
glr' l_tt l_at sk inp Accept = [True]

glr' l_tt l_at sk inp Error  = [False]


inc_glr g inp = inc_glraccept g inp
inc_glrLazy g inp = or $ inc_glr g inp
inc_glrStrict g inp = length $ inc_glr g inp


\end{code}

