-----------------------------------------------------------------------------
-- |
-- Maintainer  :  nfr@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Haskell Modules Graph Constructor
--
-----------------------------------------------------------------------------

module Language.Haskell.ModuleGraph where

import Language.Haskell.Parser
import Language.Haskell.Syntax

import Data.Relation.SetOfPairs 
import Data.Set  hiding (map)

-----------------------------------------------------------------------------
-- * Haskell Parsing 

mkHaModulesGraph :: [FilePath] -> IO (Rel String String)
mkHaModulesGraph files = do 
  g <- foldr ioUnion (return emptySet) (map mkHaModuleGraph files)
  putStrLn (show g)
  return g

mkHaModuleGraph :: FilePath -> IO (Rel String String)
mkHaModuleGraph f = do 
  texto <- readFile f
  let parseRes = parseModule texto
  case parseRes of
   (ParseOk hsmodule)  -> putStrLn "Graph Build ok" >> 
                          return (buildImportsGraph (getImports hsmodule))
   (ParseFailed src s) -> putStrLn ("Parsing Error in " ++ 
                                    srcFilename src ++ 
                                    " at line " ++ 
                                    show (srcLine src) ++ 
                                    " at column " ++ 
                                    show (srcColumn src) ++ ")") >> 
                          return emptySet

-----------------------------------------------------------------------------
-- * Graph Construction 

-- | Build module graph for a list of modules.
buildModuleGraph :: [HsModule] -> (Rel String String)
buildModuleGraph ms
  = foldr (\m r -> r `union` (buildModuleGraph1 m)) emptyRel ms

-- | Build module graph for a single module.
buildModuleGraph1 :: HsModule -> Rel String String
buildModuleGraph1 m
  = buildImportsGraph (getImports m)

buildImportsGraph :: (Module, [HsImportDecl]) -> (Rel String String)
buildImportsGraph (_, []) = emptySet
buildImportsGraph ((Module m1), ((HsImportDecl _ (Module m2) _ _ _):is)) = 
   union (unitSet (m1, m2)) (buildImportsGraph ((Module m1), is))

getImports (HsModule _ mod _ imps _ ) = (mod, imps)

ioUnion :: (Ord a) => IO (Rel a a) -> IO (Rel a a) -> IO (Rel a a)
ioUnion g1 g2 = do g3 <- g1
                   g4 <- g2
                   return (union g3 g4)
                   
                   
-----------------------------------------------------------------------------
