-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- This module holds unit tests that test for behaviour of the parser
-- on code fragments that use language extensions.
--
-----------------------------------------------------------------------------

module Language.Haskell.ParserHUnit where

import Language.Haskell.Parser
import HUnit
import Debug.HUnitExtras

main = putStrLn "Language.Haskell.ParserHUnit disabled!"

main' = do
  runTestsTT
    [ testSuite "LittleLambda Test Suite"
      [ TestCase $ do
          let result = parseModule "module M where f (a::Int) = a"
          assertEqual "pattern" Nothing (getMsg result)
      , TestCase $ do
          let result = parseModule "module M where class X a b | a -> b"
          assertEqual "functional dependency" Nothing (getMsg result)
      , TestCase $ do
          let result = parseModule "module M where data T = forall x . [x]"
          assertEqual "forall in data" Nothing (getMsg result)
      , TestCase $ do
          let result = parseModule "module M where _f = undefined"
          assertEqual "forall in data" Nothing (getMsg result)
      ]
    ]

isOk (ParseOk _) = True
isOk _           = False

getMsg (ParseFailed srcLoc msg) = Just msg
getMsg _ = Nothing

-----------------------------------------------------------------------------
