-----------------------------------------------------------------------------
-- |
-- Maintainer  :  Patrick Machado
-- Stability   :  experimental
-- Portability :  portable
--
-- Unit tests for Language.Java.CallGraph
--
-----------------------------------------------------------------------------

module Language.Java.CallGraphHUnit where

import HUnit
import Debug.HUnitExtras

import Language.Java.Syntax
import Language.Java.Declarations

-----------------------------------------------------------------------------

-- | Main for running tests.
main :: IO Counts
main = runTestsTT [
  labeledTest "Empty declarations"  $ do
    let str = "public class X { }"
    (ast::CompilationUnit) <- parseString str "CompilationUnit" javaTable
    ds <- collectMembers "" ast
    assertEqual "Should be empty" emptyDec ds,
  labeledTest "Field declaration"  $ do
    let str = "public class X { Integer x; }"
    (ast::CompilationUnit) <- parseString str "CompilationUnit" javaTable
    ds <- collectMembers "" ast
    let expected = addGlobalDec emptyDec ("X",Field "x") "Integer"
    assertEqual "Should have 1" expected ds
 ]

-- | Relative location of the Java parse table.
javaTable :: String
javaTable = "Language/Java/Java.tbl"

-----------------------------------------------------------------------------
