-----------------------------------------------------------------------------
-- |
-- Maintainer  :  Patrick Machado
-- Stability   :  experimental
-- Portability :  portable
--
-- Unit tests for Language.Java.CallGraphDeclarations
--
-----------------------------------------------------------------------------

module Language.Java.DeclarationsHUnit where

import HUnit
import Debug.HUnitExtras

import Language.Java.Syntax
import Language.Java.Util (javaTable)
import Language.Java.Declarations

-----------------------------------------------------------------------------

main :: IO Counts
main = runTestsTT [
  labeledTest "Empty declarations"  $ do
    let str = "public class X { }"
    (ast::CompilationUnit) <- parseString str "CompilationUnit" javaTable
    ds <- collectMembers "" ast
    assertEqual "Should be empty" emptyDec ds,
  labeledTest "Field declaration"  $ do
    let str = "public class X { Integer x; }"
    (ast::CompilationUnit) <- parseString str "CompilationUnit" javaTable
    ds <- collectMembers "" ast
    let expected = addGlobalDec emptyDec ("X",Field "x") "Integer"
    assertEqual "Should have 1" expected ds,
  labeledTest "Method declaration"  $ do
    let str = "public class X { int main(int a, float b){}}"
    (ast::CompilationUnit) <- parseString str "CompilationUnit" javaTable
    ds <- collectMembers "" ast
    let expected = addGlobalDec emptyDec ("X",Method "main" ["int", "float"]) "int"
    assertEqual "Should have 1 method" expected ds
 ]

-----------------------------------------------------------------------------
