-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Universidade do Minho, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  patrickmachado@gmail.com
-- Stability   :  experimental
-- Portability :  portable
--
--
-----------------------------------------------------------------------------

module Language.Java.TypeInference where

-------------------------------------------------------------------------------

import Language.Java.Syntax
import Language.Java.Declarations
import Language.Java.Util

import StrategyLib

import Control.Monad
-------------------------------------------------------------------------------

-- | Find out the types of a parameters in an invocation
findTypeOfArgs :: (Term a, MonadPlus m) => Declarations -> ClassName -> [a] -> [m TypeName]
findTypeOfArgs decs p = map (findTypeOf decs p)

-- | Generic function to infere the type of a term.
-- It dows it's best effort. Sometimes, wrong results may be expected
-- because many cases are not considered.
-- May be extended, adding more patterns to new cases
findTypeOf :: (Term a, MonadPlus m) => Declarations -> ClassName -> a -> m TypeName
findTypeOf decs tn = applyTU (once_tdTU worker)
	where
		worker = failTU 
--			`adhocTU` arrayaccess 
			`adhocTU` newcalls 
			`adhocTU` methods 
			`adhocTU` variables 
			`adhocTU` literals 
			`adhocTU` fields

		--arrayaccess (Name_Expression name index)
		--	= let
		--		(a,b) = sepname name
		--	in findType (a++[b]) tn decs
		--arrayaccess _ = mzero
		-- determine the type of new calls
		-- Ex: [new Foo()].foo()
		newcalls (New_comma (ClassOrInterfaceType1 (Name n)) _)
			= return (name2str n)
		newcalls _ = mzero
		
		-- determine the type of method calls 
		--  Ex: [foo().bar()].aaa()
		methods (PrimaryMethodInvocation prim ident args)
			= do
				tp <- findTypeOf decs tn prim
				largs <- sequence $ findTypeOfArgs decs tn args
				getGlobalType decs tp (Method ident largs)
		-- Ex: [var1].foo() *** [var1.var2].foo()
		methods (NameMethodInvocation ident args)
			= do 
				let (a,s) = sepname ident
				t <- findType a tn decs
				largs <- sequence $ findTypeOfArgs decs tn args
				getType decs t (Method s largs)
		methods _ = mzero

		variables (Primary_Identifier prim ident)
			= do
				tp <- findTypeOf decs tn prim
				getType decs tp (Field ident)
		variables _ = mzero

		fields (Name1 n) 
			= getType decs tn (Field (name2str n))
		fields _ = mzero

		-- determine the type of literals
		literals (IntegerLiteral _) = return "int"
		literals (FloatingPointLiteral _) = return "float"
		literals (BooleanLiteral _) = return "boolean"
		literals (StringLiteral _) = return "String"
		literals (CharacterLiteral _) = return "char"
		literals _ = mzero

-- Find type of a chained field access
findType :: (MonadPlus m) =>  [String] -> ClassName -> Declarations -> m TypeName
findType [] t _ = return t
findType lvars t decs = foldl func mzero lvars 
	where
		func m var = do
			xx <- m `mplus` (return t)
			getType decs xx (Field var)
