-------------------------------------------------------------------------------
--
-- | 
-- Module      :  Language.PLSql.Syntax
-- Copyright   :  (c) Tiago Alves, Paulo Silva, 2005
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- This module defines the abstract syntax for PL-Sql, which was generated
-- from an SDF specification using Sdf2Haskell.
--
-------------------------------------------------------------------------------

module Language.PLSql.Syntax where
 
data Alias = Alias (Maybe Kw_as) Name_or_keyword
 
data Aliased_select_item = Aliased_select_item Select_item
					       (Maybe Alias)
 
data Aliased_select_items = Aliased_select_items [Aliased_select_item]
 
data Aliased_select_items_or_all = Aliased_select_items_or_all_c1 Aliased_select_items
				 | Aliased_select_items_or_all_c2
 
data Aliased_table_description = Aliased_table_description Table_description
							   (Maybe Alias)
 
data Aliased_table_descriptions = Aliased_table_descriptions [Aliased_table_description]
 
data Allocate_extent_clause = Allocate_extent_clause Allocate_extent_items
 
data Allocate_extent_item = Allocate_extent_item_c1 Memory_size_clause
			  | Allocate_extent_item_c2 Filename
			  | Allocate_extent_item_c3 SqlInteger
 
data Allocate_extent_items = Allocate_extent_items [Allocate_extent_item]
 
data Allocate_statement = Allocate_statement Name
 
data Alter_cluster_item = Alter_cluster_item_c1 SqlInteger
			| Alter_cluster_item_c2 SqlInteger
			| Alter_cluster_item_c3 Memory_size_clause
			| Alter_cluster_item_c4 SqlInteger
			| Alter_cluster_item_c5 SqlInteger
			| Alter_cluster_item_c6 Storage_clause
			| Alter_cluster_item_c7 (Maybe Allocate_extent_clause)
			| Alter_cluster_item_c8 (Maybe Keep_clause)
 
data Alter_cluster_items = Alter_cluster_items [Alter_cluster_item]
 
data Alter_cluster_statement = Alter_cluster_statement Qualified_name
						       (Maybe Alter_cluster_items)
						       (Maybe Parallel_clause)
 
data Alter_database_item = Alter_database_item_c1 (Maybe
						     Kw_standby_database)
						  (Maybe Kw_exclusive_parallel)
			 | Alter_database_item_c2
			 | Alter_database_item_c3 (Maybe Kw_no_resetlogs)
			 | Alter_database_item_c4 Kw_standby_database
			 | Alter_database_item_c5
			 | Alter_database_item_c6
			 | Alter_database_item_c7 Recover_clause
			 | Alter_database_item_c8 (Maybe Thread_integer) Grouped_filespecs
			 | Alter_database_item_c9 Logfile_members
			 | Alter_database_item_c10 Group_or_filenames_list
			 | Alter_database_item_c11 Filenames
			 | Alter_database_item_c12 (Maybe Kw_unarchived)
						   Group_or_filenames_list
						   (Maybe Kw_unrecoverable_datafile)
			 | Alter_database_item_c13 File_rename_clause
			 | Alter_database_item_c14 Filename (Maybe Kw_reuse)
			 | Alter_database_item_c15 Backup_controlfile_clause
			 | Alter_database_item_c16 Qualified_name
			 | Alter_database_item_c17
			 | Alter_database_item_c18 Alter_database_set_item
			 | Alter_database_item_c19 (Maybe Kw_public) Thread_integer
			 | Alter_database_item_c20 Thread_integer
			 | Alter_database_item_c21 Filenames Filespecs
			 | Alter_database_item_c22 Filenames Alter_datafile_item
 
data Alter_database_items = Alter_database_items [Alter_database_item]
 
data Alter_database_set_item = Alter_database_set_item_c1 Text
			     | Alter_database_set_item_c2 Text
			     | Alter_database_set_item_c3 Kw_on_off
 
data Alter_database_statement = Alter_database_statement (Maybe
							    Name)
							 (Maybe Alter_database_items)
 
data Alter_datafile_item = Alter_datafile_item_c1
			 | Alter_datafile_item_c2 (Maybe Kw_drop)
			 | Alter_datafile_item_c3 Memory_size
			 | Alter_datafile_item_c4 Autoextend_clause
			 | Alter_datafile_item_c5
 
data Alter_function_statement = Alter_function_statement Qualified_name
 
data Alter_index_statement = Alter_index_statement Qualified_name
						   Alter_index_statement_items
 
data Alter_index_statement_item = Alter_index_statement_item_c1 SqlInteger
				| Alter_index_statement_item_c2 SqlInteger
				| Alter_index_statement_item_c3 SqlInteger
				| Alter_index_statement_item_c4 Storage_clause
				| Alter_index_statement_item_c5 (Maybe Allocate_extent_clause)
				| Alter_index_statement_item_c6 (Maybe Keep_clause)
				| Alter_index_statement_item_c7 Rebuild_items
 
data Alter_index_statement_items = Alter_index_statement_items [Alter_index_statement_item]
 
data Alter_package_statement = Alter_package_statement Qualified_name
						       (Maybe Kw_package_body)
 
data Alter_procedure_statement = Alter_procedure_statement Qualified_name
 
data Alter_profile_item = Alter_profile_item_c1 Alter_profile_item_1
						Integer_or_other
			| Alter_profile_item_c2 Memory_size_or_other
 
data Alter_profile_item_1 = Alter_profile_item_1_c1
			  | Alter_profile_item_1_c2
			  | Alter_profile_item_1_c3
			  | Alter_profile_item_1_c4
			  | Alter_profile_item_1_c5
			  | Alter_profile_item_1_c6
			  | Alter_profile_item_1_c7
			  | Alter_profile_item_1_c8
 
data Alter_profile_items = Alter_profile_items [Alter_profile_item]
 
data Alter_profile_statement = Alter_profile_statement Profile
						       (Maybe Alter_profile_items)
 
data Alter_resource_cost_statement = Alter_resource_cost_statement Resource_cost_items
 
data Alter_role_statement = Alter_role_statement Name Role_item
 
data Alter_rollback_segment_statement = Alter_rollback_segment_statement Name
									 Rollback_segment_item
 
data Alter_sequence_statement = Alter_sequence_statement Qualified_name
							 Sequence_items
 
data Alter_session_clause = Alter_session_clause_c1 Session_set_items
			  | Alter_session_clause_c2 Dblink
			  | Alter_session_clause_c3 Kw_commit_rollback_nothing
			  | Alter_session_clause_c4 Kw_enable_disable
 
data Alter_session_statement = Alter_session_statement Alter_session_clause
 
data Alter_snapshot_log_statement = Alter_snapshot_log_statement (Maybe
								    Kw_on)
								 Qualified_name
								 (Maybe Snap_shot_items)
 
data Alter_snapshot_statement = Alter_snapshot_statement Qualified_name
							 (Maybe Snap_shot_items)
							 (Maybe Using_index_clause)
							 (Maybe Snapshot_refresh)
 
data Alter_system_statement = Alter_system_statement System_items
 
data Alter_table_item = Alter_table_item Alter_table_items_1
					 (Maybe Alter_table_item_2)
 
data Alter_table_item_1 = Alter_table_item_1_c1 Table_items_opt_brackets
			| Alter_table_item_1_c2 Table_modify_items
			| Alter_table_item_1_c3 Table_modify_item
			| Alter_table_item_1_c4 SqlInteger
			| Alter_table_item_1_c5 SqlInteger
			| Alter_table_item_1_c6 SqlInteger
			| Alter_table_item_1_c7 SqlInteger
			| Alter_table_item_1_c8 Storage_clause
			| Alter_table_item_1_c9 Constraint
			| Alter_table_item_1_c10 (Maybe Allocate_extent_clause)
			| Alter_table_item_1_c11 (Maybe Keep_clause)
 
data Alter_table_item_2 = Alter_table_item_2_c1 En_dis_able_clauses
			| Alter_table_item_2_c2 Parallel_clause Kw_nocache_cache
 
data Alter_table_items_1 = Alter_table_items_1 [Alter_table_item_1]
 
data Alter_table_statement = Alter_table_statement Qualified_name
						   Alter_table_item
 
data Alter_tablespace_statement = Alter_tablespace_statement Tablespace
							     Tablespace_items
 
data Alter_trigger_statement = Alter_trigger_statement Qualified_name
						       Kw_enable_disable_compile
 
data Alter_user_statement = Alter_user_statement User User_items
 
data Alter_view_statement = Alter_view_statement Qualified_name
 
data Append_item = Append_item Name (Maybe Position)
			       (Maybe Datatype) (Maybe Kw_external) (Maybe Text)
 
data Append_items = Append_items [Append_item]
 
data Append_items_brackets = Append_items_brackets Append_items
 
data Archive_log_clause = Archive_log_clause (Maybe Thread_integer)
					     Archive_log_item
 
data Archive_log_item = Archive_log_item_c1 Archive_log_item_located
					    (Maybe To_location)
		      | Archive_log_item_c2
 
data Archive_log_item_located = Archive_log_item_located_c1 SqlInteger
			      | Archive_log_item_located_c2 SqlInteger
			      | Archive_log_item_located_c3
			      | Archive_log_item_located_c4 SqlInteger
			      | Archive_log_item_located_c5 Filename
			      | Archive_log_item_located_c6
			      | Archive_log_item_located_c7
			      | Archive_log_item_located_c8
 
data As_subquery = As_subquery Subquery_opt_brackets
 
data Assignment_statement = Assignment_statement Lhs
						 Assignment_symbol Top_expression
 
data Assignment_symbol = Assignment_symbol
 
data Atomic_boolean_expression = Atomic_boolean_expression_c1 Call
			       | Atomic_boolean_expression_c2 Expression Relational_operator
							      Expression
			       | Atomic_boolean_expression_c3 Expressions Relational_operator
							      Subquery_brackets
			       | Atomic_boolean_expression_c4 Expressions Relational_operator
							      Kw_any_some_all
							      Expressions_or_subquery_brackets
			       | Atomic_boolean_expression_c5 Expression (Maybe Kw_not)
							      Expressions_or_subquery_brackets
			       | Atomic_boolean_expression_c6 Expressions_brackets (Maybe Kw_not)
							      Expressions_or_subquery_brackets
			       | Atomic_boolean_expression_c7 Expression (Maybe Kw_not) Expression
							      Expression
			       | Atomic_boolean_expression_c8 Expression (Maybe Kw_not)
			       | Atomic_boolean_expression_c9 Subquery_brackets
			       | Atomic_boolean_expression_c10 Boolean_expression
			       | Atomic_boolean_expression_c11 Qualified_name
			       | Atomic_boolean_expression_c12 Name
			       | Atomic_boolean_expression_c13 Cursor_name Cursor_attribute_attr
			       | Atomic_boolean_expression_c14 Expression (Maybe Kw_not)
							       Expression
			       | Atomic_boolean_expression_c15 Expression (Maybe Kw_not) Text
							       [Text]
 
data Atomic_expression = Atomic_expression_c1 Qualified_name
		       | Atomic_expression_c2 Function_call
		       | Atomic_expression_c3 Kw_distinct_all Function_call
		       | Atomic_expression_c4 Text
		       | Atomic_expression_c5 Number
		       | Atomic_expression_c6 Expression
		       | Atomic_expression_c7 Cursor_name Cursor_attribute_attr
		       | Atomic_expression_c8 Qualified_name
		       | Atomic_expression_c9 SqlInteger SqlInteger
 
data Autoextend = Autoextend_c1
		| Autoextend_c2 (Maybe Next_memory_size) (Maybe Max_memory_size)
 
data Autoextend_clause = Autoextend_clause Autoextend
 
data Backup_controlfile_clause = Backup_controlfile_clause_c1 Filename
							      (Maybe Kw_reuse)
			       | Backup_controlfile_clause_c2 (Maybe Kw_no_resetlogs)
 
data Basic_loop_statement = Basic_loop_statement Pl_statements
						 (Maybe Label_name)
 
data Begin_declare_section_statement = Begin_declare_section_statement
 
data Binary_or_name = Binary_or_name_c1
		    | Binary_or_name_c2 Name
 
data Boolean_expression = Boolean_expression_c1 Atomic_boolean_expression
			| Boolean_expression_c2 Boolean_expression
			| Boolean_expression_c3 Boolean_expression Boolean_expression
			| Boolean_expression_c4 Boolean_expression Boolean_expression
 
data Break_item = Break_item [Sql_plus_identifier]
 
data Bulk_collect = Bulk_collect
 
data Call = Call_c1 Function_call
	  | Call_c2 Function_name
 
data Checkpoint = Checkpoint_c1
		| Checkpoint_c2
 
data Close_statement = Close_statement Cursor
 
data Col_item = Col_item_c1 Format
	      | Col_item_c2 Left_right
	      | Col_item_c3
	      | Col_item_c4
	      | Col_item_c5
	      | Col_item_c6 Text
	      | Col_item_c7 Sql_plus_identifier
	      | Col_item_c8 Sql_plus_identifier
 
data Col_items = Col_items [Col_item]
 
data Column_name = Column_name Lex_qualified_name
 
data Column_names = Column_names [Column_name]
 
data Columns_brackets = Columns_brackets Column_names
 
data Comment = Comment_c1 Text
	     | Comment_c2
 
data Comment_item = Comment_item_c1
		  | Comment_item_c2
 
data Comment_statement = Comment_statement Comment_item
					   Qualified_name Text
 
data Commit_statement = Commit_statement (Maybe Kw_work)
					 (Maybe Comment)
 
data Connect_by_clause = Connect_by_clause (Maybe
					      Connect_by_clause_prefix)
					   Boolean_expression (Maybe Qualified_name)
 
data Connect_by_clause_prefix = Connect_by_clause_prefix Boolean_expression
 
data Connect_to_clause = Connect_to_clause Name Password
 
data Connect_using_clause = Connect_using_clause Text
 
data Constant_declaration = Constant_declaration Constant_declaration_name
						 Declaration_init_value
 
data Constant_declaration_name = Constant_declaration_name Name
							   Return_type
 
data Constraint = Constraint_c1 Constraint_header
				(Maybe Constraint_item_1) (Maybe Constraint_item_2)
		| Constraint_c2 Constraint_item_1 (Maybe Constraint_item_2)
 
data Constraint_clause = Constraint_clause Constraint
 
data Constraint_header = Constraint_header_c1 Name
		       | Constraint_header_c2 Text
 
data Constraint_item_1 = Constraint_item_1_c1 Kw_unique_primary_key
					      (Maybe Columns_brackets)
		       | Constraint_item_1_c2 Columns_brackets Qualified_name
					      (Maybe Columns_brackets) (Maybe Kw_on_delete_cascade)
		       | Constraint_item_1_c3 Boolean_expression
		       | Constraint_item_1_c4 (Maybe Kw_not)
		       | Constraint_item_1_c5 Qualified_name (Maybe Columns_brackets)
					      (Maybe Kw_on_delete_cascade)
		       | Constraint_item_1_c6
 
data Constraint_item_2 = Constraint_item_2_c1 Index_parameters
		       | Constraint_item_2_c2
 
data Constraints = Constraints [Constraint]
 
data Create_database_link_statement = Create_database_link_statement (Maybe
									Kw_public)
								     Name (Maybe Connect_to_clause)
								     (Maybe Connect_using_clause)
 
data Create_function_statement = Create_function_statement (Maybe
							      Kw_or_replace)
							   Function_body_declaration
 
data Create_index_statement = Create_index_statement (Maybe
							Kw_unique)
						     (Maybe Kw_bitmap) Qualified_name
						     Index_on_clause (Maybe Index_parameters)
						     (Maybe Parallel_clause)
 
data Create_or_replace = Create_or_replace (Maybe Kw_or_replace)
 
data Create_procedure_statement = Create_procedure_statement (Maybe
								Kw_or_replace)
							     Procedure_body_declaration
 
data Create_role_statement = Create_role_statement Name
						   (Maybe Role_item)
 
data Create_rollback_segment_item = Create_rollback_segment_item_c1 Tablespace_clause
				  | Create_rollback_segment_item_c2 Storage_clause
				  | Create_rollback_segment_item_c3 Memory_size
				  | Create_rollback_segment_item_c4
 
data Create_rollback_segment_items = Create_rollback_segment_items [Create_rollback_segment_item]
 
data Create_rollback_segment_statement = Create_rollback_segment_statement (Maybe
									      Kw_public)
									   Name
									   (Maybe
									      Create_rollback_segment_items)
 
data Create_sequence_statement = Create_sequence_statement Qualified_name
							   [Sequence_item]
 
data Create_synonym_statement = Create_synonym_statement (Maybe
							    Kw_public)
							 Qualified_name Qualified_name
 
data Create_table_clause_1 = Create_table_clause_1_c1 Index_parameters
			   | Create_table_clause_1_c2 Name Columns_brackets
 
data Create_table_clause_2 = Create_table_clause_2 (Maybe
						      En_dis_able_clauses)
						   (Maybe As_subquery) (Maybe Kw_cache_nocache)
 
data Create_table_statement = Create_table_statement Qualified_name
						     (Maybe Table_items_opt_brackets)
						     (Maybe Create_table_clause_1)
						     (Maybe Parallel_clause) Create_table_clause_2
 
data Create_tablespace_statement = Create_tablespace_statement Tablespace
							       Datafile_descriptions
							       Tablespace_items
 
data Create_trigger_statement = Create_trigger_statement (Maybe
							    Kw_or_replace)
							 Qualified_name Kw_before_after_instead_of
							 Trigger_items Qualified_name
							 (Maybe Row_clause) Plsql_block
 
data Create_type_statement = Create_type_statement (Maybe
						      Kw_or_replace)
						   Name (Maybe Kw_is_as) Datatype
 
data Create_user_statement = Create_user_statement Name
						   (Maybe User_items)
 
data Create_view_statement = Create_view_statement (Maybe
						      Kw_or_replace)
						   (Maybe Kw_force_no_force) Qualified_name
						   (Maybe Names_brackets) Subquery_opt_brackets
						   (Maybe View_with_clause)
 
data Cursor = Cursor_c1 Cursor_name
	    | Cursor_c2 Host_name
 
data Cursor_attribute_attr = Cursor_attribute_attr_c1 Perc_found
			   | Cursor_attribute_attr_c2 Perc_isopen
			   | Cursor_attribute_attr_c3 Perc_notfound
			   | Cursor_attribute_attr_c4 Perc_rowcount
 
data Cursor_declaration = Cursor_declaration Cursor_declaration_kernel
					     Select_statement_without_into
 
data Cursor_declaration_kernel = Cursor_declaration_kernel Name
							   (Maybe Parameter_declarations_brackets)
 
data Cursor_name = Cursor_name Lex_name
 
data Cursor_ref = Cursor_ref Cursor_name
			     (Maybe Parameters_brackets)
 
data Cursor_return = Cursor_return Perc_typed_name
 
data Datafile_description = Datafile_description Filespec
						 (Maybe Autoextend_clause)
 
data Datafile_descriptions = Datafile_descriptions [Datafile_description]
 
data Datatype = Datatype_c1
	      | Datatype_c2
	      | Datatype_c3
	      | Datatype_c4 SqlInteger
	      | Datatype_c5 SqlInteger SqlInteger
	      | Datatype_c6
	      | Datatype_c7 SqlInteger
	      | Datatype_c8
	      | Datatype_c9
	      | Datatype_c10 SqlInteger
	      | Datatype_c11
	      | Datatype_c12 SqlInteger
	      | Datatype_c13
	      | Datatype_c14 SqlInteger
 
data Date = Date Expression
 
data Dblink = Dblink Lex_name
 
data Declaration = Declaration_c1 Function_header_declaration
		 | Declaration_c2 Procedure_header_declaration
		 | Declaration_c3 Constant_declaration
		 | Declaration_c4 Cursor_declaration
		 | Declaration_c5 Exception_declaration
		 | Declaration_c6 Var_declaration
		 | Declaration_c7 Pragma_declaration
		 | Declaration_c8 Table_type_declaration
		 | Declaration_c9 Record_type_definition
		 | Declaration_c10 Ref_type_declaration
		 | Declaration_c11 Function_body_declaration
		 | Declaration_c12 Procedure_body_declaration
 
data Declaration_init_value = Declaration_init_value (Maybe
							Kw_not_null)
						     Init_value
 
data Declarations = Declarations [Declaration]
 
data Declarations_semicolon = Declarations_semicolon Declarations
						     (Maybe Kw_semicolon)
 
data Declare_cursor_statement = Declare_cursor_statement Name
							 Select_statement_without_into
 
data Declare_section = Declare_section (Maybe
					  Declarations_semicolon)
 
data Default_expression = Default_expression Expression
 
data Delete_item = Delete_item_c1 Qualified_name
		 | Delete_item_c2 Qualified_name Alias
		 | Delete_item_c3 Subquery
 
data Delete_statement = Delete_statement (Maybe Kw_from)
					 Delete_item (Maybe Where_clause)
 
data Disable_clause = Disable_clause Disable_clause_item
 
data Disable_clause_item = Disable_clause_item_c1 Columns_brackets
						  (Maybe Kw_cascade)
			 | Disable_clause_item_c2 (Maybe Kw_cascade)
			 | Disable_clause_item_c3 Constraint_clause (Maybe Kw_cascade)
			 | Disable_clause_item_c4 Kw_all_triggers
			 | Disable_clause_item_c5
 
data Drop_database_link_statement = Drop_database_link_statement (Maybe
								    Kw_public)
								 Name
 
data Drop_function_statement = Drop_function_statement Name
 
data Drop_index_statement = Drop_index_statement Qualified_name
 
data Drop_procedure_statement = Drop_procedure_statement Name
 
data Drop_sequence_statement = Drop_sequence_statement Qualified_name
 
data Drop_synonym_statement = Drop_synonym_statement (Maybe
							Kw_public)
						     Qualified_name
 
data Drop_table_statement = Drop_table_statement Qualified_name
						 (Maybe Kw_cascade_constraints)
 
data Drop_trigger_statement = Drop_trigger_statement Qualified_name
 
data Drop_view_statement = Drop_view_statement Qualified_name
 
data Dummy_top_sort = Dummy_top_sort_c1 Top_sort
		    | Dummy_top_sort_c2 Lex_identifier
 
data Else_branch = Else_branch Pl_statements
 
data Elsif_branch = Elsif_branch Boolean_expression Pl_statements
 
data Emb_name = Emb_name Lex_host_identifier
 
data Embedded_sql_statement = Embedded_sql_statement_c1 Allocate_statement
			    | Embedded_sql_statement_c2 Begin_declare_section_statement
			    | Embedded_sql_statement_c3 Declare_cursor_statement
			    | Embedded_sql_statement_c4 End_declare_section_statement
			    | Embedded_sql_statement_c5 Include_statement
			    | Embedded_sql_statement_c6 For_sql_statement
			    | Embedded_sql_statement_c7 Whenever_sqlerror_statement
			    | Embedded_sql_statement_c8 Sql_statement
 
data Embedded_sql_statements = Embedded_sql_statements [Embedded_sql_statement]
 
data En_dis_able_clauses = En_dis_able_clauses_c1 Enable_clause
			 | En_dis_able_clauses_c2 Disable_clause
			 | En_dis_able_clauses_c3 En_dis_able_clauses Enable_clause
			 | En_dis_able_clauses_c4 En_dis_able_clauses Disable_clause
 
data Enable_clause = Enable_clause Enable_clause_item
 
data Enable_clause_item = Enable_clause_item_c1 Enable_clause_item_header
						(Maybe Using_index_clause) (Maybe Exceptions_into)
			| Enable_clause_item_c2 Kw_all_triggers
			| Enable_clause_item_c3
 
data Enable_clause_item_header = Enable_clause_item_header_c1 Columns_brackets
			       | Enable_clause_item_header_c2
			       | Enable_clause_item_header_c3 (Maybe Kw_cascade)
 
data End_declare_section_statement = End_declare_section_statement
 
data End_of_function = End_of_function (Maybe Name)
 
data End_of_package = End_of_package (Maybe Package_name)
 
data End_of_procedure = End_of_procedure (Maybe Name)
 
data Exception_declaration = Exception_declaration Exception_name
 
data Exception_handler = Exception_handler Exception_when_clause
					   Pl_statements
 
data Exception_handler_others = Exception_handler_others Pl_statements
 
data Exception_handlers = Exception_handlers [Exception_handler]
					     (Maybe Exception_handler_others)
 
data Exception_name = Exception_name_c1 Lex_qualified_name
		    | Exception_name_c2
 
data Exception_section = Exception_section Exception_handlers
 
data Exception_when_clause = Exception_when_clause Exception_name
						   [Or_exception_name]
 
data Exceptions_into = Exceptions_into Qualified_name
 
data Execute_parameter = Execute_parameter (Maybe Kw_in_out_nocopy)
					   Name
 
data Execute_statement = Execute_statement (Maybe Kw_immediate)
					   Expression (Maybe Execute_using)
					   (Maybe Into_qualified_name)
 
data Execute_using = Execute_using [Execute_parameter]
 
data Exit_statement = Exit_statement (Maybe Label_name)
				     (Maybe Exit_when_clause)
 
data Exit_when_clause = Exit_when_clause Boolean_expression
 
data Explain_plan_statement = Explain_plan_statement (Maybe
							Set_statement_id)
						     (Maybe Into_qualified_name) Sql_statement
 
data Expression = Expression_c1 Atomic_expression
		| Expression_c2 Expression
		| Expression_c3 Expression
		| Expression_c4 Expression Expression
		| Expression_Expression1 Expression Expression
		| Expression_Expression2 Expression Expression
		| Expression_Expression3 Expression Expression
		| Expression_c8 Expression Expression
		| Expression_c9 Expression Expression
 
data Expression_or_subquery_brackets = Expression_or_subquery_brackets_c1 Expression
				     | Expression_or_subquery_brackets_c2 Subquery_brackets
 
data Expressions = Expressions [Expression]
 
data Expressions_brackets = Expressions_brackets Expressions
 
data Expressions_or_subquery_brackets = Expressions_or_subquery_brackets_c1 Expressions_brackets
				      | Expressions_or_subquery_brackets_c2 Subquery_brackets
 
data Fetch_name = Fetch_name Qualified_name
 
data Fetch_names = Fetch_names [Fetch_name]
 
data Fetch_statement = Fetch_statement Cursor Fetch_names
 
data Field_declaration = Field_declaration Name Return_type
					   (Maybe Declaration_init_value)
 
data Field_declarations = Field_declarations [Field_declaration]
 
data File_rename_clause = File_rename_clause Kw_file_datafile
					     Filenames Filenames
 
data Filename = Filename Lex_text
 
data Filename_with_extension = Filename_with_extension Lex_sql_plus_identifier
 
data Filenames = Filenames [Filename]
 
data Filenames_opt_brackets = Filenames_opt_brackets_c1 Filename
			    | Filenames_opt_brackets_c2 Filenames
 
data Filespec = Filespec Filenames_opt_brackets
			 (Maybe Memory_size_clause) (Maybe Kw_reuse)
 
data Filespecs = Filespecs [Filespec]
 
data For_loop_statement = For_loop_statement Name
					     (Maybe Kw_reverse) For_loop_statement_range
					     Basic_loop_statement
 
data For_loop_statement_range = For_loop_statement_range_c1 Expression
			      | For_loop_statement_range_c2 Expression Expression
			      | For_loop_statement_range_c3 Subquery_opt_brackets
 
data For_sql_statement = For_sql_statement Host_name Sql_statement
 
data For_update_clause = For_update_clause (Maybe For_update_items)
					   (Maybe Kw_nowait)
 
data For_update_item = For_update_item Qualified_names
 
data For_update_items = For_update_items [For_update_item]
 
data Format = Format Lex_format
 
data From_location = From_location Filename
 
data Function_body_declaration = Function_body_declaration_c1 Function_header_declaration
							      Kw_is_as
							      (Maybe Declarations_semicolon)
							      Statement_block_no_end End_of_function
			       | Function_body_declaration_c2 Function_header_declaration Kw_is_as
							      (Maybe Declarations_semicolon)
							      Statement_block End_of_function
			       | Function_body_declaration_c3 Function_header_declaration Kw_is_as
							      Text
 
data Function_call = Function_call_c1 Function_name
				      Parameters_brackets
		   | Function_call_c2
		   | Function_call_c3 Parameters_brackets
 
data Function_header_declaration = Function_header_declaration Qualified_name
							       (Maybe
								  Parameter_declarations_brackets)
							       Return_type
 
data Function_name = Function_name Qualified_name
 
data Goto_statement = Goto_statement Label_name
 
data Grant_item = Grant_item_c1 Keyword_except_to_on_from
		| Grant_item_c2
		| Grant_item_c3 Columns_brackets
 
data Grant_statement = Grant_statement [Grant_item]
				       (Maybe On_clause) Grantees (Maybe Kw_with_admin_option)
 
data Grantee = Grantee_c1 Name
	     | Grantee_c2 Kw_public
 
data Grantees = Grantees [Grantee]
 
data Group_by_clause = Group_by_clause Expressions
				       (Maybe Having_clause)
 
data Group_integer = Group_integer SqlInteger
 
data Group_or_filenames = Group_or_filenames_c1 Group_integer
			| Group_or_filenames_c2 Filenames_opt_brackets
 
data Group_or_filenames_list = Group_or_filenames_list [Group_or_filenames]
 
data Grouped_filespec = Grouped_filespec (Maybe Group_integer)
					 Filespec
 
data Grouped_filespecs = Grouped_filespecs [Grouped_filespec]
 
data Having_clause = Having_clause Boolean_expression
 
data Host_language_statement = Host_language_statement Embedded_sql_statements
 
data Host_name = Host_name Lex_host_name
 
data If_statement = If_statement Boolean_expression Then_branch
				 [Elsif_branch] (Maybe Else_branch)
 
data Include_statement = Include_statement Name
 
data Index_on_clause = Index_on_clause_c1 Qualified_name
					  Ordered_columns
		     | Index_on_clause_c2 Qualified_name
 
data Index_parameter = Index_parameter_c1 SqlInteger
		     | Index_parameter_c2 SqlInteger
		     | Index_parameter_c3 SqlInteger
		     | Index_parameter_c4 Tablespace_clause
		     | Index_parameter_c5 Storage_clause
		     | Index_parameter_c6
		     | Index_parameter_c7
		     | Index_parameter_c8
		     | Index_parameter_c9 SqlInteger
		     | Index_parameter_c10
 
data Index_parameters = Index_parameters [Index_parameter]
 
data Init_value = Init_value Kw_assignment_default Top_expression
 
data Insert_statement = Insert_statement_c1 Kw_into
					    Aliased_table_description (Maybe Columns_brackets)
		      | Insert_statement_c2 Kw_into Aliased_table_description
					    (Maybe Columns_brackets) Values_description
 
data SqlInteger = Integer Lex_integer
 
data Integer_or_default = Integer_or_default_c1 SqlInteger
			| Integer_or_default_c2
 
data Integer_or_other = Integer_or_other_c1 SqlInteger
		      | Integer_or_other_c2
		      | Integer_or_other_c3
 
data Integer_or_unlimited = Integer_or_unlimited_c1 SqlInteger
			  | Integer_or_unlimited_c2
 
data Integer_text_var = Integer_text_var_c1 SqlInteger
		      | Integer_text_var_c2 Text
		      | Integer_text_var_c3 Qualified_name
		      | Integer_text_var_c4 Col_item
 
data Into = Into (Maybe Bulk_collect)
 
data Into_qualified_name = Into_qualified_name Into Qualified_name
 
data Into_qualified_names = Into_qualified_names Into
						 Qualified_names
 
data Isolation_level = Isolation_level_c1
		     | Isolation_level_c2
 
data Isolation_level_clause = Isolation_level_clause Isolation_level
 
data Keep_clause = Keep_clause Memory_size
 
data Keyword_except_to_on_from = Keyword_except_to_on_from Lex_keyword_except_to_on_from
 
data Kw_all_triggers = Kw_all_triggers
 
data Kw_any_some_all = Kw_any_some_all_c1
		     | Kw_any_some_all_c2
		     | Kw_any_some_all_c3
 
data Kw_append = Kw_append
 
data Kw_as = Kw_as
 
data Kw_assignment_default = Kw_assignment_default_c1 Assignment_symbol
			   | Kw_assignment_default_c2
 
data Kw_automatic = Kw_automatic
 
data Kw_before_after_instead_of = Kw_before_after_instead_of_c1
				| Kw_before_after_instead_of_c2
				| Kw_before_after_instead_of_c3
 
data Kw_bitmap = Kw_bitmap
 
data Kw_cache_nocache = Kw_cache_nocache_c1
		      | Kw_cache_nocache_c2
 
data Kw_cascade = Kw_cascade
 
data Kw_cascade_constraints = Kw_cascade_constraints
 
data Kw_commit_rollback_nothing = Kw_commit_rollback_nothing_c1
				| Kw_commit_rollback_nothing_c2
				| Kw_commit_rollback_nothing_c3
 
data Kw_default = Kw_default
 
data Kw_desc_asc = Kw_desc_asc_c1
		 | Kw_desc_asc_c2
 
data Kw_distinct_all = Kw_distinct_all_c1
		     | Kw_distinct_all_c2
 
data Kw_drop = Kw_drop
 
data Kw_enable_disable = Kw_enable_disable_c1
		       | Kw_enable_disable_c2
 
data Kw_enable_disable_compile = Kw_enable_disable_compile_c1
			       | Kw_enable_disable_compile_c2
			       | Kw_enable_disable_compile_c3
 
data Kw_exclusive_parallel = Kw_exclusive_parallel_c1
			   | Kw_exclusive_parallel_c2
 
data Kw_external = Kw_external
 
data Kw_fast_complete_force = Kw_fast_complete_force_c1
			    | Kw_fast_complete_force_c2
			    | Kw_fast_complete_force_c3
 
data Kw_file_datafile = Kw_file_datafile_c1
		      | Kw_file_datafile_c2
 
data Kw_force_no_force = Kw_force_no_force_c1
		       | Kw_force_no_force_c2
 
data Kw_from = Kw_from
 
data Kw_global_local = Kw_global_local_c1
		     | Kw_global_local_c2
 
data Kw_immediate = Kw_immediate
 
data Kw_in_out_nocopy = Kw_in_out_nocopy_c1
		      | Kw_in_out_nocopy_c2
		      | Kw_in_out_nocopy_c3
 
data Kw_into = Kw_into
 
data Kw_is_as = Kw_is_as_c1
	      | Kw_is_as_c2
 
data Kw_local = Kw_local
 
data Kw_no_resetlogs = Kw_no_resetlogs_c1
		     | Kw_no_resetlogs_c2
 
data Kw_nocache_cache = Kw_nocache_cache_c1
		      | Kw_nocache_cache_c2
 
data Kw_nologging = Kw_nologging
 
data Kw_normal_temporary_immediate = Kw_normal_temporary_immediate_c1
				   | Kw_normal_temporary_immediate_c2
				   | Kw_normal_temporary_immediate_c3
 
data Kw_not = Kw_not
 
data Kw_not_null = Kw_not_null
 
data Kw_nowait = Kw_nowait
 
data Kw_old_new = Kw_old_new_c1
		| Kw_old_new_c2
 
data Kw_on = Kw_on
 
data Kw_on_delete_cascade = Kw_on_delete_cascade
 
data Kw_on_off = Kw_on_off_c1
	       | Kw_on_off_c2
 
data Kw_or_replace = Kw_or_replace
 
data Kw_package_body = Kw_package_body_c1
		     | Kw_package_body_c2
 
data Kw_public = Kw_public
 
data Kw_reuse = Kw_reuse
 
data Kw_reverse = Kw_reverse
 
data Kw_savepoint = Kw_savepoint
 
data Kw_semicolon = Kw_semicolon
 
data Kw_sql_trace_global_names = Kw_sql_trace_global_names_c1
			       | Kw_sql_trace_global_names_c2
 
data Kw_standby_database = Kw_standby_database (Maybe ())
 
data Kw_table_cluster = Kw_table_cluster_c1
		      | Kw_table_cluster_c2
 
data Kw_timestamp_signature = Kw_timestamp_signature_c1
			    | Kw_timestamp_signature_c2
 
data Kw_to = Kw_to
 
data Kw_true_false = Kw_true_false_c1
		   | Kw_true_false_c2
 
data Kw_unarchived = Kw_unarchived
 
data Kw_unique = Kw_unique
 
data Kw_unique_primary_key = Kw_unique_primary_key_c1
			   | Kw_unique_primary_key_c2
 
data Kw_unrecoverable_datafile = Kw_unrecoverable_datafile
 
data Kw_with_admin_option = Kw_with_admin_option
 
data Kw_work = Kw_work
 
data Label_header = Label_header Label_name
 
data Label_name = Label_name Lex_name
 
data Language = Language Lex_name
 
data Left_right = Left_right_c1
		| Left_right_c2
		| Left_right_c3
		| Left_right_c4
 
data Lhs = Lhs_c1 Qualified_name
	 | Lhs_c2 Qualified_name Expression
 
data Load_data_clause = Load_data_clause [Load_data_parameter_value]
					 (Maybe Kw_append) Qualified_name (Maybe Kw_append)
					 (Maybe When_clause) (Maybe Trailing_clause)
					 (Maybe Append_items_brackets)
 
data Load_data_option = Load_data_option Sql_plus_identifier
					 SqlInteger
 
data Load_data_options = Load_data_options [Load_data_option]
 
data Load_data_parameter = Load_data_parameter_c1
			 | Load_data_parameter_c2
			 | Load_data_parameter_c3
			 | Load_data_parameter_c4
			 | Load_data_parameter_c5
 
data Load_data_parameter_value = Load_data_parameter_value Load_data_parameter
							   Load_data_value
 
data Load_data_statement = Load_data_statement [Load_data_clause]
 
data Load_data_value = Load_data_value_c1 Qualified_name
		     | Load_data_value_c2 SqlInteger
 
data Lock_mode = Lock_mode_c1
	       | Lock_mode_c2
	       | Lock_mode_c3
	       | Lock_mode_c4
	       | Lock_mode_c5
	       | Lock_mode_c6
 
data Lock_table_statement = Lock_table_statement Qualified_name
						 Lock_mode (Maybe Kw_nowait)
 
data Logfile_member = Logfile_member Reused_filenames
				     Group_or_filenames_list
 
data Logfile_members = Logfile_members [Logfile_member]
 
data Loop_statement = Loop_statement_c1 Basic_loop_statement
		    | Loop_statement_c2 While_loop_statement
		    | Loop_statement_c3 For_loop_statement
 
data Max_memory_size = Max_memory_size_c1
		     | Max_memory_size_c2 Memory_size
 
data Memory_size = Memory_size Lex_memory_size
 
data Memory_size_clause = Memory_size_clause Memory_size
 
data Memory_size_or_null = Memory_size_or_null_c1 Memory_size
			 | Memory_size_or_null_c2
 
data Memory_size_or_other = Memory_size_or_other_c1 Memory_size
			  | Memory_size_or_other_c2
			  | Memory_size_or_other_c3
 
data Memory_size_or_unlimited = Memory_size_or_unlimited_c1 Memory_size
			      | Memory_size_or_unlimited_c2
 
data Meta_parameter = Meta_parameter_c1
		    | Meta_parameter_c2
		    | Meta_parameter_c3
		    | Meta_parameter_c4
		    | Meta_parameter_c5
		    | Meta_parameter_c6
		    | Meta_parameter_c7
		    | Meta_parameter_c8
		    | Meta_parameter_c9
		    | Meta_parameter_c10
		    | Meta_parameter_c11
		    | Meta_parameter_c12
		    | Meta_parameter_c13
		    | Meta_parameter_c14
		    | Meta_parameter_c15
		    | Meta_parameter_c16
		    | Meta_parameter_c17
		    | Meta_parameter_c18
		    | Meta_parameter_c19
		    | Meta_parameter_c20
		    | Meta_parameter_c21
		    | Meta_parameter_c22
		    | Meta_parameter_c23
		    | Meta_parameter_c24
 
data Meta_parameter_value = Meta_parameter_value Meta_parameter
						 Sql_plus_value_or_text
 
data Meta_parameter_values = Meta_parameter_values [Meta_parameter_value]
 
data Meta_statement = Meta_statement_c1 Sql_plus_statement
		    | Meta_statement_c2 Sql_plus_macro
		    | Meta_statement_c3 Package_declaration
		    | Meta_statement_c4 Pl_statement
		    | Meta_statement_c5 Function_header_declaration
		    | Meta_statement_c6 Function_body_declaration
		    | Meta_statement_c7 Procedure_header_declaration
		    | Meta_statement_c8 Procedure_body_declaration
 
data Meta_statement_semicolon = Meta_statement_semicolon Meta_statement
							 (Maybe Kw_semicolon)
 
data Name = Name_c1 Lex_name
	  | Name_c2
	  | Name_c3
	  | Name_c4
	  | Name_c5
 
data Name_or_keyword = Name_or_keyword Lex_name_or_keyword
 
data Names = Names [Name]
 
data Names_brackets = Names_brackets Names
 
data Next_date = Next_date Date
 
data Next_memory_size = Next_memory_size Memory_size
 
data Null_statement = Null_statement
 
data Number = Number Lex_number
 
data Of_columns = Of_columns Names
 
data On_clause = On_clause Qualified_name
 
data Open_for_clause = Open_for_clause_c1 Select_statement
		     | Open_for_clause_c2 Name Parameters
 
data Open_for_cursor = Open_for_cursor_c1 Cursor_name
		     | Open_for_cursor_c2 Host_name
 
data Open_for_statement = Open_for_statement Open_for_cursor
					     Open_for_clause
 
data Open_statement = Open_statement Cursor_ref
 
data Or_exception_name = Or_exception_name Exception_name
 
data Order_by_clause = Order_by_clause Order_items
 
data Order_item = Order_item Expression (Maybe Kw_desc_asc)
 
data Order_items = Order_items [Order_item]
 
data Ordered_column = Ordered_column Column_name
				     (Maybe Kw_desc_asc)
 
data Ordered_columns = Ordered_columns [Ordered_column]
 
data Package_body = Package_body_c1 Package_body_prefix
				    (Maybe Declarations_semicolon) (Maybe Statement_block_no_end)
				    End_of_package
		  | Package_body_c2 Package_body_prefix
				    (Maybe Declarations_semicolon) (Maybe Declarations_semicolon)
				    Statement_block (Maybe Kw_semicolon) End_of_package
				    (Maybe Kw_semicolon) (Maybe Trailing_declarations)
 
data Package_body_prefix = Package_body_prefix (Maybe
						  Create_or_replace)
					       Package_name Kw_is_as
 
data Package_declaration = Package_declaration_c1 Package_header
			 | Package_declaration_c2 Package_body
 
data Package_header = Package_header Package_header_prefix
				     (Maybe Declarations_semicolon) End_of_package
 
data Package_header_prefix = Package_header_prefix (Maybe
						      Create_or_replace)
						   Package_name Kw_is_as
 
data Package_name = Package_name Lex_qualified_name
 
data Parallel_clause = Parallel_clause_c1
		     | Parallel_clause_c2 Parallel_items
 
data Parallel_items = Parallel_items_c1 Integer_or_default
		    | Parallel_items_c2 Integer_or_default
		    | Parallel_items_c3 Integer_or_default Parallel_items
 
data Parameter = Parameter_c1 Top_expression
	       | Parameter_c2 Name Top_expression
 
data Parameter_declaration = Parameter_declaration Name
						   [Kw_in_out_nocopy] Return_type (Maybe Init_value)
 
data Parameter_declarations = Parameter_declarations [Parameter_declaration]
 
data Parameter_declarations_brackets = Parameter_declarations_brackets Parameter_declarations
 
data Parameters = Parameters [Parameter]
 
data Parameters_brackets = Parameters_brackets Parameters
 
data Password = Password Lex_name
 
data Perc_found = Perc_found
 
data Perc_isopen = Perc_isopen
 
data Perc_notfound = Perc_notfound
 
data Perc_rowcount = Perc_rowcount
 
data Perc_rowtype = Perc_rowtype
 
data Perc_type = Perc_type
 
data Perc_typed_name = Perc_typed_name_c1 Name
		     | Perc_typed_name_c2 Name Name
		     | Perc_typed_name_c3 Name Perc_type
		     | Perc_typed_name_c4 Name Name Perc_type
		     | Perc_typed_name_c5 Name Perc_rowtype
		     | Perc_typed_name_c6 Name Name Perc_rowtype
		     | Perc_typed_name_c7 Datatype
 
data Permission = Permission_c1
		| Permission_c2
 
data Pl_statement = Pl_statement (Maybe Label_header)
				 Unlabelled_pl_statement
 
data Pl_statements = Pl_statements_c1 Pl_statement
				      (Maybe Kw_semicolon)
		   | Pl_statements_c2 Pl_statement Kw_semicolon Pl_statements
 
data Plsql_block = Plsql_block (Maybe Label_header)
			       (Maybe Declare_section) Statement_block
 
data Position = Position_c1 SqlInteger SqlInteger
	      | Position_c2 Atomic_expression
 
data Pragma_declaration = Pragma_declaration_c1 Exception_name
						SqlInteger
			| Pragma_declaration_c2 Exception_name Names
			| Pragma_declaration_c3
 
data Procedure_body_declaration = Procedure_body_declaration_c1 Procedure_header_declaration
								Kw_is_as
								(Maybe Declarations_semicolon)
								Statement_block_no_end
								End_of_procedure
				| Procedure_body_declaration_c2 Procedure_header_declaration
								Kw_is_as
								(Maybe Declarations_semicolon)
								Statement_block End_of_procedure
				| Procedure_body_declaration_c3 Procedure_header_declaration
								Kw_is_as Text
 
data Procedure_header_declaration = Procedure_header_declaration Qualified_name
								 (Maybe
								    Parameter_declarations_brackets)
 
data Profile = Profile Lex_name
 
data Qualified_name = Qualified_name_c1 Lex_qualified_name
		    | Qualified_name_c2 Lex_qualified_host_name
		    | Qualified_name_c3 Function_call Name
		    | Qualified_name_c4
		    | Qualified_name_c5
		    | Qualified_name_c6
		    | Qualified_name_c7
		    | Qualified_name_c8 Name
 
data Qualified_names = Qualified_names [Qualified_name]
 
data Quota = Quota Memory_size_or_unlimited Tablespace
 
data Quotas = Quotas [Quota]
 
data Raise_statement = Raise_statement (Maybe Exception_name)
 
data Rebuild_item = Rebuild_item_c1 SqlInteger
		  | Rebuild_item_c2
		  | Rebuild_item_c3
		  | Rebuild_item_c4
		  | Rebuild_item_c5 Tablespace_clause
 
data Rebuild_items = Rebuild_items [Rebuild_item]
 
data Record_type_definition = Record_type_definition Type_name
						     Field_declarations
 
data Recover_clause = Recover_clause (Maybe Kw_automatic)
				     (Maybe From_location) (Maybe Recover_item)
				     (Maybe Parallel_clause)
 
data Recover_database_item = Recover_database_item_c1
			   | Recover_database_item_c2 Date
			   | Recover_database_item_c3 SqlInteger
			   | Recover_database_item_c4
 
data Recover_item = Recover_item_c1 (Maybe Kw_standby_database)
				    Recover_database_item
		  | Recover_item_c2 Names
		  | Recover_item_c3 Filenames
		  | Recover_item_c4 Filename
		  | Recover_item_c5 (Maybe Kw_default)
		  | Recover_item_c6
 
data Ref_type_declaration = Ref_type_declaration Type_name
						 (Maybe Cursor_return)
 
data Referencing = Referencing Referencing_items
 
data Referencing_item = Referencing_item Kw_old_new Name
 
data Referencing_items = Referencing_items [Referencing_item]
 
data Relational_operator = Relational_operator_c1
			 | Relational_operator_c2
			 | Relational_operator_c3
			 | Relational_operator_c4
			 | Relational_operator_c5
			 | Relational_operator_c6
			 | Relational_operator_c7
 
data Rename_statement = Rename_statement Name Name
 
data Resource_cost_item = Resource_cost_item_c1 SqlInteger
			| Resource_cost_item_c2 SqlInteger
			| Resource_cost_item_c3 SqlInteger
			| Resource_cost_item_c4 SqlInteger
 
data Resource_cost_items = Resource_cost_items [Resource_cost_item]
 
data Return_statement = Return_statement (Maybe Top_expression)
 
data Return_type = Return_type Perc_typed_name
 
data Reused_filename = Reused_filename Filename (Maybe Kw_reuse)
 
data Reused_filenames = Reused_filenames [Reused_filename]
 
data Revoke_statement = Revoke_statement [Grant_item]
					 (Maybe On_clause) Grantees (Maybe Kw_cascade_constraints)
 
data Role = Role Lex_name
 
data Role_description = Role_description_c1 Roles
		      | Role_description_c2
		      | Role_description_c3 Roles
		      | Role_description_c4
 
data Role_item = Role_item_c1
	       | Role_item_c2 User_identification
 
data Roles = Roles [Role]
 
data Rollback_segment_item = Rollback_segment_item_c1
			   | Rollback_segment_item_c2
			   | Rollback_segment_item_c3 Storage_clause
			   | Rollback_segment_item_c4 (Maybe Kw_to) (Maybe Memory_size)
 
data Rollback_statement = Rollback_statement (Maybe Kw_work)
					     (Maybe Savepoint)
 
data Row_clause = Row_clause (Maybe Referencing)
			     (Maybe When_clause)
 
data Savepoint = Savepoint_c1 (Maybe Kw_savepoint) Name
	       | Savepoint_c2 Text
	       | Savepoint_c3
 
data Savepoint_statement = Savepoint_statement Name
 
data Schema_qualifier = Schema_qualifier Lex_schema_qualifier
 
data Select_clause = Select_clause_c1 Where_clause
		   | Select_clause_c2 Group_by_clause
		   | Select_clause_c3 Order_by_clause
		   | Select_clause_c4 Connect_by_clause
		   | Select_clause_c5 For_update_clause
 
data Select_clause_operator = Select_clause_operator_c1
			    | Select_clause_operator_c2
			    | Select_clause_operator_c3
			    | Select_clause_operator_c4
 
data Select_item = Select_item_c1 Schema_qualifier (Maybe Text)
		 | Select_item_c2 Expression (Maybe Text)
		 | Select_item_c3 Subquery_brackets
 
data Select_statement = Select_statement (Maybe Kw_distinct_all)
					 Aliased_select_items_or_all (Maybe Into_qualified_names)
					 Aliased_table_descriptions [Select_clause]
 
data Select_statement_without_into = Select_statement_without_into_c1 (Maybe
									 Kw_distinct_all)
								      Aliased_select_items_or_all
								      Aliased_table_descriptions
								      [Select_clause]
				   | Select_statement_without_into_c2 Select_statement_without_into
								      Select_clause_operator
								      Select_statement_without_into
 
data Sequence_item = Sequence_item_c1 SqlInteger
		   | Sequence_item_c2 SqlInteger
		   | Sequence_item_c3 SqlInteger
		   | Sequence_item_c4
		   | Sequence_item_c5 SqlInteger
		   | Sequence_item_c6
		   | Sequence_item_c7
		   | Sequence_item_c8
		   | Sequence_item_c9 SqlInteger
		   | Sequence_item_c10
		   | Sequence_item_c11
		   | Sequence_item_c12
 
data Sequence_items = Sequence_items [Sequence_item]
 
data Session_set_item = Session_set_item_c1 Kw_sql_trace_global_names
					    Kw_true_false
		      | Session_set_item_c2 Language
		      | Session_set_item_c3 Territory
		      | Session_set_item_c4 Text
		      | Session_set_item_c5 Language
		      | Session_set_item_c6 Text
		      | Session_set_item_c7 Territory
		      | Session_set_item_c8 Text
		      | Session_set_item_c9 Binary_or_name
		      | Session_set_item_c10 Text
		      | Session_set_item_c11 Set_label_item
		      | Session_set_item_c12 Name
		      | Session_set_item_c13 Set_optimizer_goal_item
		      | Session_set_item_c14 Set_flagger_item
		      | Session_set_item_c15 Kw_true_false
		      | Session_set_item_c16 SqlInteger
		      | Session_set_item_c17 Kw_true_false
		      | Session_set_item_c18 SqlInteger
		      | Session_set_item_c19 SqlInteger
		      | Session_set_item_c20 Isolation_level
		      | Session_set_item_c21 Text
 
data Session_set_items = Session_set_items [Session_set_item]
 
data Set_flagger_item = Set_flagger_item_c1
		      | Set_flagger_item_c2
		      | Set_flagger_item_c3
		      | Set_flagger_item_c4
 
data Set_item = Set_item_c1 Qualified_name
			    Expression_or_subquery_brackets
	      | Set_item_c2 Columns_brackets Subquery_brackets
 
data Set_items = Set_items [Set_item]
 
data Set_label_item = Set_label_item_c1 Text
		    | Set_label_item_c2
		    | Set_label_item_c3
 
data Set_optimizer_goal_item = Set_optimizer_goal_item_c1
			     | Set_optimizer_goal_item_c2
			     | Set_optimizer_goal_item_c3
			     | Set_optimizer_goal_item_c4
 
data Set_statement_id = Set_statement_id Text
 
data Set_transaction_clause = Set_transaction_clause_c1 Permission
			    | Set_transaction_clause_c2 Isolation_level_clause
			    | Set_transaction_clause_c3 Name
 
data Set_transaction_statement = Set_transaction_statement (Maybe
							      Kw_local)
							   Set_transaction_clause
 
data Snap_shot_item = Snap_shot_item_c1 SqlInteger
		    | Snap_shot_item_c2 SqlInteger
		    | Snap_shot_item_c3 SqlInteger
		    | Snap_shot_item_c4 SqlInteger
		    | Snap_shot_item_c5 Tablespace_clause
		    | Snap_shot_item_c6 Storage_clause
 
data Snap_shot_items = Snap_shot_items [Snap_shot_item]
 
data Snapshot_refresh = Snapshot_refresh (Maybe
					    Kw_fast_complete_force)
					 (Maybe Start_date) (Maybe Next_date)
 
data Sql_plus_identifier = Sql_plus_identifier Lex_sql_plus_identifier
 
data Sql_plus_macro = Sql_plus_macro Lex_sql_plus_macro
 
data Sql_plus_statement = Sql_plus_statement_c1 Filename_with_extension
			| Sql_plus_statement_c2
			| Sql_plus_statement_c3 (Maybe Text) Ttitle_items
			| Sql_plus_statement_c4 Ttitle_items
			| Sql_plus_statement_c5 Sql_plus_identifier Text
			| Sql_plus_statement_c6 Sql_plus_identifier SqlInteger
			| Sql_plus_statement_c7 Sql_plus_identifier Sql_plus_identifier
			| Sql_plus_statement_c8 Name
			| Sql_plus_statement_c9 SqlInteger
			| Sql_plus_statement_c10 Meta_parameter_values
			| Sql_plus_statement_c11 Sql_plus_identifier
			| Sql_plus_statement_c12 Whenever_sqlerror_clause
			| Sql_plus_statement_c13 [Sql_plus_identifier]
			| Sql_plus_statement_c14 Sql_plus_identifier Col_items
			| Sql_plus_statement_c15 Sql_plus_identifier Col_items
			| Sql_plus_statement_c16 [Break_item]
			| Sql_plus_statement_c17 Sql_plus_identifier Text
			| Sql_plus_statement_c18 [Sql_plus_identifier]
			| Sql_plus_statement_c19 Load_data_options
			| Sql_plus_statement_c20
			| Sql_plus_statement_c21
			| Sql_plus_statement_c22
			| Sql_plus_statement_c23 SqlInteger
 
data Sql_plus_value = Sql_plus_value Lex_sql_plus_identifier
 
data Sql_plus_value_or_text = Sql_plus_value_or_text_c1 Sql_plus_value
			    | Sql_plus_value_or_text_c2 Text
 
data Sql_statement = Sql_statement_c1 Alter_procedure_statement
		   | Sql_statement_c2 Alter_table_statement
		   | Sql_statement_c3 Close_statement
		   | Sql_statement_c4 Commit_statement
		   | Sql_statement_c5 Delete_statement
		   | Sql_statement_c6 Insert_statement
		   | Sql_statement_c7 Open_statement
		   | Sql_statement_c8 Rollback_statement
		   | Sql_statement_c9 Savepoint_statement
		   | Sql_statement_c10 Select_statement
		   | Sql_statement_c11 Set_transaction_statement
		   | Sql_statement_c12 Update_statement
		   | Sql_statement_c13 Lock_table_statement
		   | Sql_statement_c14 Open_for_statement
		   | Sql_statement_c15 Alter_cluster_statement
		   | Sql_statement_c16 Alter_database_statement
		   | Sql_statement_c17 Alter_function_statement
		   | Sql_statement_c18 Alter_index_statement
		   | Sql_statement_c19 Alter_package_statement
		   | Sql_statement_c20 Alter_profile_statement
		   | Sql_statement_c21 Alter_resource_cost_statement
		   | Sql_statement_c22 Alter_role_statement
		   | Sql_statement_c23 Alter_rollback_segment_statement
		   | Sql_statement_c24 Alter_sequence_statement
		   | Sql_statement_c25 Alter_session_statement
		   | Sql_statement_c26 Alter_snapshot_statement
		   | Sql_statement_c27 Alter_snapshot_log_statement
		   | Sql_statement_c28 Alter_system_statement
		   | Sql_statement_c29 Alter_tablespace_statement
		   | Sql_statement_c30 Alter_trigger_statement
		   | Sql_statement_c31 Alter_user_statement
		   | Sql_statement_c32 Alter_view_statement
		   | Sql_statement_c33 Create_database_link_statement
		   | Sql_statement_c34 Create_index_statement
		   | Sql_statement_c35 Create_role_statement
		   | Sql_statement_c36 Create_rollback_segment_statement
		   | Sql_statement_c37 Create_sequence_statement
		   | Sql_statement_c38 Create_synonym_statement
		   | Sql_statement_c39 Create_table_statement
		   | Sql_statement_c40 Create_tablespace_statement
		   | Sql_statement_c41 Create_view_statement
		   | Sql_statement_c42 Create_user_statement
		   | Sql_statement_c43 Drop_database_link_statement
		   | Sql_statement_c44 Drop_index_statement
		   | Sql_statement_c45 Drop_function_statement
		   | Sql_statement_c46 Drop_procedure_statement
		   | Sql_statement_c47 Drop_sequence_statement
		   | Sql_statement_c48 Drop_synonym_statement
		   | Sql_statement_c49 Drop_table_statement
		   | Sql_statement_c50 Drop_trigger_statement
		   | Sql_statement_c51 Drop_view_statement
		   | Sql_statement_c52 Explain_plan_statement
		   | Sql_statement_c53 Fetch_statement
		   | Sql_statement_c54 Grant_statement
		   | Sql_statement_c55 Load_data_statement
		   | Sql_statement_c56 Rename_statement
		   | Sql_statement_c57 Revoke_statement
		   | Sql_statement_c58 Truncate_statement
		   | Sql_statement_c59 Create_function_statement
		   | Sql_statement_c60 Create_procedure_statement
		   | Sql_statement_c61 Create_trigger_statement
 
data Start_date = Start_date Date
 
data Statement_block = Statement_block Statement_block_no_end
				       (Maybe Label_name)
 
data Statement_block_no_end = Statement_block_no_end Pl_statements
						     (Maybe Exception_section)
 
data Storage_clause = Storage_clause Storage_items_opt_brackets
 
data Storage_item = Storage_item_c1 Memory_size
		  | Storage_item_c2 Next_memory_size
		  | Storage_item_c3 SqlInteger
		  | Storage_item_c4 Integer_or_unlimited
		  | Storage_item_c5 SqlInteger
		  | Storage_item_c6 SqlInteger
		  | Storage_item_c7 SqlInteger
		  | Storage_item_c8 Memory_size_or_null
 
data Storage_items = Storage_items [Storage_item]
 
data Storage_items_opt_brackets = Storage_items_opt_brackets_c1 Storage_item
				| Storage_items_opt_brackets_c2 Storage_items
 
data Subquery = Subquery Select_statement_without_into
 
data Subquery_brackets = Subquery_brackets Subquery
 
data Subquery_opt_brackets = Subquery_opt_brackets_c1 Subquery
			   | Subquery_opt_brackets_c2 Subquery
 
data Symbol_dot = Symbol_dot
 
data System_item = System_item_c1 Kw_enable_disable
		 | System_item_c2
		 | System_item_c3 Checkpoint (Maybe Kw_global_local)
		 | System_item_c4 System_set_items
		 | System_item_c5
		 | System_item_c6 Kw_enable_disable
		 | System_item_c7 Archive_log_clause
		 | System_item_c8 SqlInteger SqlInteger
 
data System_items = System_items [System_item]
 
data System_set_item = System_set_item_c1 Kw_true_false
		     | System_set_item_c2 Kw_true_false
		     | System_set_item_c3 SqlInteger
		     | System_set_item_c4 SqlInteger
		     | System_set_item_c5 SqlInteger
		     | System_set_item_c6 Name SqlInteger
		     | System_set_item_c7 SqlInteger
		     | System_set_item_c8 SqlInteger
		     | System_set_item_c9 SqlInteger
		     | System_set_item_c10 Kw_timestamp_signature
 
data System_set_items = System_set_items [System_set_item]
 
data Table_description = Table_description_c1 Qualified_name
		       | Table_description_c2 Subquery_brackets
 
data Table_item = Table_item_c1 Column_name Datatype
				(Maybe Default_expression) (Maybe Constraints)
		| Table_item_c2 Constraint
 
data Table_items = Table_items [Table_item]
 
data Table_items_opt_brackets = Table_items_opt_brackets_c1 Table_items
			      | Table_items_opt_brackets_c2 Table_item
 
data Table_modify_item = Table_modify_item Column_name
					   (Maybe Datatype) (Maybe Default_expression)
					   (Maybe Constraints)
 
data Table_modify_items = Table_modify_items [Table_modify_item]
 
data Table_type_declaration = Table_type_declaration Type_name
						     Perc_typed_name (Maybe Kw_not_null)
 
data Tablespace = Tablespace Lex_name
 
data Tablespace_clause = Tablespace_clause Name
					   (Maybe Kw_nologging)
 
data Tablespace_item = Tablespace_item_c1 Datafile_descriptions
		     | Tablespace_item_c2 File_rename_clause
		     | Tablespace_item_c3
		     | Tablespace_item_c4 Storage_clause
		     | Tablespace_item_c5
		     | Tablespace_item_c6 (Maybe Kw_normal_temporary_immediate)
		     | Tablespace_item_c7
		     | Tablespace_item_c8
		     | Tablespace_item_c9
		     | Tablespace_item_c10
		     | Tablespace_item_c11
		     | Tablespace_item_c12
 
data Tablespace_items = Tablespace_items [Tablespace_item]
 
data Territory = Territory Lex_name
 
data Text = Text_c1 Lex_text
	  | Text_c2
 
data Then_branch = Then_branch Pl_statements
 
data Thread_integer = Thread_integer SqlInteger
 
data To_location = To_location Filename
 
data Top_expression = Top_expression_c1 Expression
		    | Top_expression_c2 Boolean_expression
 
data Top_sort = Top_sort_c1 Top_sort_cobol_embedded_sql
	      | Top_sort_c2 Top_sort_pl_sql
 
data Top_sort_cobol_embedded_sql = Top_sort_cobol_embedded_sql [Host_language_statement]
							       (Maybe Symbol_dot)
 
data Top_sort_pl_sql = Top_sort_pl_sql [Meta_statement_semicolon]
 
data Trailing_clause = Trailing_clause
 
data Trailing_declarations = Trailing_declarations (Maybe
						      Declarations_semicolon)
						   (Maybe Package_name)
 
data Trigger_item = Trigger_item_c1
		  | Trigger_item_c2
		  | Trigger_item_c3 (Maybe Of_columns)
 
data Trigger_items = Trigger_items [Trigger_item]
 
data Truncate_clause = Truncate_clause_c1
		     | Truncate_clause_c2
 
data Truncate_statement = Truncate_statement Kw_table_cluster
					     Qualified_name (Maybe Truncate_clause)
 
data Ttitle_date = Ttitle_date_c1
		 | Ttitle_date_c2
 
data Ttitle_item = Ttitle_item_c1
		 | Ttitle_item_c2
		 | Ttitle_item_c3 [Integer_text_var]
		 | Ttitle_item_c4 [Integer_text_var]
		 | Ttitle_item_c5 (Maybe Ttitle_date) [Integer_text_var]
		 | Ttitle_item_c6 (Maybe Ttitle_date) [Integer_text_var]
 
data Ttitle_items = Ttitle_items [Ttitle_item]
 
data Type_name = Type_name Lex_name
 
data Unlabelled_pl_statement = Unlabelled_pl_statement_c1 Null_statement
			     | Unlabelled_pl_statement_c2 Sql_statement
			     | Unlabelled_pl_statement_c3 Assignment_statement
			     | Unlabelled_pl_statement_c4 Call
			     | Unlabelled_pl_statement_c5 Exit_statement
			     | Unlabelled_pl_statement_c6 Goto_statement
			     | Unlabelled_pl_statement_c7 If_statement
			     | Unlabelled_pl_statement_c8 Loop_statement
			     | Unlabelled_pl_statement_c9 Plsql_block
			     | Unlabelled_pl_statement_c10 Raise_statement
			     | Unlabelled_pl_statement_c11 Return_statement
			     | Unlabelled_pl_statement_c12 Comment_statement
			     | Unlabelled_pl_statement_c13 Execute_statement
			     | Unlabelled_pl_statement_c14 Create_type_statement
 
data Update_statement = Update_statement Aliased_table_description
					 Set_items (Maybe Where_clause)
 
data User = User Lex_name
 
data User_identification = User_identification_c1 Password
			 | User_identification_c2
 
data User_item = User_item_c1 User_identification
	       | User_item_c2 Tablespace
	       | User_item_c3 Tablespace
	       | User_item_c4 Quotas
	       | User_item_c5 Profile
	       | User_item_c6 Role_description
 
data User_items = User_items [User_item]
 
data Using_index_clause = Using_index_clause (Maybe
						Using_index_item)
 
data Using_index_item = Using_index_item_c1 SqlInteger
		      | Using_index_item_c2 SqlInteger
		      | Using_index_item_c3 Tablespace_clause
		      | Using_index_item_c4 Storage_clause
		      | Using_index_item_c5 SqlInteger
		      | Using_index_item_c6 SqlInteger
 
data Values_description = Values_description_c1 Expressions
			| Values_description_c2 Subquery
			| Values_description_c3 Subquery_brackets
 
data Var_declaration = Var_declaration Name Return_type
				       (Maybe Declaration_init_value)
 
data View_with_clause = View_with_clause_c1
		      | View_with_clause_c2
		      | View_with_clause_c3 (Maybe Constraint_clause)
 
data When_clause = When_clause Boolean_expression
 
data Whenever_sqlerror_clause = Whenever_sqlerror_clause_c1
			      | Whenever_sqlerror_clause_c2
			      | Whenever_sqlerror_clause_c3
 
data Whenever_sqlerror_statement = Whenever_sqlerror_statement_c1 Emb_name
				 | Whenever_sqlerror_statement_c2
 
data Where_clause = Where_clause Boolean_expression
 
data While_loop_statement = While_loop_statement Boolean_expression
						 Basic_loop_statement
 
data Lex_name = Lex_identifier Lex_identifier
	      | Lex_quoted_name Lex_quoted_name
	      | Lex_ampersand_identifier Lex_ampersand_identifier
	      | REF
	      | Lex_name_kw Lex_name_kw
 
data Lex_qualified_name = Lex_name Lex_name
			| Lex_name_Lex_dot_Lex_name Lex_name Lex_dot Lex_name
			| Lex_name_Lex_dot_Lex_name_Lex_dot_Lex_name Lex_name Lex_dot
								     Lex_name Lex_dot Lex_name
			| Lex_name_Lex_at_Lex_name Lex_name Lex_at Lex_name
			| Lex_name_Lex_dot_Lex_name_Lex_at_Lex_name Lex_name Lex_dot
								    Lex_name Lex_at Lex_name
 
data Lex_host_name = Lex_host_identifier Lex_host_identifier
 
data Lex_qualified_host_name = Lex_host_name_p [Lex_host_name]
			     | Lex_host_name_Lex_dot_Lex_host_identifier Lex_host_name Lex_dot
									 Lex_host_identifier
			     | Lex_host_name_Lex_host_identifier Lex_host_name
								 Lex_host_identifier
 
data Lex_schema_qualifier = Lex_name_Lex_dot_star Lex_name
						  Lex_dot_star
			  | Lex_name_Lex_dot_Lex_name_Lex_dot_star Lex_name Lex_dot Lex_name
								   Lex_dot_star
 
data Kw_k_m = K
	    | M
 
data Lex_memory_size = Lex_memory_size_int_Kw_k_m_opt Lex_memory_size_int
						      (Maybe Kw_k_m)
 
data Lex_sql_plus_macro = Lex_at_Lex_at_Lex_sql_plus_identifier Lex_at
								Lex_at Lex_sql_plus_identifier
 
type Comment_word = String
 
type Comment_spaces = String
 
type Comment_asterisks = String
 
type Comment_start = String
 
type Comment_end = String
 
type Comment_token = String
 
type Lex_identifier = String
 
type Lex_quoted_name = String
 
type Lex_text = String
 
type Lex_integer = String
 
type Lex_decimal = String
 
type Lex_number = String
 
type Lex_precision = String
 
type Lex_dot = String
 
type Lex_host_identifier = String
 
type Lex_memory_size_int = String
 
type Lex_sql_plus_identifier = String
 
type Lex_ampersand_identifier = String
 
type Lex_keyword_except_to_on_from = String
 
type Lex_format = String
 
type Lex_at = String
 
type Lex_dot_star = String
 
type Lex_name_or_keyword = String
 
type Lex_name_kw = String