-------------------------------------------------------------------------------
--
-- | 
-- Module      :  Language.PLSql.SyntaxPP
-- Copyright   :  (c) Tiago Alves, Paulo Silva, 2005
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- This module defines the pretty-printer for PL-Sql, which was generated
-- from an SDF specification using Sdf2Haskell.
--
-------------------------------------------------------------------------------

module Language.PLSql.SyntaxPP where

import Language.PLSql.Syntax
import Language.PLSql.TermInstances()
import Text.PrettyPrint.HughesPJ hiding (Mode)
import GPP
import StrategyLib
 
instance PP Alias where
	pp gpp (Alias _0 _1) = fsep [gppMaybe gpp _0, gpp _1]
 
instance PP Aliased_select_item where
	pp gpp (Aliased_select_item _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Aliased_select_items where
	pp gpp (Aliased_select_items _0) = fsep [gppListSep gpp "," _0]
 
instance PP Aliased_select_items_or_all where
	pp gpp (Aliased_select_items_or_all_c1 _0) = fsep [gpp _0]
	pp gpp (Aliased_select_items_or_all_c2) = fsep [gpp "*"]
 
instance PP Aliased_table_description where
	pp gpp (Aliased_table_description _0 _1)
	  = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Aliased_table_descriptions where
	pp gpp (Aliased_table_descriptions _0)
	  = fsep [gppListSep gpp "," _0]
 
instance PP Allocate_extent_clause where
	pp gpp (Allocate_extent_clause _1)
	  = fsep [gpp "(", gpp _1, gpp ")"]
 
instance PP Allocate_extent_item where
	pp gpp (Allocate_extent_item_c1 _0) = fsep [gpp _0]
	pp gpp (Allocate_extent_item_c2 _1) = fsep [gpp "DATAFILE", gpp _1]
	pp gpp (Allocate_extent_item_c3 _1) = fsep [gpp "INSTANCE", gpp _1]
 
instance PP Allocate_extent_items where
	pp gpp (Allocate_extent_items _0) = fsep [gppList gpp _0]
 
instance PP Allocate_statement where
	pp gpp (Allocate_statement _1) = fsep [gpp "ALLOCATE", gpp _1]
 
instance PP Alter_cluster_item where
	pp gpp (Alter_cluster_item_c1 _1) = fsep [gpp "PCTUSED", gpp _1]
	pp gpp (Alter_cluster_item_c2 _1) = fsep [gpp "PCTFREE", gpp _1]
	pp gpp (Alter_cluster_item_c3 _0) = fsep [gpp _0]
	pp gpp (Alter_cluster_item_c4 _1) = fsep [gpp "INITRANS", gpp _1]
	pp gpp (Alter_cluster_item_c5 _1) = fsep [gpp "MAXTRANS", gpp _1]
	pp gpp (Alter_cluster_item_c6 _0) = fsep [gpp _0]
	pp gpp (Alter_cluster_item_c7 _2)
	  = fsep [gpp "ALLOCATE", gpp "EXTENT", gppMaybe gpp _2]
	pp gpp (Alter_cluster_item_c8 _2)
	  = fsep [gpp "DEALLOCATE", gpp "UNUSED", gppMaybe gpp _2]
 
instance PP Alter_cluster_items where
	pp gpp (Alter_cluster_items _0) = fsep [gppList gpp _0]
 
instance PP Alter_cluster_statement where
	pp gpp (Alter_cluster_statement _2 _3 _4)
	  = fsep
	      [gpp "ALTER", gpp "CLUSTER", gpp _2, gppMaybe gpp _3,
	       gppMaybe gpp _4]
 
instance PP Alter_database_item where
	pp gpp (Alter_database_item_c1 _1 _2)
	  = fsep [gpp "MOUNT", gppMaybe gpp _1, gppMaybe gpp _2]
	pp gpp (Alter_database_item_c2) = fsep [gpp "CONVERT"]
	pp gpp (Alter_database_item_c3 _1)
	  = fsep [gpp "OPEN", gppMaybe gpp _1]
	pp gpp (Alter_database_item_c4 _1) = fsep [gpp "ACTIVATE", gpp _1]
	pp gpp (Alter_database_item_c5) = fsep [gpp "ARCHIVELOG"]
	pp gpp (Alter_database_item_c6) = fsep [gpp "NOARCHIVELOG"]
	pp gpp (Alter_database_item_c7 _1) = fsep [gpp "RECOVER", gpp _1]
	pp gpp (Alter_database_item_c8 _2 _3)
	  = fsep [gpp "ADD", gpp "LOGFILE", gppMaybe gpp _2, gpp _3]
	pp gpp (Alter_database_item_c9 _3)
	  = fsep [gpp "ADD", gpp "LOGFILE", gpp "MEMBER", gpp _3]
	pp gpp (Alter_database_item_c10 _2)
	  = fsep [gpp "DROP", gpp "LOGFILE", gpp _2]
	pp gpp (Alter_database_item_c11 _3)
	  = fsep [gpp "DROP", gpp "LOGFILE", gpp "MEMBER", gpp _3]
	pp gpp (Alter_database_item_c12 _1 _3 _4)
	  = fsep
	      [gpp "CLEAR", gppMaybe gpp _1, gpp "LOGFILE", gpp _3,
	       gppMaybe gpp _4]
	pp gpp (Alter_database_item_c13 _0) = fsep [gpp _0]
	pp gpp (Alter_database_item_c14 _4 _5)
	  = fsep
	      [gpp "CREATE", gpp "STANDBY", gpp "CONTROLFILE", gpp "AS", gpp _4,
	       gppMaybe gpp _5]
	pp gpp (Alter_database_item_c15 _2)
	  = fsep [gpp "BACKUP", gpp "CONTROLFILE", gpp _2]
	pp gpp (Alter_database_item_c16 _3)
	  = fsep [gpp "RENAME", gpp "GLOBAL_NAME", gpp "TO", gpp _3]
	pp gpp (Alter_database_item_c17)
	  = fsep [gpp "RESET", gpp "COMPATIBILITY"]
	pp gpp (Alter_database_item_c18 _1) = fsep [gpp "SET", gpp _1]
	pp gpp (Alter_database_item_c19 _1 _2)
	  = fsep [gpp "ENABLE", gppMaybe gpp _1, gpp _2]
	pp gpp (Alter_database_item_c20 _1) = fsep [gpp "DISABLE", gpp _1]
	pp gpp (Alter_database_item_c21 _2 _4)
	  = fsep [gpp "CREATE", gpp "DATAFILE", gpp _2, gpp "AS", gpp _4]
	pp gpp (Alter_database_item_c22 _1 _2)
	  = fsep [gpp "DATAFILE", gpp _1, gpp _2]
 
instance PP Alter_database_items where
	pp gpp (Alter_database_items _0) = fsep [gppList gpp _0]
 
instance PP Alter_database_set_item where
	pp gpp (Alter_database_set_item_c1 _2)
	  = fsep [gpp "DBLOW", gpp "=", gpp _2]
	pp gpp (Alter_database_set_item_c2 _2)
	  = fsep [gpp "DBHIGH", gpp "=", gpp _2]
	pp gpp (Alter_database_set_item_c3 _1) = fsep [gpp "DBMAC", gpp _1]
 
instance PP Alter_database_statement where
	pp gpp (Alter_database_statement _2 _3)
	  = fsep
	      [gpp "ALTER", gpp "DATABASE", gppMaybe gpp _2, gppMaybe gpp _3]
 
instance PP Alter_datafile_item where
	pp gpp (Alter_datafile_item_c1) = fsep [gpp "ONLINE"]
	pp gpp (Alter_datafile_item_c2 _1)
	  = fsep [gpp "OFFLINE", gppMaybe gpp _1]
	pp gpp (Alter_datafile_item_c3 _1) = fsep [gpp "RESIZE", gpp _1]
	pp gpp (Alter_datafile_item_c4 _0) = fsep [gpp _0]
	pp gpp (Alter_datafile_item_c5) = fsep [gpp "END", gpp "BACKUP"]
 
instance PP Alter_function_statement where
	pp gpp (Alter_function_statement _2)
	  = fsep [gpp "ALTER", gpp "FUNCTION", gpp _2, gpp "COMPILE"]
 
instance PP Alter_index_statement where
	pp gpp (Alter_index_statement _2 _3)
	  = fsep [gpp "ALTER", gpp "INDEX", gpp _2, gpp _3]
 
instance PP Alter_index_statement_item where
	pp gpp (Alter_index_statement_item_c1 _1)
	  = fsep [gpp "PCTFREE", gpp _1]
	pp gpp (Alter_index_statement_item_c2 _1)
	  = fsep [gpp "INITRANS", gpp _1]
	pp gpp (Alter_index_statement_item_c3 _1)
	  = fsep [gpp "MAXTRANS", gpp _1]
	pp gpp (Alter_index_statement_item_c4 _0) = fsep [gpp _0]
	pp gpp (Alter_index_statement_item_c5 _2)
	  = fsep [gpp "ALLOCATE", gpp "EXTENT", gppMaybe gpp _2]
	pp gpp (Alter_index_statement_item_c6 _2)
	  = fsep [gpp "DEALLOCATE", gpp "UNUSED", gppMaybe gpp _2]
	pp gpp (Alter_index_statement_item_c7 _1)
	  = fsep [gpp "REBUILD", gpp _1]
 
instance PP Alter_index_statement_items where
	pp gpp (Alter_index_statement_items _0) = fsep [gppList gpp _0]
 
instance PP Alter_package_statement where
	pp gpp (Alter_package_statement _2 _4)
	  = fsep
	      [gpp "ALTER", gpp "PACKAGE", gpp _2, gpp "COMPILE",
	       gppMaybe gpp _4]
 
instance PP Alter_procedure_statement where
	pp gpp (Alter_procedure_statement _2)
	  = fsep [gpp "ALTER", gpp "PROCEDURE", gpp _2, gpp "COMPILE"]
 
instance PP Alter_profile_item where
	pp gpp (Alter_profile_item_c1 _0 _1) = fsep [gpp _0, gpp _1]
	pp gpp (Alter_profile_item_c2 _1)
	  = fsep [gpp "PRIVATE_SGA", gpp _1]
 
instance PP Alter_profile_item_1 where
	pp gpp (Alter_profile_item_1_c1) = fsep [gpp "SESSIONS_PER_USER"]
	pp gpp (Alter_profile_item_1_c2) = fsep [gpp "CPU_PER_SESSION"]
	pp gpp (Alter_profile_item_1_c3) = fsep [gpp "CPU_PER_CALL"]
	pp gpp (Alter_profile_item_1_c4) = fsep [gpp "CONNECT_TIME"]
	pp gpp (Alter_profile_item_1_c5) = fsep [gpp "IDLE_TIME"]
	pp gpp (Alter_profile_item_1_c6)
	  = fsep [gpp "LOGICAL_READS_PER_SESSION"]
	pp gpp (Alter_profile_item_1_c7)
	  = fsep [gpp "LOGICAL_READS_PER_CALL"]
	pp gpp (Alter_profile_item_1_c8) = fsep [gpp "COMPOSITE_LIMIT"]
 
instance PP Alter_profile_items where
	pp gpp (Alter_profile_items _0) = fsep [gppList gpp _0]
 
instance PP Alter_profile_statement where
	pp gpp (Alter_profile_statement _2 _4)
	  = fsep
	      [gpp "ALTER", gpp "PROFILE", gpp _2, gpp "LIMIT", gppMaybe gpp _4]
 
instance PP Alter_resource_cost_statement where
	pp gpp (Alter_resource_cost_statement _3)
	  = fsep [gpp "ALTER", gpp "RESOURCE", gpp "COST", gpp _3]
 
instance PP Alter_role_statement where
	pp gpp (Alter_role_statement _2 _3)
	  = fsep [gpp "ALTER", gpp "ROLE", gpp _2, gpp _3]
 
instance PP Alter_rollback_segment_statement where
	pp gpp (Alter_rollback_segment_statement _3 _4)
	  = fsep [gpp "ALTER", gpp "ROLLBACK", gpp "SEGMENT", gpp _3, gpp _4]
 
instance PP Alter_sequence_statement where
	pp gpp (Alter_sequence_statement _2 _3)
	  = fsep [gpp "ALTER", gpp "SEQUENCE", gpp _2, gpp _3]
 
instance PP Alter_session_clause where
	pp gpp (Alter_session_clause_c1 _1) = fsep [gpp "SET", gpp _1]
	pp gpp (Alter_session_clause_c2 _3)
	  = fsep [gpp "CLOSE", gpp "DATABASE", gpp "LINK", gpp _3]
	pp gpp (Alter_session_clause_c3 _1) = fsep [gpp "ADVISE", gpp _1]
	pp gpp (Alter_session_clause_c4 _0)
	  = fsep [gpp _0, gpp "COMMIT", gpp "IN", gpp "PROCEDURE"]
 
instance PP Alter_session_statement where
	pp gpp (Alter_session_statement _2)
	  = fsep [gpp "ALTER", gpp "SESSION", gpp _2]
 
instance PP Alter_snapshot_log_statement where
	pp gpp (Alter_snapshot_log_statement _3 _4 _5)
	  = fsep
	      [gpp "ALTER", gpp "SNAPSHOT", gpp "LOG", gppMaybe gpp _3, gpp _4,
	       gppMaybe gpp _5]
 
instance PP Alter_snapshot_statement where
	pp gpp (Alter_snapshot_statement _2 _3 _4 _5)
	  = fsep
	      [gpp "ALTER", gpp "SNAPSHOT", gpp _2, gppMaybe gpp _3,
	       gppMaybe gpp _4, gppMaybe gpp _5]
 
instance PP Alter_system_statement where
	pp gpp (Alter_system_statement _2)
	  = fsep [gpp "ALTER", gpp "SYSTEM", gpp _2]
 
instance PP Alter_table_item where
	pp gpp (Alter_table_item _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Alter_table_item_1 where
	pp gpp (Alter_table_item_1_c1 _1) = fsep [gpp "ADD", gpp _1]
	pp gpp (Alter_table_item_1_c2 _2)
	  = fsep [gpp "MODIFY", gpp "(", gpp _2, gpp ")"]
	pp gpp (Alter_table_item_1_c3 _1) = fsep [gpp "MODIFY", gpp _1]
	pp gpp (Alter_table_item_1_c4 _1) = fsep [gpp "PCTFREE", gpp _1]
	pp gpp (Alter_table_item_1_c5 _1) = fsep [gpp "PCTUSED", gpp _1]
	pp gpp (Alter_table_item_1_c6 _1) = fsep [gpp "INITRANS", gpp _1]
	pp gpp (Alter_table_item_1_c7 _1) = fsep [gpp "MAXTRANS", gpp _1]
	pp gpp (Alter_table_item_1_c8 _0) = fsep [gpp _0]
	pp gpp (Alter_table_item_1_c9 _1) = fsep [gpp "DROP", gpp _1]
	pp gpp (Alter_table_item_1_c10 _2)
	  = fsep [gpp "ALLOCATE", gpp "EXTENT", gppMaybe gpp _2]
	pp gpp (Alter_table_item_1_c11 _2)
	  = fsep [gpp "DEALLOCATE", gpp "UNUSED", gppMaybe gpp _2]
 
instance PP Alter_table_item_2 where
	pp gpp (Alter_table_item_2_c1 _0) = fsep [gpp _0]
	pp gpp (Alter_table_item_2_c2 _0 _1) = fsep [gpp _0, gpp _1]
 
instance PP Alter_table_items_1 where
	pp gpp (Alter_table_items_1 _0) = fsep [gppList gpp _0]
 
instance PP Alter_table_statement where
	pp gpp (Alter_table_statement _2 _3)
	  = fsep [gpp "ALTER", gpp "TABLE", gpp _2, gpp _3]
 
instance PP Alter_tablespace_statement where
	pp gpp (Alter_tablespace_statement _2 _3)
	  = fsep [gpp "ALTER", gpp "TABLESPACE", gpp _2, gpp _3]
 
instance PP Alter_trigger_statement where
	pp gpp (Alter_trigger_statement _2 _3)
	  = fsep [gpp "ALTER", gpp "TRIGGER", gpp _2, gpp _3]
 
instance PP Alter_user_statement where
	pp gpp (Alter_user_statement _2 _3)
	  = fsep [gpp "ALTER", gpp "USER", gpp _2, gpp _3]
 
instance PP Alter_view_statement where
	pp gpp (Alter_view_statement _2)
	  = fsep [gpp "ALTER", gpp "VIEW", gpp _2, gpp "COMPILE"]
 
instance PP Append_item where
	pp gpp (Append_item _0 _1 _2 _3 _4)
	  = fsep
	      [gpp _0, gppMaybe gpp _1, gppMaybe gpp _2, gppMaybe gpp _3,
	       gppMaybe gpp _4]
 
instance PP Append_items where
	pp gpp (Append_items _0) = fsep [gppListSep gpp "," _0]
 
instance PP Append_items_brackets where
	pp gpp (Append_items_brackets _1) = fsep [gpp "(", gpp _1, gpp ")"]
 
instance PP Archive_log_clause where
	pp gpp (Archive_log_clause _2 _3)
	  = fsep [gpp "ARCHIVE", gpp "LOG", gppMaybe gpp _2, gpp _3]
 
instance PP Archive_log_item where
	pp gpp (Archive_log_item_c1 _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
	pp gpp (Archive_log_item_c2) = fsep [gpp "STOP"]
 
instance PP Archive_log_item_located where
	pp gpp (Archive_log_item_located_c1 _1)
	  = fsep [gpp "SEQUENCE", gpp _1]
	pp gpp (Archive_log_item_located_c2 _1)
	  = fsep [gpp "CHANGE", gpp _1]
	pp gpp (Archive_log_item_located_c3) = fsep [gpp "CURRENT"]
	pp gpp (Archive_log_item_located_c4 _1)
	  = fsep [gpp "GROUP", gpp _1]
	pp gpp (Archive_log_item_located_c5 _1)
	  = fsep [gpp "LOGFILE", gpp _1]
	pp gpp (Archive_log_item_located_c6) = fsep [gpp "NEXT"]
	pp gpp (Archive_log_item_located_c7) = fsep [gpp "ALL"]
	pp gpp (Archive_log_item_located_c8) = fsep [gpp "START"]
 
instance PP As_subquery where
	pp gpp (As_subquery _1) = fsep [gpp "AS", gpp _1]
 
instance PP Assignment_statement where
	pp gpp (Assignment_statement _0 _1 _2)
	  = fsep [gpp _0, gpp _1, gpp _2]
 
instance PP Assignment_symbol where
	pp gpp (Assignment_symbol) = fsep [gpp ":="]
 
instance PP Atomic_boolean_expression where
	pp gpp (Atomic_boolean_expression_c1 _0) = fsep [gpp _0]
	pp gpp (Atomic_boolean_expression_c2 _0 _1 _2)
	  = fsep [gpp _0, gpp _1, gpp _2]
	pp gpp (Atomic_boolean_expression_c3 _0 _1 _2)
	  = fsep [gpp _0, gpp _1, gpp _2]
	pp gpp (Atomic_boolean_expression_c4 _0 _1 _2 _3)
	  = fsep [gpp _0, gpp _1, gpp _2, gpp _3]
	pp gpp (Atomic_boolean_expression_c5 _0 _1 _3)
	  = fsep [gpp _0, gppMaybe gpp _1, gpp "IN", gpp _3]
	pp gpp (Atomic_boolean_expression_c6 _0 _1 _3)
	  = fsep [gpp _0, gppMaybe gpp _1, gpp "IN", gpp _3]
	pp gpp (Atomic_boolean_expression_c7 _0 _1 _3 _5)
	  = fsep
	      [gpp _0, gppMaybe gpp _1, gpp "BETWEEN", gpp _3, gpp "AND", gpp _5]
	pp gpp (Atomic_boolean_expression_c8 _0 _2)
	  = fsep [gpp _0, gpp "IS", gppMaybe gpp _2, gpp "NULL"]
	pp gpp (Atomic_boolean_expression_c9 _1)
	  = fsep [gpp "EXISTS", gpp _1]
	pp gpp (Atomic_boolean_expression_c10 _1)
	  = fsep [gpp "(", gpp _1, gpp ")"]
	pp gpp (Atomic_boolean_expression_c11 _0) = fsep [gpp _0]
	pp gpp (Atomic_boolean_expression_c12 _2)
	  = fsep [gpp "CURRENT", gpp "OF", gpp _2]
	pp gpp (Atomic_boolean_expression_c13 _0 _1)
	  = fsep [gpp _0, gpp _1]
	pp gpp (Atomic_boolean_expression_c14 _0 _1 _3)
	  = fsep [gpp _0, gppMaybe gpp _1, gpp "LIKE", gpp _3]
	pp gpp (Atomic_boolean_expression_c15 _0 _1 _3 _4)
	  = fsep
	      [gpp _0, gppMaybe gpp _1, gpp "LIKE", gpp _3, gppList gpp _4]
 
instance PP Atomic_expression where
	pp gpp (Atomic_expression_c1 _0) = fsep [gpp _0]
	pp gpp (Atomic_expression_c2 _0) = fsep [gpp _0]
	pp gpp (Atomic_expression_c3 _0 _1) = fsep [gpp _0, gpp _1]
	pp gpp (Atomic_expression_c4 _0) = fsep [gpp _0]
	pp gpp (Atomic_expression_c5 _0) = fsep [gpp _0]
	pp gpp (Atomic_expression_c6 _1) = fsep [gpp "(", gpp _1, gpp ")"]
	pp gpp (Atomic_expression_c7 _0 _1) = fsep [gpp _0, gpp _1]
	pp gpp (Atomic_expression_c8 _0)
	  = fsep [gpp _0, gpp "(", gpp "+", gpp ")"]
	pp gpp (Atomic_expression_c9 _1 _3)
	  = fsep [gpp "(", gpp _1, gpp ":", gpp _3, gpp ")"]
 
instance PP Autoextend where
	pp gpp (Autoextend_c1) = fsep [gpp "OFF"]
	pp gpp (Autoextend_c2 _1 _2)
	  = fsep [gpp "ON", gppMaybe gpp _1, gppMaybe gpp _2]
 
instance PP Autoextend_clause where
	pp gpp (Autoextend_clause _1) = fsep [gpp "AUTOEXTEND", gpp _1]
 
instance PP Backup_controlfile_clause where
	pp gpp (Backup_controlfile_clause_c1 _1 _2)
	  = fsep [gpp "TO", gpp _1, gppMaybe gpp _2]
	pp gpp (Backup_controlfile_clause_c2 _2)
	  = fsep [gpp "TO", gpp "TRACE", gppMaybe gpp _2]
 
instance PP Basic_loop_statement where
	pp gpp (Basic_loop_statement _1 _4)
	  = fsep [gpp "LOOP", gpp _1, gpp "END", gpp "LOOP", gppMaybe gpp _4]
 
instance PP Begin_declare_section_statement where
	pp gpp (Begin_declare_section_statement)
	  = fsep [gpp "BEGIN", gpp "DECLARE", gpp "SECTION"]
 
instance PP Binary_or_name where
	pp gpp (Binary_or_name_c1) = fsep [gpp "BINARY"]
	pp gpp (Binary_or_name_c2 _0) = fsep [gpp _0]
 
instance PP Boolean_expression where
	pp gpp (Boolean_expression_c1 _0) = fsep [gpp _0]
	pp gpp (Boolean_expression_c2 _1) = fsep [gpp "NOT", gpp _1]
	pp gpp (Boolean_expression_c3 _0 _2)
	  = fsep [gpp _0, gpp "AND", gpp _2]
	pp gpp (Boolean_expression_c4 _0 _2)
	  = fsep [gpp _0, gpp "OR", gpp _2]
 
instance PP Break_item where
	pp gpp (Break_item _1) = fsep [gpp "ON", gppList gpp _1]
 
instance PP Bulk_collect where
	pp gpp (Bulk_collect) = fsep [gpp "BULK", gpp "COLLECT"]
 
instance PP Call where
	pp gpp (Call_c1 _0) = fsep [gpp _0]
	pp gpp (Call_c2 _0) = fsep [gpp _0]
 
instance PP Checkpoint where
	pp gpp (Checkpoint_c1) = fsep [gpp "CHECKPOINT"]
	pp gpp (Checkpoint_c2) = fsep [gpp "CHECK", gpp "DATAFILES"]
 
instance PP Close_statement where
	pp gpp (Close_statement _1) = fsep [gpp "CLOSE", gpp _1]
 
instance PP Col_item where
	pp gpp (Col_item_c1 _1) = fsep [gpp "FORMAT", gpp _1]
	pp gpp (Col_item_c2 _1) = fsep [gpp "JUST", gpp _1]
	pp gpp (Col_item_c3) = fsep [gpp "NEWLINE"]
	pp gpp (Col_item_c4) = fsep [gpp "NOPRINT"]
	pp gpp (Col_item_c5) = fsep [gpp "PRINT"]
	pp gpp (Col_item_c6 _1) = fsep [gpp "HEADING", gpp _1]
	pp gpp (Col_item_c7 _1) = fsep [gpp "HEADING", gpp _1]
	pp gpp (Col_item_c8 _1) = fsep [gpp "NEW_VALUE", gpp _1]
 
instance PP Col_items where
	pp gpp (Col_items _0) = fsep [gppList gpp _0]
 
instance PP Column_name where
	pp gpp (Column_name _0) = fsep [gpp _0]
 
instance PP Column_names where
	pp gpp (Column_names _0) = fsep [gppListSep gpp "," _0]
 
instance PP Columns_brackets where
	pp gpp (Columns_brackets _1) = fsep [gpp "(", gpp _1, gpp ")"]
 
instance PP Comment where
	pp gpp (Comment_c1 _1) = fsep [gpp "COMMENT", gpp _1]
	pp gpp (Comment_c2) = fsep [gpp "RELEASE"]
 
instance PP Comment_item where
	pp gpp (Comment_item_c1) = fsep [gpp "TABLE"]
	pp gpp (Comment_item_c2) = fsep [gpp "COLUMN"]
 
instance PP Comment_statement where
	pp gpp (Comment_statement _2 _3 _5)
	  = fsep [gpp "COMMENT", gpp "ON", gpp _2, gpp _3, gpp "IS", gpp _5]
 
instance PP Commit_statement where
	pp gpp (Commit_statement _1 _2)
	  = fsep [gpp "COMMIT", gppMaybe gpp _1, gppMaybe gpp _2]
 
instance PP Connect_by_clause where
	pp gpp (Connect_by_clause _0 _3 _4)
	  = fsep
	      [gppMaybe gpp _0, gpp "CONNECT", gpp "BY", gpp _3, gppMaybe gpp _4]
 
instance PP Connect_by_clause_prefix where
	pp gpp (Connect_by_clause_prefix _2)
	  = fsep [gpp "START", gpp "WITH", gpp _2]
 
instance PP Connect_to_clause where
	pp gpp (Connect_to_clause _2 _5)
	  = fsep
	      [gpp "CONNECT", gpp "TO", gpp _2, gpp "IDENTIFIED", gpp "BY",
	       gpp _5]
 
instance PP Connect_using_clause where
	pp gpp (Connect_using_clause _1) = fsep [gpp "USING", gpp _1]
 
instance PP Constant_declaration where
	pp gpp (Constant_declaration _0 _1) = fsep [gpp _0, gpp _1]
 
instance PP Constant_declaration_name where
	pp gpp (Constant_declaration_name _0 _2)
	  = fsep [gpp _0, gpp "CONSTANT", gpp _2]
 
instance PP Constraint where
	pp gpp (Constraint_c1 _0 _1 _2)
	  = fsep [gpp _0, gppMaybe gpp _1, gppMaybe gpp _2]
	pp gpp (Constraint_c2 _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Constraint_clause where
	pp gpp (Constraint_clause _1) = fsep [gpp "CONSTRAINT", gpp _1]
 
instance PP Constraint_header where
	pp gpp (Constraint_header_c1 _1) = fsep [gpp "CONSTRAINT", gpp _1]
	pp gpp (Constraint_header_c2 _1) = fsep [gpp "CONSTRAINT", gpp _1]
 
instance PP Constraint_item_1 where
	pp gpp (Constraint_item_1_c1 _0 _1)
	  = fsep [gpp _0, gppMaybe gpp _1]
	pp gpp (Constraint_item_1_c2 _2 _4 _5 _6)
	  = fsep
	      [gpp "FOREIGN", gpp "KEY", gpp _2, gpp "REFERENCES", gpp _4,
	       gppMaybe gpp _5, gppMaybe gpp _6]
	pp gpp (Constraint_item_1_c3 _2)
	  = fsep [gpp "CHECK", gpp "(", gpp _2, gpp ")"]
	pp gpp (Constraint_item_1_c4 _0)
	  = fsep [gppMaybe gpp _0, gpp "NULL"]
	pp gpp (Constraint_item_1_c5 _1 _2 _3)
	  = fsep [gpp "REFERENCES", gpp _1, gppMaybe gpp _2, gppMaybe gpp _3]
	pp gpp (Constraint_item_1_c6) = fsep [gpp "CASCADE"]
 
instance PP Constraint_item_2 where
	pp gpp (Constraint_item_2_c1 _2)
	  = fsep [gpp "USING", gpp "INDEX", gpp _2]
	pp gpp (Constraint_item_2_c2) = fsep [gpp "DISABLE"]
 
instance PP Constraints where
	pp gpp (Constraints _0) = fsep [gppList gpp _0]
 
instance PP Create_database_link_statement where
	pp gpp (Create_database_link_statement _1 _4 _5 _6)
	  = fsep
	      [gpp "CREATE", gppMaybe gpp _1, gpp "DATABASE", gpp "LINK", gpp _4,
	       gppMaybe gpp _5, gppMaybe gpp _6]
 
instance PP Create_function_statement where
	pp gpp (Create_function_statement _1 _2)
	  = fsep [gpp "CREATE", gppMaybe gpp _1, gpp _2]
 
instance PP Create_index_statement where
	pp gpp (Create_index_statement _1 _2 _4 _6 _7 _8)
	  = fsep
	      [gpp "CREATE", gppMaybe gpp _1, gppMaybe gpp _2, gpp "INDEX",
	       gpp _4, gpp "ON", gpp _6, gppMaybe gpp _7, gppMaybe gpp _8]
 
instance PP Create_or_replace where
	pp gpp (Create_or_replace _1)
	  = fsep [gpp "CREATE", gppMaybe gpp _1]
 
instance PP Create_procedure_statement where
	pp gpp (Create_procedure_statement _1 _2)
	  = fsep [gpp "CREATE", gppMaybe gpp _1, gpp _2]
 
instance PP Create_role_statement where
	pp gpp (Create_role_statement _2 _3)
	  = fsep [gpp "CREATE", gpp "ROLE", gpp _2, gppMaybe gpp _3]
 
instance PP Create_rollback_segment_item where
	pp gpp (Create_rollback_segment_item_c1 _0) = fsep [gpp _0]
	pp gpp (Create_rollback_segment_item_c2 _0) = fsep [gpp _0]
	pp gpp (Create_rollback_segment_item_c3 _2)
	  = fsep [gpp "OPTIMAL", gpp "TO", gpp _2]
	pp gpp (Create_rollback_segment_item_c4)
	  = fsep [gpp "OPTIMAL", gpp "NULL"]
 
instance PP Create_rollback_segment_items where
	pp gpp (Create_rollback_segment_items _0) = fsep [gppList gpp _0]
 
instance PP Create_rollback_segment_statement where
	pp gpp (Create_rollback_segment_statement _1 _4 _5)
	  = fsep
	      [gpp "CREATE", gppMaybe gpp _1, gpp "ROLLBACK", gpp "SEGMENT",
	       gpp _4, gppMaybe gpp _5]
 
instance PP Create_sequence_statement where
	pp gpp (Create_sequence_statement _2 _3)
	  = fsep [gpp "CREATE", gpp "SEQUENCE", gpp _2, gppList gpp _3]
 
instance PP Create_synonym_statement where
	pp gpp (Create_synonym_statement _1 _3 _5)
	  = fsep
	      [gpp "CREATE", gppMaybe gpp _1, gpp "SYNONYM", gpp _3, gpp "FOR",
	       gpp _5]
 
instance PP Create_table_clause_1 where
	pp gpp (Create_table_clause_1_c1 _0) = fsep [gpp _0]
	pp gpp (Create_table_clause_1_c2 _1 _2)
	  = fsep [gpp "CLUSTER", gpp _1, gpp _2]
 
instance PP Create_table_clause_2 where
	pp gpp (Create_table_clause_2 _0 _1 _2)
	  = fsep [gppMaybe gpp _0, gppMaybe gpp _1, gppMaybe gpp _2]
 
instance PP Create_table_statement where
	pp gpp (Create_table_statement _2 _3 _4 _5 _6)
	  = fsep
	      [gpp "CREATE", gpp "TABLE", gpp _2, gppMaybe gpp _3,
	       gppMaybe gpp _4, gppMaybe gpp _5, gpp _6]
 
instance PP Create_tablespace_statement where
	pp gpp (Create_tablespace_statement _2 _4 _5)
	  = fsep
	      [gpp "CREATE", gpp "TABLESPACE", gpp _2, gpp "DATAFILE", gpp _4,
	       gpp _5]
 
instance PP Create_trigger_statement where
	pp gpp (Create_trigger_statement _1 _3 _4 _5 _7 _8 _9)
	  = fsep
	      [gpp "CREATE", gppMaybe gpp _1, gpp "TRIGGER", gpp _3, gpp _4,
	       gpp _5, gpp "ON", gpp _7, gppMaybe gpp _8, gpp _9]
 
instance PP Create_type_statement where
	pp gpp (Create_type_statement _1 _3 _4 _7)
	  = fsep
	      [gpp "CREATE", gppMaybe gpp _1, gpp "TYPE", gpp _3,
	       gppMaybe gpp _4, gpp "TABLE", gpp "OF", gpp _7]
 
instance PP Create_user_statement where
	pp gpp (Create_user_statement _2 _3)
	  = fsep [gpp "CREATE", gpp "USER", gpp _2, gppMaybe gpp _3]
 
instance PP Create_view_statement where
	pp gpp (Create_view_statement _1 _2 _4 _5 _7 _8)
	  = fsep
	      [gpp "CREATE", gppMaybe gpp _1, gppMaybe gpp _2, gpp "VIEW",
	       gpp _4, gppMaybe gpp _5, gpp "AS", gpp _7, gppMaybe gpp _8]
 
instance PP Cursor where
	pp gpp (Cursor_c1 _0) = fsep [gpp _0]
	pp gpp (Cursor_c2 _1) = fsep [gpp ":", gpp _1]
 
instance PP Cursor_attribute_attr where
	pp gpp (Cursor_attribute_attr_c1 _0) = fsep [gpp _0]
	pp gpp (Cursor_attribute_attr_c2 _0) = fsep [gpp _0]
	pp gpp (Cursor_attribute_attr_c3 _0) = fsep [gpp _0]
	pp gpp (Cursor_attribute_attr_c4 _0) = fsep [gpp _0]
 
instance PP Cursor_declaration where
	pp gpp (Cursor_declaration _0 _2) = fsep [gpp _0, gpp "IS", gpp _2]
 
instance PP Cursor_declaration_kernel where
	pp gpp (Cursor_declaration_kernel _1 _2)
	  = fsep [gpp "CURSOR", gpp _1, gppMaybe gpp _2]
 
instance PP Cursor_name where
	pp gpp (Cursor_name _0) = fsep [gpp _0]
 
instance PP Cursor_ref where
	pp gpp (Cursor_ref _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Cursor_return where
	pp gpp (Cursor_return _1) = fsep [gpp "RETURN", gpp _1]
 
instance PP Datafile_description where
	pp gpp (Datafile_description _0 _1)
	  = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Datafile_descriptions where
	pp gpp (Datafile_descriptions _0) = fsep [gppListSep gpp "," _0]
 
instance PP Datatype where
	pp gpp (Datatype_c1) = fsep [gpp "BINARY_INTEGER"]
	pp gpp (Datatype_c2) = fsep [gpp "INTEGER"]
	pp gpp (Datatype_c3) = fsep [gpp "NUMBER"]
	pp gpp (Datatype_c4 _2)
	  = fsep [gpp "NUMBER", gpp "(", gpp _2, gpp ")"]
	pp gpp (Datatype_c5 _2 _4)
	  = fsep [gpp "NUMBER", gpp "(", gpp _2, gpp ",", gpp _4, gpp ")"]
	pp gpp (Datatype_c6) = fsep [gpp "CHAR"]
	pp gpp (Datatype_c7 _2)
	  = fsep [gpp "CHAR", gpp "(", gpp _2, gpp ")"]
	pp gpp (Datatype_c8) = fsep [gpp "DATE"]
	pp gpp (Datatype_c9) = fsep [gpp "VARCHAR"]
	pp gpp (Datatype_c10 _2)
	  = fsep [gpp "VARCHAR", gpp "(", gpp _2, gpp ")"]
	pp gpp (Datatype_c11) = fsep [gpp "VARCHAR2"]
	pp gpp (Datatype_c12 _2)
	  = fsep [gpp "VARCHAR2", gpp "(", gpp _2, gpp ")"]
	pp gpp (Datatype_c13) = fsep [gpp "LONG"]
	pp gpp (Datatype_c14 _2)
	  = fsep [gpp "INTEGER", gpp "(", gpp _2, gpp ")"]
 
instance PP Date where
	pp gpp (Date _0) = fsep [gpp _0]
 
instance PP Dblink where
	pp gpp (Dblink _0) = fsep [gpp _0]
 
instance PP Declaration where
	pp gpp (Declaration_c1 _0) = fsep [gpp _0]
	pp gpp (Declaration_c2 _0) = fsep [gpp _0]
	pp gpp (Declaration_c3 _0) = fsep [gpp _0]
	pp gpp (Declaration_c4 _0) = fsep [gpp _0]
	pp gpp (Declaration_c5 _0) = fsep [gpp _0]
	pp gpp (Declaration_c6 _0) = fsep [gpp _0]
	pp gpp (Declaration_c7 _0) = fsep [gpp _0]
	pp gpp (Declaration_c8 _0) = fsep [gpp _0]
	pp gpp (Declaration_c9 _0) = fsep [gpp _0]
	pp gpp (Declaration_c10 _0) = fsep [gpp _0]
	pp gpp (Declaration_c11 _0) = fsep [gpp _0]
	pp gpp (Declaration_c12 _0) = fsep [gpp _0]
 
instance PP Declaration_init_value where
	pp gpp (Declaration_init_value _0 _1)
	  = fsep [gppMaybe gpp _0, gpp _1]
 
instance PP Declarations where
	pp gpp (Declarations _0) = fsep [gppListSep gpp ";" _0]
 
instance PP Declarations_semicolon where
	pp gpp (Declarations_semicolon _0 _1)
	  = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Declare_cursor_statement where
	pp gpp (Declare_cursor_statement _1 _4)
	  = fsep [gpp "DECLARE", gpp _1, gpp "CURSOR", gpp "FOR", gpp _4]
 
instance PP Declare_section where
	pp gpp (Declare_section _1) = fsep [gpp "DECLARE", gppMaybe gpp _1]
 
instance PP Default_expression where
	pp gpp (Default_expression _1) = fsep [gpp "DEFAULT", gpp _1]
 
instance PP Delete_item where
	pp gpp (Delete_item_c1 _0) = fsep [gpp _0]
	pp gpp (Delete_item_c2 _0 _1) = fsep [gpp _0, gpp _1]
	pp gpp (Delete_item_c3 _0) = fsep [gpp _0]
 
instance PP Delete_statement where
	pp gpp (Delete_statement _1 _2 _3)
	  = fsep [gpp "DELETE", gppMaybe gpp _1, gpp _2, gppMaybe gpp _3]
 
instance PP Disable_clause where
	pp gpp (Disable_clause _1) = fsep [gpp "DISABLE", gpp _1]
 
instance PP Disable_clause_item where
	pp gpp (Disable_clause_item_c1 _1 _2)
	  = fsep [gpp "UNIQUE", gpp _1, gppMaybe gpp _2]
	pp gpp (Disable_clause_item_c2 _2)
	  = fsep [gpp "PRIMARY", gpp "KEY", gppMaybe gpp _2]
	pp gpp (Disable_clause_item_c3 _0 _1)
	  = fsep [gpp _0, gppMaybe gpp _1]
	pp gpp (Disable_clause_item_c4 _0) = fsep [gpp _0]
	pp gpp (Disable_clause_item_c5) = fsep [gpp "TABLE", gpp "LOCK"]
 
instance PP Drop_database_link_statement where
	pp gpp (Drop_database_link_statement _1 _4)
	  = fsep
	      [gpp "DROP", gppMaybe gpp _1, gpp "DATABASE", gpp "LINK", gpp _4]
 
instance PP Drop_function_statement where
	pp gpp (Drop_function_statement _2)
	  = fsep [gpp "DROP", gpp "FUNCTION", gpp _2]
 
instance PP Drop_index_statement where
	pp gpp (Drop_index_statement _2)
	  = fsep [gpp "DROP", gpp "INDEX", gpp _2]
 
instance PP Drop_procedure_statement where
	pp gpp (Drop_procedure_statement _2)
	  = fsep [gpp "DROP", gpp "PROCEDURE", gpp _2]
 
instance PP Drop_sequence_statement where
	pp gpp (Drop_sequence_statement _2)
	  = fsep [gpp "DROP", gpp "SEQUENCE", gpp _2]
 
instance PP Drop_synonym_statement where
	pp gpp (Drop_synonym_statement _1 _3)
	  = fsep [gpp "DROP", gppMaybe gpp _1, gpp "SYNONYM", gpp _3]
 
instance PP Drop_table_statement where
	pp gpp (Drop_table_statement _2 _3)
	  = fsep [gpp "DROP", gpp "TABLE", gpp _2, gppMaybe gpp _3]
 
instance PP Drop_trigger_statement where
	pp gpp (Drop_trigger_statement _2)
	  = fsep [gpp "DROP", gpp "TRIGGER", gpp _2]
 
instance PP Drop_view_statement where
	pp gpp (Drop_view_statement _2)
	  = fsep [gpp "DROP", gpp "VIEW", gpp _2]
 
instance PP Dummy_top_sort where
	pp gpp (Dummy_top_sort_c1 _0) = fsep [gpp _0]
	pp gpp (Dummy_top_sort_c2 _0) = fsep [gpp _0]
 
instance PP Else_branch where
	pp gpp (Else_branch _1) = fsep [gpp "ELSE", gpp _1]
 
instance PP Elsif_branch where
	pp gpp (Elsif_branch _1 _3)
	  = fsep [gpp "ELSIF", gpp _1, gpp "THEN", gpp _3]
 
instance PP Emb_name where
	pp gpp (Emb_name _0) = fsep [gpp _0]
 
instance PP Embedded_sql_statement where
	pp gpp (Embedded_sql_statement_c1 _0) = fsep [gpp _0]
	pp gpp (Embedded_sql_statement_c2 _0) = fsep [gpp _0]
	pp gpp (Embedded_sql_statement_c3 _0) = fsep [gpp _0]
	pp gpp (Embedded_sql_statement_c4 _0) = fsep [gpp _0]
	pp gpp (Embedded_sql_statement_c5 _0) = fsep [gpp _0]
	pp gpp (Embedded_sql_statement_c6 _0) = fsep [gpp _0]
	pp gpp (Embedded_sql_statement_c7 _0) = fsep [gpp _0]
	pp gpp (Embedded_sql_statement_c8 _0) = fsep [gpp _0]
 
instance PP Embedded_sql_statements where
	pp gpp (Embedded_sql_statements _0) = fsep [gppListSep gpp ";" _0]
 
instance PP En_dis_able_clauses where
	pp gpp (En_dis_able_clauses_c1 _0) = fsep [gpp _0]
	pp gpp (En_dis_able_clauses_c2 _0) = fsep [gpp _0]
	pp gpp (En_dis_able_clauses_c3 _0 _1) = fsep [gpp _0, gpp _1]
	pp gpp (En_dis_able_clauses_c4 _0 _1) = fsep [gpp _0, gpp _1]
 
instance PP Enable_clause where
	pp gpp (Enable_clause _1) = fsep [gpp "ENABLE", gpp _1]
 
instance PP Enable_clause_item where
	pp gpp (Enable_clause_item_c1 _0 _1 _2)
	  = fsep [gpp _0, gppMaybe gpp _1, gppMaybe gpp _2]
	pp gpp (Enable_clause_item_c2 _0) = fsep [gpp _0]
	pp gpp (Enable_clause_item_c3) = fsep [gpp "TABLE", gpp "LOCK"]
 
instance PP Enable_clause_item_header where
	pp gpp (Enable_clause_item_header_c1 _1)
	  = fsep [gpp "UNIQUE", gpp _1]
	pp gpp (Enable_clause_item_header_c2)
	  = fsep [gpp "PRIMARY", gpp "KEY"]
	pp gpp (Enable_clause_item_header_c3 _0) = fsep [gppMaybe gpp _0]
 
instance PP End_declare_section_statement where
	pp gpp (End_declare_section_statement)
	  = fsep [gpp "END", gpp "DECLARE", gpp "SECTION"]
 
instance PP End_of_function where
	pp gpp (End_of_function _1) = fsep [gpp "END", gppMaybe gpp _1]
 
instance PP End_of_package where
	pp gpp (End_of_package _1) = fsep [gpp "END", gppMaybe gpp _1]
 
instance PP End_of_procedure where
	pp gpp (End_of_procedure _1) = fsep [gpp "END", gppMaybe gpp _1]
 
instance PP Exception_declaration where
	pp gpp (Exception_declaration _0) = fsep [gpp _0, gpp "EXCEPTION"]
 
instance PP Exception_handler where
	pp gpp (Exception_handler _1 _3)
	  = fsep [gpp "WHEN", gpp _1, gpp "THEN", gpp _3]
 
instance PP Exception_handler_others where
	pp gpp (Exception_handler_others _3)
	  = fsep [gpp "WHEN", gpp "OTHERS", gpp "THEN", gpp _3]
 
instance PP Exception_handlers where
	pp gpp (Exception_handlers _0 _1)
	  = fsep [gppList gpp _0, gppMaybe gpp _1]
 
instance PP Exception_name where
	pp gpp (Exception_name_c1 _0) = fsep [gpp _0]
	pp gpp (Exception_name_c2) = fsep [gpp "OTHERS"]
 
instance PP Exception_section where
	pp gpp (Exception_section _1) = fsep [gpp "EXCEPTION", gpp _1]
 
instance PP Exception_when_clause where
	pp gpp (Exception_when_clause _0 _1)
	  = fsep [gpp _0, gppList gpp _1]
 
instance PP Exceptions_into where
	pp gpp (Exceptions_into _2)
	  = fsep [gpp "EXCEPTIONS", gpp "INTO", gpp _2]
 
instance PP Execute_parameter where
	pp gpp (Execute_parameter _0 _1) = fsep [gppMaybe gpp _0, gpp _1]
 
instance PP Execute_statement where
	pp gpp (Execute_statement _1 _2 _3 _4)
	  = fsep
	      [gpp "EXECUTE", gppMaybe gpp _1, gpp _2, gppMaybe gpp _3,
	       gppMaybe gpp _4]
 
instance PP Execute_using where
	pp gpp (Execute_using _1)
	  = fsep [gpp "USING", gppListSep gpp "," _1]
 
instance PP Exit_statement where
	pp gpp (Exit_statement _1 _2)
	  = fsep [gpp "EXIT", gppMaybe gpp _1, gppMaybe gpp _2]
 
instance PP Exit_when_clause where
	pp gpp (Exit_when_clause _1) = fsep [gpp "WHEN", gpp _1]
 
instance PP Explain_plan_statement where
	pp gpp (Explain_plan_statement _2 _3 _5)
	  = fsep
	      [gpp "EXPLAIN", gpp "PLAN", gppMaybe gpp _2, gppMaybe gpp _3,
	       gpp "FOR", gpp _5]
 
instance PP Expression where
	pp gpp (Expression_c1 _0) = fsep [gpp _0]
	pp gpp (Expression_c2 _1) = fsep [gpp "-", gpp _1]
	pp gpp (Expression_c3 _1) = fsep [gpp "+", gpp _1]
	pp gpp (Expression_c4 _0 _2) = fsep [gpp _0, gpp "**", gpp _2]
	pp gpp (Expression_Expression1 _0 _2)
	  = fsep [gpp _0, gpp "*", gpp _2]
	pp gpp (Expression_Expression2 _0 _2)
	  = fsep [gpp _0, gpp "/", gpp _2]
	pp gpp (Expression_Expression3 _0 _2)
	  = fsep [gpp _0, gpp "+", gpp _2]
	pp gpp (Expression_c8 _0 _2) = fsep [gpp _0, gpp "-", gpp _2]
	pp gpp (Expression_c9 _0 _2) = fsep [gpp _0, gpp "||", gpp _2]
 
instance PP Expression_or_subquery_brackets where
	pp gpp (Expression_or_subquery_brackets_c1 _0) = fsep [gpp _0]
	pp gpp (Expression_or_subquery_brackets_c2 _0) = fsep [gpp _0]
 
instance PP Expressions where
	pp gpp (Expressions _0) = fsep [gppListSep gpp "," _0]
 
instance PP Expressions_brackets where
	pp gpp (Expressions_brackets _1) = fsep [gpp "(", gpp _1, gpp ")"]
 
instance PP Expressions_or_subquery_brackets where
	pp gpp (Expressions_or_subquery_brackets_c1 _0) = fsep [gpp _0]
	pp gpp (Expressions_or_subquery_brackets_c2 _0) = fsep [gpp _0]
 
instance PP Fetch_name where
	pp gpp (Fetch_name _0) = fsep [gpp _0]
 
instance PP Fetch_names where
	pp gpp (Fetch_names _0) = fsep [gppListSep gpp "," _0]
 
instance PP Fetch_statement where
	pp gpp (Fetch_statement _1 _3)
	  = fsep [gpp "FETCH", gpp _1, gpp "INTO", gpp _3]
 
instance PP Field_declaration where
	pp gpp (Field_declaration _0 _1 _2)
	  = fsep [gpp _0, gpp _1, gppMaybe gpp _2]
 
instance PP Field_declarations where
	pp gpp (Field_declarations _0) = fsep [gppListSep gpp "," _0]
 
instance PP File_rename_clause where
	pp gpp (File_rename_clause _1 _2 _4)
	  = fsep [gpp "RENAME", gpp _1, gpp _2, gpp "TO", gpp _4]
 
instance PP Filename where
	pp gpp (Filename _0) = fsep [gpp _0]
 
instance PP Filename_with_extension where
	pp gpp (Filename_with_extension _0) = fsep [gpp _0]
 
instance PP Filenames where
	pp gpp (Filenames _0) = fsep [gppListSep gpp "," _0]
 
instance PP Filenames_opt_brackets where
	pp gpp (Filenames_opt_brackets_c1 _0) = fsep [gpp _0]
	pp gpp (Filenames_opt_brackets_c2 _1)
	  = fsep [gpp "(", gpp _1, gpp ")"]
 
instance PP Filespec where
	pp gpp (Filespec _0 _1 _2)
	  = fsep [gpp _0, gppMaybe gpp _1, gppMaybe gpp _2]
 
instance PP Filespecs where
	pp gpp (Filespecs _0) = fsep [gppListSep gpp "," _0]
 
instance PP For_loop_statement where
	pp gpp (For_loop_statement _1 _3 _4 _5)
	  = fsep
	      [gpp "FOR", gpp _1, gpp "IN", gppMaybe gpp _3, gpp _4, gpp _5]
 
instance PP For_loop_statement_range where
	pp gpp (For_loop_statement_range_c1 _0) = fsep [gpp _0]
	pp gpp (For_loop_statement_range_c2 _0 _2)
	  = fsep [gpp _0, gpp "..", gpp _2]
	pp gpp (For_loop_statement_range_c3 _0) = fsep [gpp _0]
 
instance PP For_sql_statement where
	pp gpp (For_sql_statement _1 _2) = fsep [gpp "FOR", gpp _1, gpp _2]
 
instance PP For_update_clause where
	pp gpp (For_update_clause _2 _3)
	  = fsep [gpp "FOR", gpp "UPDATE", gppMaybe gpp _2, gppMaybe gpp _3]
 
instance PP For_update_item where
	pp gpp (For_update_item _1) = fsep [gpp "OF", gpp _1]
 
instance PP For_update_items where
	pp gpp (For_update_items _0) = fsep [gppListSep gpp "," _0]
 
instance PP Format where
	pp gpp (Format _0) = fsep [gpp _0]
 
instance PP From_location where
	pp gpp (From_location _1) = fsep [gpp "FROM", gpp _1]
 
instance PP Function_body_declaration where
	pp gpp (Function_body_declaration_c1 _0 _1 _2 _3 _4)
	  = fsep [gpp _0, gpp _1, gppMaybe gpp _2, gpp _3, gpp _4]
	pp gpp (Function_body_declaration_c2 _0 _1 _3 _4 _5)
	  = fsep
	      [gpp _0, gpp _1, gpp "BEGIN", gppMaybe gpp _3, gpp _4, gpp _5]
	pp gpp (Function_body_declaration_c3 _0 _1 _5)
	  = fsep
	      [gpp _0, gpp _1, gpp "LANGUAGE", gpp "JAVA", gpp "NAME", gpp _5]
 
instance PP Function_call where
	pp gpp (Function_call_c1 _0 _1) = fsep [gpp _0, gpp _1]
	pp gpp (Function_call_c2)
	  = fsep [gpp "COUNT", gpp "(", gpp "*", gpp ")"]
	pp gpp (Function_call_c3 _1) = fsep [gpp "COUNT", gpp _1]
 
instance PP Function_header_declaration where
	pp gpp (Function_header_declaration _1 _2 _4)
	  = fsep
	      [gpp "FUNCTION", gpp _1, gppMaybe gpp _2, gpp "RETURN", gpp _4]
 
instance PP Function_name where
	pp gpp (Function_name _0) = fsep [gpp _0]
 
instance PP Goto_statement where
	pp gpp (Goto_statement _1) = fsep [gpp "GOTO", gpp _1]
 
instance PP Grant_item where
	pp gpp (Grant_item_c1 _0) = fsep [gpp _0]
	pp gpp (Grant_item_c2) = fsep [gpp ","]
	pp gpp (Grant_item_c3 _0) = fsep [gpp _0]
 
instance PP Grant_statement where
	pp gpp (Grant_statement _1 _2 _4 _5)
	  = fsep
	      [gpp "GRANT", gppList gpp _1, gppMaybe gpp _2, gpp "TO", gpp _4,
	       gppMaybe gpp _5]
 
instance PP Grantee where
	pp gpp (Grantee_c1 _0) = fsep [gpp _0]
	pp gpp (Grantee_c2 _0) = fsep [gpp _0]
 
instance PP Grantees where
	pp gpp (Grantees _0) = fsep [gppListSep gpp "," _0]
 
instance PP Group_by_clause where
	pp gpp (Group_by_clause _2 _3)
	  = fsep [gpp "GROUP", gpp "BY", gpp _2, gppMaybe gpp _3]
 
instance PP Group_integer where
	pp gpp (Group_integer _1) = fsep [gpp "GROUP", gpp _1]
 
instance PP Group_or_filenames where
	pp gpp (Group_or_filenames_c1 _0) = fsep [gpp _0]
	pp gpp (Group_or_filenames_c2 _0) = fsep [gpp _0]
 
instance PP Group_or_filenames_list where
	pp gpp (Group_or_filenames_list _0) = fsep [gppListSep gpp "," _0]
 
instance PP Grouped_filespec where
	pp gpp (Grouped_filespec _0 _1) = fsep [gppMaybe gpp _0, gpp _1]
 
instance PP Grouped_filespecs where
	pp gpp (Grouped_filespecs _0) = fsep [gppListSep gpp "," _0]
 
instance PP Having_clause where
	pp gpp (Having_clause _1) = fsep [gpp "HAVING", gpp _1]
 
instance PP Host_language_statement where
	pp gpp (Host_language_statement _2)
	  = fsep [gpp "EXEC", gpp "SQL", gpp _2, gpp "END-EXEC"]
 
instance PP Host_name where
	pp gpp (Host_name _0) = fsep [gpp _0]
 
instance PP If_statement where
	pp gpp (If_statement _1 _2 _3 _4)
	  = fsep
	      [gpp "IF", gpp _1, gpp _2, gppList gpp _3, gppMaybe gpp _4,
	       gpp "END", gpp "IF"]
 
instance PP Include_statement where
	pp gpp (Include_statement _1) = fsep [gpp "INCLUDE", gpp _1]
 
instance PP Index_on_clause where
	pp gpp (Index_on_clause_c1 _0 _2)
	  = fsep [gpp _0, gpp "(", gpp _2, gpp ")"]
	pp gpp (Index_on_clause_c2 _1) = fsep [gpp "CLUSTER", gpp _1]
 
instance PP Index_parameter where
	pp gpp (Index_parameter_c1 _1) = fsep [gpp "PCTFREE", gpp _1]
	pp gpp (Index_parameter_c2 _1) = fsep [gpp "INITRANS", gpp _1]
	pp gpp (Index_parameter_c3 _1) = fsep [gpp "MAXTRANS", gpp _1]
	pp gpp (Index_parameter_c4 _0) = fsep [gpp _0]
	pp gpp (Index_parameter_c5 _0) = fsep [gpp _0]
	pp gpp (Index_parameter_c6) = fsep [gpp "NOSORT"]
	pp gpp (Index_parameter_c7) = fsep [gpp "RECOVERABLE"]
	pp gpp (Index_parameter_c8) = fsep [gpp "UNRECOVERABLE"]
	pp gpp (Index_parameter_c9 _1) = fsep [gpp "PCTUSED", gpp _1]
	pp gpp (Index_parameter_c10)
	  = fsep [gpp "ORGANIZATION", gpp "INDEX", gpp "NOCOMPRESS"]
 
instance PP Index_parameters where
	pp gpp (Index_parameters _0) = fsep [gppList gpp _0]
 
instance PP Init_value where
	pp gpp (Init_value _0 _1) = fsep [gpp _0, gpp _1]
 
instance PP Insert_statement where
	pp gpp (Insert_statement_c1 _1 _2 _3)
	  = fsep [gpp "INSERT", gpp _1, gpp _2, gppMaybe gpp _3]
	pp gpp (Insert_statement_c2 _1 _2 _3 _4)
	  = fsep [gpp "INSERT", gpp _1, gpp _2, gppMaybe gpp _3, gpp _4]
 
instance PP SqlInteger where
	pp gpp (Integer _0) = fsep [gpp _0]
 
instance PP Integer_or_default where
	pp gpp (Integer_or_default_c1 _0) = fsep [gpp _0]
	pp gpp (Integer_or_default_c2) = fsep [gpp "DEFAULT"]
 
instance PP Integer_or_other where
	pp gpp (Integer_or_other_c1 _0) = fsep [gpp _0]
	pp gpp (Integer_or_other_c2) = fsep [gpp "UNLIMITED"]
	pp gpp (Integer_or_other_c3) = fsep [gpp "DEFAULT"]
 
instance PP Integer_or_unlimited where
	pp gpp (Integer_or_unlimited_c1 _0) = fsep [gpp _0]
	pp gpp (Integer_or_unlimited_c2) = fsep [gpp "UNLIMITED"]
 
instance PP Integer_text_var where
	pp gpp (Integer_text_var_c1 _0) = fsep [gpp _0]
	pp gpp (Integer_text_var_c2 _0) = fsep [gpp _0]
	pp gpp (Integer_text_var_c3 _0) = fsep [gpp _0]
	pp gpp (Integer_text_var_c4 _0) = fsep [gpp _0]
 
instance PP Into where
	pp gpp (Into _0) = fsep [gppMaybe gpp _0, gpp "INTO"]
 
instance PP Into_qualified_name where
	pp gpp (Into_qualified_name _0 _1) = fsep [gpp _0, gpp _1]
 
instance PP Into_qualified_names where
	pp gpp (Into_qualified_names _0 _1) = fsep [gpp _0, gpp _1]
 
instance PP Isolation_level where
	pp gpp (Isolation_level_c1) = fsep [gpp "READ", gpp "COMMITTED"]
	pp gpp (Isolation_level_c2) = fsep [gpp "SERIALIZABLE"]
 
instance PP Isolation_level_clause where
	pp gpp (Isolation_level_clause _2)
	  = fsep [gpp "ISOLATION", gpp "LEVEL", gpp _2]
 
instance PP Keep_clause where
	pp gpp (Keep_clause _1) = fsep [gpp "KEEP", gpp _1]
 
instance PP Keyword_except_to_on_from where
	pp gpp (Keyword_except_to_on_from _0) = fsep [gpp _0]
 
instance PP Kw_all_triggers where
	pp gpp (Kw_all_triggers) = fsep [gpp "ALL", gpp "TRIGGERS"]
 
instance PP Kw_any_some_all where
	pp gpp (Kw_any_some_all_c1) = fsep [gpp "ANY"]
	pp gpp (Kw_any_some_all_c2) = fsep [gpp "SOME"]
	pp gpp (Kw_any_some_all_c3) = fsep [gpp "ALL"]
 
instance PP Kw_append where
	pp gpp (Kw_append) = fsep [gpp "APPEND"]
 
instance PP Kw_as where
	pp gpp (Kw_as) = fsep [gpp "AS"]
 
instance PP Kw_assignment_default where
	pp gpp (Kw_assignment_default_c1 _0) = fsep [gpp _0]
	pp gpp (Kw_assignment_default_c2) = fsep [gpp "DEFAULT"]
 
instance PP Kw_automatic where
	pp gpp (Kw_automatic) = fsep [gpp "AUTOMATIC"]
 
instance PP Kw_before_after_instead_of where
	pp gpp (Kw_before_after_instead_of_c1) = fsep [gpp "BEFORE"]
	pp gpp (Kw_before_after_instead_of_c2) = fsep [gpp "AFTER"]
	pp gpp (Kw_before_after_instead_of_c3)
	  = fsep [gpp "INSTEAD", gpp "OF"]
 
instance PP Kw_bitmap where
	pp gpp (Kw_bitmap) = fsep [gpp "BITMAP"]
 
instance PP Kw_cache_nocache where
	pp gpp (Kw_cache_nocache_c1) = fsep [gpp "CACHE"]
	pp gpp (Kw_cache_nocache_c2) = fsep [gpp "NOCACHE"]
 
instance PP Kw_cascade where
	pp gpp (Kw_cascade) = fsep [gpp "CASCADE"]
 
instance PP Kw_cascade_constraints where
	pp gpp (Kw_cascade_constraints)
	  = fsep [gpp "CASCADE", gpp "CONSTRAINTS"]
 
instance PP Kw_commit_rollback_nothing where
	pp gpp (Kw_commit_rollback_nothing_c1) = fsep [gpp "COMMIT"]
	pp gpp (Kw_commit_rollback_nothing_c2) = fsep [gpp "ROLLBACK"]
	pp gpp (Kw_commit_rollback_nothing_c3) = fsep [gpp "NOTHING"]
 
instance PP Kw_default where
	pp gpp (Kw_default) = fsep [gpp "DEFAULT"]
 
instance PP Kw_desc_asc where
	pp gpp (Kw_desc_asc_c1) = fsep [gpp "DESC"]
	pp gpp (Kw_desc_asc_c2) = fsep [gpp "ASC"]
 
instance PP Kw_distinct_all where
	pp gpp (Kw_distinct_all_c1) = fsep [gpp "DISTINCT"]
	pp gpp (Kw_distinct_all_c2) = fsep [gpp "ALL"]
 
instance PP Kw_drop where
	pp gpp (Kw_drop) = fsep [gpp "DROP"]
 
instance PP Kw_enable_disable where
	pp gpp (Kw_enable_disable_c1) = fsep [gpp "ENABLE"]
	pp gpp (Kw_enable_disable_c2) = fsep [gpp "DISABLE"]
 
instance PP Kw_enable_disable_compile where
	pp gpp (Kw_enable_disable_compile_c1) = fsep [gpp "ENABLE"]
	pp gpp (Kw_enable_disable_compile_c2) = fsep [gpp "DISABLE"]
	pp gpp (Kw_enable_disable_compile_c3) = fsep [gpp "COMPILE"]
 
instance PP Kw_exclusive_parallel where
	pp gpp (Kw_exclusive_parallel_c1) = fsep [gpp "EXCLUSIVE"]
	pp gpp (Kw_exclusive_parallel_c2) = fsep [gpp "PARALLEL"]
 
instance PP Kw_external where
	pp gpp (Kw_external) = fsep [gpp "EXTERNAL"]
 
instance PP Kw_fast_complete_force where
	pp gpp (Kw_fast_complete_force_c1) = fsep [gpp "FAST"]
	pp gpp (Kw_fast_complete_force_c2) = fsep [gpp "COMPLETE"]
	pp gpp (Kw_fast_complete_force_c3) = fsep [gpp "FORCE"]
 
instance PP Kw_file_datafile where
	pp gpp (Kw_file_datafile_c1) = fsep [gpp "FILE"]
	pp gpp (Kw_file_datafile_c2) = fsep [gpp "DATAFILE"]
 
instance PP Kw_force_no_force where
	pp gpp (Kw_force_no_force_c1) = fsep [gpp "FORCE"]
	pp gpp (Kw_force_no_force_c2) = fsep [gpp "NO", gpp "FORCE"]
 
instance PP Kw_from where
	pp gpp (Kw_from) = fsep [gpp "FROM"]
 
instance PP Kw_global_local where
	pp gpp (Kw_global_local_c1) = fsep [gpp "GLOBAL"]
	pp gpp (Kw_global_local_c2) = fsep [gpp "LOCAL"]
 
instance PP Kw_immediate where
	pp gpp (Kw_immediate) = fsep [gpp "IMMEDIATE"]
 
instance PP Kw_in_out_nocopy where
	pp gpp (Kw_in_out_nocopy_c1) = fsep [gpp "IN"]
	pp gpp (Kw_in_out_nocopy_c2) = fsep [gpp "OUT"]
	pp gpp (Kw_in_out_nocopy_c3) = fsep [gpp "NOCOPY"]
 
instance PP Kw_into where
	pp gpp (Kw_into) = fsep [gpp "INTO"]
 
instance PP Kw_is_as where
	pp gpp (Kw_is_as_c1) = fsep [gpp "IS"]
	pp gpp (Kw_is_as_c2) = fsep [gpp "AS"]
 
instance PP Kw_local where
	pp gpp (Kw_local) = fsep [gpp "LOCAL"]
 
instance PP Kw_no_resetlogs where
	pp gpp (Kw_no_resetlogs_c1) = fsep [gpp "RESETLOGS"]
	pp gpp (Kw_no_resetlogs_c2) = fsep [gpp "NORESETLOGS"]
 
instance PP Kw_nocache_cache where
	pp gpp (Kw_nocache_cache_c1) = fsep [gpp "NOCACHE"]
	pp gpp (Kw_nocache_cache_c2) = fsep [gpp "CACHE"]
 
instance PP Kw_nologging where
	pp gpp (Kw_nologging) = fsep [gpp "NOLOGGING"]
 
instance PP Kw_normal_temporary_immediate where
	pp gpp (Kw_normal_temporary_immediate_c1) = fsep [gpp "NORMAL"]
	pp gpp (Kw_normal_temporary_immediate_c2) = fsep [gpp "TEMPORARY"]
	pp gpp (Kw_normal_temporary_immediate_c3) = fsep [gpp "IMMEDIATE"]
 
instance PP Kw_not where
	pp gpp (Kw_not) = fsep [gpp "NOT"]
 
instance PP Kw_not_null where
	pp gpp (Kw_not_null) = fsep [gpp "NOT", gpp "NULL"]
 
instance PP Kw_nowait where
	pp gpp (Kw_nowait) = fsep [gpp "NOWAIT"]
 
instance PP Kw_old_new where
	pp gpp (Kw_old_new_c1) = fsep [gpp "OLD"]
	pp gpp (Kw_old_new_c2) = fsep [gpp "NEW"]
 
instance PP Kw_on where
	pp gpp (Kw_on) = fsep [gpp "ON"]
 
instance PP Kw_on_delete_cascade where
	pp gpp (Kw_on_delete_cascade)
	  = fsep [gpp "ON", gpp "DELETE", gpp "CASCADE"]
 
instance PP Kw_on_off where
	pp gpp (Kw_on_off_c1) = fsep [gpp "ON"]
	pp gpp (Kw_on_off_c2) = fsep [gpp "OFF"]
 
instance PP Kw_or_replace where
	pp gpp (Kw_or_replace) = fsep [gpp "OR", gpp "REPLACE"]
 
instance PP Kw_package_body where
	pp gpp (Kw_package_body_c1) = fsep [gpp "PACKAGE"]
	pp gpp (Kw_package_body_c2) = fsep [gpp "BODY"]
 
instance PP Kw_public where
	pp gpp (Kw_public) = fsep [gpp "PUBLIC"]
 
instance PP Kw_reuse where
	pp gpp (Kw_reuse) = fsep [gpp "REUSE"]
 
instance PP Kw_reverse where
	pp gpp (Kw_reverse) = fsep [gpp "REVERSE"]
 
instance PP Kw_savepoint where
	pp gpp (Kw_savepoint) = fsep [gpp "SAVEPOINT"]
 
instance PP Kw_semicolon where
	pp gpp (Kw_semicolon) = fsep [gpp ";"]
 
instance PP Kw_sql_trace_global_names where
	pp gpp (Kw_sql_trace_global_names_c1) = fsep [gpp "SQL_TRACE"]
	pp gpp (Kw_sql_trace_global_names_c2) = fsep [gpp "GLOBAL_NAMES"]
 
instance PP Kw_standby_database where
	pp gpp (Kw_standby_database _0)
	  = fsep [gppMaybe gpp _0, gpp "DATABASE"]
 
instance PP Kw_table_cluster where
	pp gpp (Kw_table_cluster_c1) = fsep [gpp "TABLE"]
	pp gpp (Kw_table_cluster_c2) = fsep [gpp "CLUSTER"]
 
instance PP Kw_timestamp_signature where
	pp gpp (Kw_timestamp_signature_c1) = fsep [gpp "TIMESTAMP"]
	pp gpp (Kw_timestamp_signature_c2) = fsep [gpp "SIGNATURE"]
 
instance PP Kw_to where
	pp gpp (Kw_to) = fsep [gpp "TO"]
 
instance PP Kw_true_false where
	pp gpp (Kw_true_false_c1) = fsep [gpp "TRUE"]
	pp gpp (Kw_true_false_c2) = fsep [gpp "FALSE"]
 
instance PP Kw_unarchived where
	pp gpp (Kw_unarchived) = fsep [gpp "UNARCHIVED"]
 
instance PP Kw_unique where
	pp gpp (Kw_unique) = fsep [gpp "UNIQUE"]
 
instance PP Kw_unique_primary_key where
	pp gpp (Kw_unique_primary_key_c1) = fsep [gpp "UNIQUE"]
	pp gpp (Kw_unique_primary_key_c2) = fsep [gpp "PRIMARY", gpp "KEY"]
 
instance PP Kw_unrecoverable_datafile where
	pp gpp (Kw_unrecoverable_datafile)
	  = fsep [gpp "UNRECOVERABLE", gpp "DATAFILE"]
 
instance PP Kw_with_admin_option where
	pp gpp (Kw_with_admin_option)
	  = fsep [gpp "WITH", gpp "ADMIN", gpp "OPTION"]
 
instance PP Kw_work where
	pp gpp (Kw_work) = fsep [gpp "WORK"]
 
instance PP Label_header where
	pp gpp (Label_header _1) = fsep [gpp "<<", gpp _1, gpp ">>"]
 
instance PP Label_name where
	pp gpp (Label_name _0) = fsep [gpp _0]
 
instance PP Language where
	pp gpp (Language _0) = fsep [gpp _0]
 
instance PP Left_right where
	pp gpp (Left_right_c1) = fsep [gpp "RIGHT"]
	pp gpp (Left_right_c2) = fsep [gpp "R"]
	pp gpp (Left_right_c3) = fsep [gpp "LEFT"]
	pp gpp (Left_right_c4) = fsep [gpp "L"]
 
instance PP Lhs where
	pp gpp (Lhs_c1 _0) = fsep [gpp _0]
	pp gpp (Lhs_c2 _0 _2) = fsep [gpp _0, gpp "(", gpp _2, gpp ")"]
 
instance PP Load_data_clause where
	pp gpp (Load_data_clause _0 _1 _4 _5 _6 _7 _8)
	  = fsep
	      [gppList gpp _0, gppMaybe gpp _1, gpp "INTO", gpp "TABLE", gpp _4,
	       gppMaybe gpp _5, gppMaybe gpp _6, gppMaybe gpp _7, gppMaybe gpp _8]
 
instance PP Load_data_option where
	pp gpp (Load_data_option _0 _2) = fsep [gpp _0, gpp "=", gpp _2]
 
instance PP Load_data_options where
	pp gpp (Load_data_options _0) = fsep [gppListSep gpp "," _0]
 
instance PP Load_data_parameter where
	pp gpp (Load_data_parameter_c1) = fsep [gpp "INFILE"]
	pp gpp (Load_data_parameter_c2) = fsep [gpp "BADFILE"]
	pp gpp (Load_data_parameter_c3) = fsep [gpp "DISCARDFILE"]
	pp gpp (Load_data_parameter_c4) = fsep [gpp "APPEND", gpp "RECLEN"]
	pp gpp (Load_data_parameter_c5)
	  = fsep [gpp "REPLACE", gpp "RECLEN"]
 
instance PP Load_data_parameter_value where
	pp gpp (Load_data_parameter_value _0 _1) = fsep [gpp _0, gpp _1]
 
instance PP Load_data_statement where
	pp gpp (Load_data_statement _2)
	  = fsep [gpp "LOAD", gpp "DATA", gppList gpp _2]
 
instance PP Load_data_value where
	pp gpp (Load_data_value_c1 _0) = fsep [gpp _0]
	pp gpp (Load_data_value_c2 _0) = fsep [gpp _0]
 
instance PP Lock_mode where
	pp gpp (Lock_mode_c1) = fsep [gpp "ROW", gpp "SHARE"]
	pp gpp (Lock_mode_c2) = fsep [gpp "ROW", gpp "EXCLUSIVE"]
	pp gpp (Lock_mode_c3) = fsep [gpp "SHARE", gpp "UPDATE"]
	pp gpp (Lock_mode_c4) = fsep [gpp "SHARE"]
	pp gpp (Lock_mode_c5)
	  = fsep [gpp "SHARE", gpp "ROW", gpp "EXCLUSIVE"]
	pp gpp (Lock_mode_c6) = fsep [gpp "EXCLUSIVE"]
 
instance PP Lock_table_statement where
	pp gpp (Lock_table_statement _2 _4 _6)
	  = fsep
	      [gpp "LOCK", gpp "TABLE", gpp _2, gpp "IN", gpp _4, gpp "MODE",
	       gppMaybe gpp _6]
 
instance PP Logfile_member where
	pp gpp (Logfile_member _0 _2) = fsep [gpp _0, gpp "TO", gpp _2]
 
instance PP Logfile_members where
	pp gpp (Logfile_members _0) = fsep [gppListSep gpp "," _0]
 
instance PP Loop_statement where
	pp gpp (Loop_statement_c1 _0) = fsep [gpp _0]
	pp gpp (Loop_statement_c2 _0) = fsep [gpp _0]
	pp gpp (Loop_statement_c3 _0) = fsep [gpp _0]
 
instance PP Max_memory_size where
	pp gpp (Max_memory_size_c1) = fsep [gpp "MAXSIZE", gpp "UNLIMITED"]
	pp gpp (Max_memory_size_c2 _1) = fsep [gpp "MAXSIZE", gpp _1]
 
instance PP Memory_size where
	pp gpp (Memory_size _0) = fsep [gpp _0]
 
instance PP Memory_size_clause where
	pp gpp (Memory_size_clause _1) = fsep [gpp "SIZE", gpp _1]
 
instance PP Memory_size_or_null where
	pp gpp (Memory_size_or_null_c1 _0) = fsep [gpp _0]
	pp gpp (Memory_size_or_null_c2) = fsep [gpp "NULL"]
 
instance PP Memory_size_or_other where
	pp gpp (Memory_size_or_other_c1 _0) = fsep [gpp _0]
	pp gpp (Memory_size_or_other_c2) = fsep [gpp "UNLIMITED"]
	pp gpp (Memory_size_or_other_c3) = fsep [gpp "DEFAULT"]
 
instance PP Memory_size_or_unlimited where
	pp gpp (Memory_size_or_unlimited_c1 _0) = fsep [gpp _0]
	pp gpp (Memory_size_or_unlimited_c2) = fsep [gpp "UNLIMITED"]
 
instance PP Meta_parameter where
	pp gpp (Meta_parameter_c1) = fsep [gpp "ARRAYSIZE"]
	pp gpp (Meta_parameter_c2) = fsep [gpp "ECHO"]
	pp gpp (Meta_parameter_c3) = fsep [gpp "ESCAPE"]
	pp gpp (Meta_parameter_c4) = fsep [gpp "EMBEDDED"]
	pp gpp (Meta_parameter_c5) = fsep [gpp "FEEDBACK"]
	pp gpp (Meta_parameter_c6) = fsep [gpp "HEADING"]
	pp gpp (Meta_parameter_c7) = fsep [gpp "LINES"]
	pp gpp (Meta_parameter_c8) = fsep [gpp "LINESIZE"]
	pp gpp (Meta_parameter_c9) = fsep [gpp "NEWPAGE"]
	pp gpp (Meta_parameter_c10) = fsep [gpp "NULL"]
	pp gpp (Meta_parameter_c11) = fsep [gpp "NUMW"]
	pp gpp (Meta_parameter_c12) = fsep [gpp "NUMWIDTH"]
	pp gpp (Meta_parameter_c13) = fsep [gpp "PAGESIZE"]
	pp gpp (Meta_parameter_c14) = fsep [gpp "PAUSE"]
	pp gpp (Meta_parameter_c15) = fsep [gpp "RECSEP"]
	pp gpp (Meta_parameter_c16) = fsep [gpp "SERVEROUTPUT"]
	pp gpp (Meta_parameter_c17) = fsep [gpp "SPACE"]
	pp gpp (Meta_parameter_c18) = fsep [gpp "TERM"]
	pp gpp (Meta_parameter_c19) = fsep [gpp "TERMOUT"]
	pp gpp (Meta_parameter_c20) = fsep [gpp "TIME"]
	pp gpp (Meta_parameter_c21) = fsep [gpp "TIMING"]
	pp gpp (Meta_parameter_c22) = fsep [gpp "TRIMSPOOL"]
	pp gpp (Meta_parameter_c23) = fsep [gpp "VERIFY"]
	pp gpp (Meta_parameter_c24) = fsep [gpp "WRAP"]
 
instance PP Meta_parameter_value where
	pp gpp (Meta_parameter_value _0 _1) = fsep [gpp _0, gpp _1]
 
instance PP Meta_parameter_values where
	pp gpp (Meta_parameter_values _0) = fsep [gppList gpp _0]
 
instance PP Meta_statement where
	pp gpp (Meta_statement_c1 _0) = fsep [gpp _0]
	pp gpp (Meta_statement_c2 _0) = fsep [gpp _0]
	pp gpp (Meta_statement_c3 _0) = fsep [gpp _0]
	pp gpp (Meta_statement_c4 _0) = fsep [gpp _0]
	pp gpp (Meta_statement_c5 _0) = fsep [gpp _0]
	pp gpp (Meta_statement_c6 _0) = fsep [gpp _0]
	pp gpp (Meta_statement_c7 _0) = fsep [gpp _0]
	pp gpp (Meta_statement_c8 _0) = fsep [gpp _0]
 
instance PP Meta_statement_semicolon where
	pp gpp (Meta_statement_semicolon _0 _1)
	  = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Name where
	pp gpp (Name_c1 _0) = fsep [gpp _0]
	pp gpp (Name_c2) = fsep [gpp "CONVERT"]
	pp gpp (Name_c3) = fsep [gpp "PROFILE"]
	pp gpp (Name_c4) = fsep [gpp "LANGUAGE"]
	pp gpp (Name_c5) = fsep [gpp "NAME"]
 
instance PP Name_or_keyword where
	pp gpp (Name_or_keyword _0) = fsep [gpp _0]
 
instance PP Names where
	pp gpp (Names _0) = fsep [gppListSep gpp "," _0]
 
instance PP Names_brackets where
	pp gpp (Names_brackets _1) = fsep [gpp "(", gpp _1, gpp ")"]
 
instance PP Next_date where
	pp gpp (Next_date _1) = fsep [gpp "NEXT", gpp _1]
 
instance PP Next_memory_size where
	pp gpp (Next_memory_size _1) = fsep [gpp "NEXT", gpp _1]
 
instance PP Null_statement where
	pp gpp (Null_statement) = fsep [gpp "NULL"]
 
instance PP Number where
	pp gpp (Number _0) = fsep [gpp _0]
 
instance PP Of_columns where
	pp gpp (Of_columns _1) = fsep [gpp "OF", gpp _1]
 
instance PP On_clause where
	pp gpp (On_clause _1) = fsep [gpp "ON", gpp _1]
 
instance PP Open_for_clause where
	pp gpp (Open_for_clause_c1 _0) = fsep [gpp _0]
	pp gpp (Open_for_clause_c2 _0 _2)
	  = fsep [gpp _0, gpp "USING", gpp _2]
 
instance PP Open_for_cursor where
	pp gpp (Open_for_cursor_c1 _0) = fsep [gpp _0]
	pp gpp (Open_for_cursor_c2 _1) = fsep [gpp ":", gpp _1]
 
instance PP Open_for_statement where
	pp gpp (Open_for_statement _1 _3)
	  = fsep [gpp "OPEN", gpp _1, gpp "FOR", gpp _3]
 
instance PP Open_statement where
	pp gpp (Open_statement _1) = fsep [gpp "OPEN", gpp _1]
 
instance PP Or_exception_name where
	pp gpp (Or_exception_name _1) = fsep [gpp "OR", gpp _1]
 
instance PP Order_by_clause where
	pp gpp (Order_by_clause _2) = fsep [gpp "ORDER", gpp "BY", gpp _2]
 
instance PP Order_item where
	pp gpp (Order_item _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Order_items where
	pp gpp (Order_items _0) = fsep [gppListSep gpp "," _0]
 
instance PP Ordered_column where
	pp gpp (Ordered_column _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Ordered_columns where
	pp gpp (Ordered_columns _0) = fsep [gppListSep gpp "," _0]
 
instance PP Package_body where
	pp gpp (Package_body_c1 _0 _1 _2 _3)
	  = fsep [gpp _0, gppMaybe gpp _1, gppMaybe gpp _2, gpp _3]
	pp gpp (Package_body_c2 _0 _1 _3 _4 _5 _6 _7 _8)
	  = fsep
	      [gpp _0, gppMaybe gpp _1, gpp "BEGIN", gppMaybe gpp _3, gpp _4,
	       gppMaybe gpp _5, gpp _6, gppMaybe gpp _7, gppMaybe gpp _8]
 
instance PP Package_body_prefix where
	pp gpp (Package_body_prefix _0 _3 _4)
	  = fsep [gppMaybe gpp _0, gpp "PACKAGE", gpp "BODY", gpp _3, gpp _4]
 
instance PP Package_declaration where
	pp gpp (Package_declaration_c1 _0) = fsep [gpp _0]
	pp gpp (Package_declaration_c2 _0) = fsep [gpp _0]
 
instance PP Package_header where
	pp gpp (Package_header _0 _1 _2)
	  = fsep [gpp _0, gppMaybe gpp _1, gpp _2]
 
instance PP Package_header_prefix where
	pp gpp (Package_header_prefix _0 _2 _3)
	  = fsep [gppMaybe gpp _0, gpp "PACKAGE", gpp _2, gpp _3]
 
instance PP Package_name where
	pp gpp (Package_name _0) = fsep [gpp _0]
 
instance PP Parallel_clause where
	pp gpp (Parallel_clause_c1) = fsep [gpp "NOPARALLEL"]
	pp gpp (Parallel_clause_c2 _2)
	  = fsep [gpp "PARALLEL", gpp "(", gpp _2, gpp ")"]
 
instance PP Parallel_items where
	pp gpp (Parallel_items_c1 _1) = fsep [gpp "DEGREE", gpp _1]
	pp gpp (Parallel_items_c2 _1) = fsep [gpp "INSTANCES", gpp _1]
	pp gpp (Parallel_items_c3 _1 _2)
	  = fsep [gpp "DEGREE", gpp _1, gpp _2]
 
instance PP Parameter where
	pp gpp (Parameter_c1 _0) = fsep [gpp _0]
	pp gpp (Parameter_c2 _0 _2) = fsep [gpp _0, gpp "=>", gpp _2]
 
instance PP Parameter_declaration where
	pp gpp (Parameter_declaration _0 _1 _2 _3)
	  = fsep [gpp _0, gppList gpp _1, gpp _2, gppMaybe gpp _3]
 
instance PP Parameter_declarations where
	pp gpp (Parameter_declarations _0) = fsep [gppListSep gpp "," _0]
 
instance PP Parameter_declarations_brackets where
	pp gpp (Parameter_declarations_brackets _1)
	  = fsep [gpp "(", gpp _1, gpp ")"]
 
instance PP Parameters where
	pp gpp (Parameters _0) = fsep [gppListSep gpp "," _0]
 
instance PP Parameters_brackets where
	pp gpp (Parameters_brackets _1) = fsep [gpp "(", gpp _1, gpp ")"]
 
instance PP Password where
	pp gpp (Password _0) = fsep [gpp _0]
 
instance PP Perc_found where
	pp gpp (Perc_found) = fsep [gpp "%FOUND"]
 
instance PP Perc_isopen where
	pp gpp (Perc_isopen) = fsep [gpp "%ISOPEN"]
 
instance PP Perc_notfound where
	pp gpp (Perc_notfound) = fsep [gpp "%NOTFOUND"]
 
instance PP Perc_rowcount where
	pp gpp (Perc_rowcount) = fsep [gpp "%ROWCOUNT"]
 
instance PP Perc_rowtype where
	pp gpp (Perc_rowtype) = fsep [gpp "%ROWTYPE"]
 
instance PP Perc_type where
	pp gpp (Perc_type) = fsep [gpp "%TYPE"]
 
instance PP Perc_typed_name where
	pp gpp (Perc_typed_name_c1 _0) = fsep [gpp _0]
	pp gpp (Perc_typed_name_c2 _0 _2) = fsep [gpp _0, gpp ".", gpp _2]
	pp gpp (Perc_typed_name_c3 _0 _1) = fsep [gpp _0, gpp _1]
	pp gpp (Perc_typed_name_c4 _0 _2 _3)
	  = fsep [gpp _0, gpp ".", gpp _2, gpp _3]
	pp gpp (Perc_typed_name_c5 _0 _1) = fsep [gpp _0, gpp _1]
	pp gpp (Perc_typed_name_c6 _0 _2 _3)
	  = fsep [gpp _0, gpp ".", gpp _2, gpp _3]
	pp gpp (Perc_typed_name_c7 _0) = fsep [gpp _0]
 
instance PP Permission where
	pp gpp (Permission_c1) = fsep [gpp "READ", gpp "ONLY"]
	pp gpp (Permission_c2) = fsep [gpp "READ", gpp "WRITE"]
 
instance PP Pl_statement where
	pp gpp (Pl_statement _0 _1) = fsep [gppMaybe gpp _0, gpp _1]
 
instance PP Pl_statements where
	pp gpp (Pl_statements_c1 _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
	pp gpp (Pl_statements_c2 _0 _1 _2) = fsep [gpp _0, gpp _1, gpp _2]
 
instance PP Plsql_block where
	pp gpp (Plsql_block _0 _1 _2)
	  = fsep [gppMaybe gpp _0, gppMaybe gpp _1, gpp _2]
 
instance PP Position where
	pp gpp (Position_c1 _2 _4)
	  = fsep [gpp "POSITION", gpp "(", gpp _2, gpp ":", gpp _4, gpp ")"]
	pp gpp (Position_c2 _1) = fsep [gpp "CONSTANT", gpp _1]
 
instance PP Pragma_declaration where
	pp gpp (Pragma_declaration_c1 _3 _5)
	  = fsep
	      [gpp "PRAGMA", gpp "EXCEPTION_INIT", gpp "(", gpp _3, gpp ",",
	       gpp _5, gpp ")"]
	pp gpp (Pragma_declaration_c2 _3 _5)
	  = fsep
	      [gpp "PRAGMA", gpp "RESTRICT_REFERENCES", gpp "(", gpp _3, gpp ",",
	       gpp _5, gpp ")"]
	pp gpp (Pragma_declaration_c3)
	  = fsep [gpp "PRAGMA", gpp "AUTONOMOUS_TRANSACTION"]
 
instance PP Procedure_body_declaration where
	pp gpp (Procedure_body_declaration_c1 _0 _1 _2 _3 _4)
	  = fsep [gpp _0, gpp _1, gppMaybe gpp _2, gpp _3, gpp _4]
	pp gpp (Procedure_body_declaration_c2 _0 _1 _3 _4 _5)
	  = fsep
	      [gpp _0, gpp _1, gpp "BEGIN", gppMaybe gpp _3, gpp _4, gpp _5]
	pp gpp (Procedure_body_declaration_c3 _0 _1 _5)
	  = fsep
	      [gpp _0, gpp _1, gpp "LANGUAGE", gpp "JAVA", gpp "NAME", gpp _5]
 
instance PP Procedure_header_declaration where
	pp gpp (Procedure_header_declaration _1 _2)
	  = fsep [gpp "PROCEDURE", gpp _1, gppMaybe gpp _2]
 
instance PP Profile where
	pp gpp (Profile _0) = fsep [gpp _0]
 
instance PP Qualified_name where
	pp gpp (Qualified_name_c1 _0) = fsep [gpp _0]
	pp gpp (Qualified_name_c2 _0) = fsep [gpp _0]
	pp gpp (Qualified_name_c3 _0 _2) = fsep [gpp _0, gpp ".", gpp _2]
	pp gpp (Qualified_name_c4) = fsep [gpp "CONVERT"]
	pp gpp (Qualified_name_c5) = fsep [gpp "PROFILE"]
	pp gpp (Qualified_name_c6) = fsep [gpp "LANGUAGE"]
	pp gpp (Qualified_name_c7) = fsep [gpp "NAME"]
	pp gpp (Qualified_name_c8 _0)
	  = fsep [gpp _0, gpp ".", gpp "EXECUTE"]
 
instance PP Qualified_names where
	pp gpp (Qualified_names _0) = fsep [gppListSep gpp "," _0]
 
instance PP Quota where
	pp gpp (Quota _1 _3) = fsep [gpp "QUOTA", gpp _1, gpp "ON", gpp _3]
 
instance PP Quotas where
	pp gpp (Quotas _0) = fsep [gppList gpp _0]
 
instance PP Raise_statement where
	pp gpp (Raise_statement _1) = fsep [gpp "RAISE", gppMaybe gpp _1]
 
instance PP Rebuild_item where
	pp gpp (Rebuild_item_c1 _1) = fsep [gpp "PARALLEL", gpp _1]
	pp gpp (Rebuild_item_c2) = fsep [gpp "NOPARALLEL"]
	pp gpp (Rebuild_item_c3) = fsep [gpp "RECOVERABLE"]
	pp gpp (Rebuild_item_c4) = fsep [gpp "UNRECOVERABLE"]
	pp gpp (Rebuild_item_c5 _0) = fsep [gpp _0]
 
instance PP Rebuild_items where
	pp gpp (Rebuild_items _0) = fsep [gppList gpp _0]
 
instance PP Record_type_definition where
	pp gpp (Record_type_definition _1 _5)
	  = fsep
	      [gpp "TYPE", gpp _1, gpp "IS", gpp "RECORD", gpp "(", gpp _5,
	       gpp ")"]
 
instance PP Recover_clause where
	pp gpp (Recover_clause _1 _2 _3 _4)
	  = fsep
	      [gpp "RECOVER", gppMaybe gpp _1, gppMaybe gpp _2, gppMaybe gpp _3,
	       gppMaybe gpp _4]
 
instance PP Recover_database_item where
	pp gpp (Recover_database_item_c1)
	  = fsep [gpp "UNTIL", gpp "CANCEL"]
	pp gpp (Recover_database_item_c2 _2)
	  = fsep [gpp "UNTIL", gpp "TIME", gpp _2]
	pp gpp (Recover_database_item_c3 _2)
	  = fsep [gpp "UNTIL", gpp "CHANGE", gpp _2]
	pp gpp (Recover_database_item_c4)
	  = fsep [gpp "USING", gpp "BACKUP", gpp "CONTROLFILE"]
 
instance PP Recover_item where
	pp gpp (Recover_item_c1 _0 _1) = fsep [gppMaybe gpp _0, gpp _1]
	pp gpp (Recover_item_c2 _1) = fsep [gpp "TABLESPACE", gpp _1]
	pp gpp (Recover_item_c3 _1) = fsep [gpp "DATABASE", gpp _1]
	pp gpp (Recover_item_c4 _1) = fsep [gpp "LOGFILE", gpp _1]
	pp gpp (Recover_item_c5 _1)
	  = fsep [gpp "CONTINUE", gppMaybe gpp _1]
	pp gpp (Recover_item_c6) = fsep [gpp "CANCEL"]
 
instance PP Ref_type_declaration where
	pp gpp (Ref_type_declaration _1 _5)
	  = fsep
	      [gpp "TYPE", gpp _1, gpp "IS", gpp "REF", gpp "CURSOR",
	       gppMaybe gpp _5]
 
instance PP Referencing where
	pp gpp (Referencing _1) = fsep [gpp "REFERENCING", gpp _1]
 
instance PP Referencing_item where
	pp gpp (Referencing_item _0 _2) = fsep [gpp _0, gpp "AS", gpp _2]
 
instance PP Referencing_items where
	pp gpp (Referencing_items _0) = fsep [gppList gpp _0]
 
instance PP Relational_operator where
	pp gpp (Relational_operator_c1) = fsep [gpp "<"]
	pp gpp (Relational_operator_c2) = fsep [gpp "<="]
	pp gpp (Relational_operator_c3) = fsep [gpp ">"]
	pp gpp (Relational_operator_c4) = fsep [gpp ">="]
	pp gpp (Relational_operator_c5) = fsep [gpp "="]
	pp gpp (Relational_operator_c6) = fsep [gpp "<>"]
	pp gpp (Relational_operator_c7) = fsep [gpp "!="]
 
instance PP Rename_statement where
	pp gpp (Rename_statement _1 _3)
	  = fsep [gpp "RENAME", gpp _1, gpp "TO", gpp _3]
 
instance PP Resource_cost_item where
	pp gpp (Resource_cost_item_c1 _1)
	  = fsep [gpp "CPU_PER_SESSION", gpp _1]
	pp gpp (Resource_cost_item_c2 _1)
	  = fsep [gpp "CONNECT_TIME", gpp _1]
	pp gpp (Resource_cost_item_c3 _1)
	  = fsep [gpp "LOGICAL_READS_PER_SECOND", gpp _1]
	pp gpp (Resource_cost_item_c4 _1)
	  = fsep [gpp "PRIVATE_SGA", gpp _1]
 
instance PP Resource_cost_items where
	pp gpp (Resource_cost_items _0) = fsep [gppList gpp _0]
 
instance PP Return_statement where
	pp gpp (Return_statement _1) = fsep [gpp "RETURN", gppMaybe gpp _1]
 
instance PP Return_type where
	pp gpp (Return_type _0) = fsep [gpp _0]
 
instance PP Reused_filename where
	pp gpp (Reused_filename _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Reused_filenames where
	pp gpp (Reused_filenames _0) = fsep [gppListSep gpp "," _0]
 
instance PP Revoke_statement where
	pp gpp (Revoke_statement _1 _2 _4 _5)
	  = fsep
	      [gpp "REVOKE", gppList gpp _1, gppMaybe gpp _2, gpp "FROM", gpp _4,
	       gppMaybe gpp _5]
 
instance PP Role where
	pp gpp (Role _0) = fsep [gpp _0]
 
instance PP Role_description where
	pp gpp (Role_description_c1 _0) = fsep [gpp _0]
	pp gpp (Role_description_c2) = fsep [gpp "ALL"]
	pp gpp (Role_description_c3 _2)
	  = fsep [gpp "ALL", gpp "EXCEPT", gpp _2]
	pp gpp (Role_description_c4) = fsep [gpp "NONE"]
 
instance PP Role_item where
	pp gpp (Role_item_c1) = fsep [gpp "NOT", gpp "IDENTIFIED"]
	pp gpp (Role_item_c2 _1) = fsep [gpp "IDENTIFIED", gpp _1]
 
instance PP Roles where
	pp gpp (Roles _0) = fsep [gppList gpp _0]
 
instance PP Rollback_segment_item where
	pp gpp (Rollback_segment_item_c1) = fsep [gpp "ONLINE"]
	pp gpp (Rollback_segment_item_c2) = fsep [gpp "OFFLINE"]
	pp gpp (Rollback_segment_item_c3 _0) = fsep [gpp _0]
	pp gpp (Rollback_segment_item_c4 _1 _2)
	  = fsep [gpp "SHRINK", gppMaybe gpp _1, gppMaybe gpp _2]
 
instance PP Rollback_statement where
	pp gpp (Rollback_statement _1 _2)
	  = fsep [gpp "ROLLBACK", gppMaybe gpp _1, gppMaybe gpp _2]
 
instance PP Row_clause where
	pp gpp (Row_clause _0 _4)
	  = fsep
	      [gppMaybe gpp _0, gpp "FOR", gpp "EACH", gpp "ROW",
	       gppMaybe gpp _4]
 
instance PP Savepoint where
	pp gpp (Savepoint_c1 _1 _2)
	  = fsep [gpp "TO", gppMaybe gpp _1, gpp _2]
	pp gpp (Savepoint_c2 _1) = fsep [gpp "FORCE", gpp _1]
	pp gpp (Savepoint_c3) = fsep [gpp "RELEASE"]
 
instance PP Savepoint_statement where
	pp gpp (Savepoint_statement _1) = fsep [gpp "SAVEPOINT", gpp _1]
 
instance PP Schema_qualifier where
	pp gpp (Schema_qualifier _0) = fsep [gpp _0]
 
instance PP Select_clause where
	pp gpp (Select_clause_c1 _0) = fsep [gpp _0]
	pp gpp (Select_clause_c2 _0) = fsep [gpp _0]
	pp gpp (Select_clause_c3 _0) = fsep [gpp _0]
	pp gpp (Select_clause_c4 _0) = fsep [gpp _0]
	pp gpp (Select_clause_c5 _0) = fsep [gpp _0]
 
instance PP Select_clause_operator where
	pp gpp (Select_clause_operator_c1) = fsep [gpp "UNION"]
	pp gpp (Select_clause_operator_c2) = fsep [gpp "UNION", gpp "ALL"]
	pp gpp (Select_clause_operator_c3) = fsep [gpp "INTERSECT"]
	pp gpp (Select_clause_operator_c4) = fsep [gpp "MINUS"]
 
instance PP Select_item where
	pp gpp (Select_item_c1 _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
	pp gpp (Select_item_c2 _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
	pp gpp (Select_item_c3 _0) = fsep [gpp _0]
 
instance PP Select_statement where
	pp gpp (Select_statement _1 _2 _3 _5 _6)
	  = fsep
	      [gpp "SELECT", gppMaybe gpp _1, gpp _2, gppMaybe gpp _3,
	       gpp "FROM", gpp _5, gppList gpp _6]
 
instance PP Select_statement_without_into where
	pp gpp (Select_statement_without_into_c1 _1 _2 _4 _5)
	  = fsep
	      [gpp "SELECT", gppMaybe gpp _1, gpp _2, gpp "FROM", gpp _4,
	       gppList gpp _5]
	pp gpp (Select_statement_without_into_c2 _0 _1 _2)
	  = fsep [gpp _0, gpp _1, gpp _2]
 
instance PP Sequence_item where
	pp gpp (Sequence_item_c1 _2)
	  = fsep [gpp "INCREMENT", gpp "BY", gpp _2]
	pp gpp (Sequence_item_c2 _2)
	  = fsep [gpp "START", gpp "WITH", gpp _2]
	pp gpp (Sequence_item_c3 _1) = fsep [gpp "MAXVALUE", gpp _1]
	pp gpp (Sequence_item_c4) = fsep [gpp "NOMAXVALUE"]
	pp gpp (Sequence_item_c5 _1) = fsep [gpp "MINVALUE", gpp _1]
	pp gpp (Sequence_item_c6) = fsep [gpp "NOMINVALUE"]
	pp gpp (Sequence_item_c7) = fsep [gpp "CYCLE"]
	pp gpp (Sequence_item_c8) = fsep [gpp "NOCYCLE"]
	pp gpp (Sequence_item_c9 _1) = fsep [gpp "CACHE", gpp _1]
	pp gpp (Sequence_item_c10) = fsep [gpp "NOCACHE"]
	pp gpp (Sequence_item_c11) = fsep [gpp "ORDER"]
	pp gpp (Sequence_item_c12) = fsep [gpp "NOORDER"]
 
instance PP Sequence_items where
	pp gpp (Sequence_items _0) = fsep [gppList gpp _0]
 
instance PP Session_set_item where
	pp gpp (Session_set_item_c1 _0 _2) = fsep [gpp _0, gpp "=", gpp _2]
	pp gpp (Session_set_item_c2 _2)
	  = fsep [gpp "NLS_LANGUAGE", gpp "=", gpp _2]
	pp gpp (Session_set_item_c3 _2)
	  = fsep [gpp "NLS_TERRITORY", gpp "=", gpp _2]
	pp gpp (Session_set_item_c4 _2)
	  = fsep [gpp "NLS_DATE_FORMAT", gpp "=", gpp _2]
	pp gpp (Session_set_item_c5 _2)
	  = fsep [gpp "NLS_DATE_LANGUAGE", gpp "=", gpp _2]
	pp gpp (Session_set_item_c6 _2)
	  = fsep [gpp "NLS_NUMERIC_CHARACTERS", gpp "=", gpp _2]
	pp gpp (Session_set_item_c7 _2)
	  = fsep [gpp "NLS_ISO_CURRENCY", gpp "=", gpp _2]
	pp gpp (Session_set_item_c8 _2)
	  = fsep [gpp "NLS_CURRENCY", gpp "=", gpp _2]
	pp gpp (Session_set_item_c9 _2)
	  = fsep [gpp "NLS_SORT", gpp "=", gpp _2]
	pp gpp (Session_set_item_c10 _2)
	  = fsep [gpp "NLS_CALENDAR", gpp "=", gpp _2]
	pp gpp (Session_set_item_c11 _2)
	  = fsep [gpp "LABEL", gpp "=", gpp _2]
	pp gpp (Session_set_item_c12 _2)
	  = fsep [gpp "MLS_LABEL_FORMAT", gpp "=", gpp _2]
	pp gpp (Session_set_item_c13 _2)
	  = fsep [gpp "OPTIMIZER_GOAL", gpp "=", gpp _2]
	pp gpp (Session_set_item_c14 _2)
	  = fsep [gpp "FLAGGER", gpp "=", gpp _2]
	pp gpp (Session_set_item_c15 _2)
	  = fsep [gpp "SESSION_CACHED_OPEN_CURSORS", gpp "=", gpp _2]
	pp gpp (Session_set_item_c16 _2)
	  = fsep [gpp "INSTANCE", gpp "=", gpp _2]
	pp gpp (Session_set_item_c17 _2)
	  = fsep [gpp "HASH_JOIN_ENABLED", gpp "=", gpp _2]
	pp gpp (Session_set_item_c18 _2)
	  = fsep [gpp "HASH_AREA_SIZE", gpp "=", gpp _2]
	pp gpp (Session_set_item_c19 _2)
	  = fsep [gpp "HASH_MULTIBLOCK_IO_COUNT", gpp "=", gpp _2]
	pp gpp (Session_set_item_c20 _2)
	  = fsep [gpp "ISOLATION_LEVEL", gpp "=", gpp _2]
	pp gpp (Session_set_item_c21 _1) = fsep [gpp "EVENTS", gpp _1]
 
instance PP Session_set_items where
	pp gpp (Session_set_items _0) = fsep [gppList gpp _0]
 
instance PP Set_flagger_item where
	pp gpp (Set_flagger_item_c1) = fsep [gpp "ENTRY"]
	pp gpp (Set_flagger_item_c2) = fsep [gpp "INTERMEDIATE"]
	pp gpp (Set_flagger_item_c3) = fsep [gpp "FULL"]
	pp gpp (Set_flagger_item_c4) = fsep [gpp "OFF"]
 
instance PP Set_item where
	pp gpp (Set_item_c1 _0 _2) = fsep [gpp _0, gpp "=", gpp _2]
	pp gpp (Set_item_c2 _0 _2) = fsep [gpp _0, gpp "=", gpp _2]
 
instance PP Set_items where
	pp gpp (Set_items _0) = fsep [gppListSep gpp "," _0]
 
instance PP Set_label_item where
	pp gpp (Set_label_item_c1 _0) = fsep [gpp _0]
	pp gpp (Set_label_item_c2) = fsep [gpp "DBHIGH"]
	pp gpp (Set_label_item_c3) = fsep [gpp "DBLOW"]
 
instance PP Set_optimizer_goal_item where
	pp gpp (Set_optimizer_goal_item_c1) = fsep [gpp "ALL_ROWS"]
	pp gpp (Set_optimizer_goal_item_c2) = fsep [gpp "FIRST_ROWS"]
	pp gpp (Set_optimizer_goal_item_c3) = fsep [gpp "RULE"]
	pp gpp (Set_optimizer_goal_item_c4) = fsep [gpp "CHOOSE"]
 
instance PP Set_statement_id where
	pp gpp (Set_statement_id _2)
	  = fsep [gpp "SET", gpp "STATEMENT-ID", gpp _2]
 
instance PP Set_transaction_clause where
	pp gpp (Set_transaction_clause_c1 _0) = fsep [gpp _0]
	pp gpp (Set_transaction_clause_c2 _0) = fsep [gpp _0]
	pp gpp (Set_transaction_clause_c3 _3)
	  = fsep [gpp "USE", gpp "ROLLBACK", gpp "SEGMENT", gpp _3]
 
instance PP Set_transaction_statement where
	pp gpp (Set_transaction_statement _1 _3)
	  = fsep [gpp "SET", gppMaybe gpp _1, gpp "TRANSACTION", gpp _3]
 
instance PP Snap_shot_item where
	pp gpp (Snap_shot_item_c1 _1) = fsep [gpp "PCTFREE", gpp _1]
	pp gpp (Snap_shot_item_c2 _1) = fsep [gpp "PCTUSED", gpp _1]
	pp gpp (Snap_shot_item_c3 _1) = fsep [gpp "INITRANS", gpp _1]
	pp gpp (Snap_shot_item_c4 _1) = fsep [gpp "MAXTRANS", gpp _1]
	pp gpp (Snap_shot_item_c5 _0) = fsep [gpp _0]
	pp gpp (Snap_shot_item_c6 _0) = fsep [gpp _0]
 
instance PP Snap_shot_items where
	pp gpp (Snap_shot_items _0) = fsep [gppList gpp _0]
 
instance PP Snapshot_refresh where
	pp gpp (Snapshot_refresh _1 _2 _3)
	  = fsep
	      [gpp "REFRESH", gppMaybe gpp _1, gppMaybe gpp _2, gppMaybe gpp _3]
 
instance PP Sql_plus_identifier where
	pp gpp (Sql_plus_identifier _0) = fsep [gpp _0]
 
instance PP Sql_plus_macro where
	pp gpp (Sql_plus_macro _0) = fsep [gpp _0]
 
instance PP Sql_plus_statement where
	pp gpp (Sql_plus_statement_c1 _1) = fsep [gpp "SPOOL", gpp _1]
	pp gpp (Sql_plus_statement_c2) = fsep [gpp "SPOOL", gpp "OFF"]
	pp gpp (Sql_plus_statement_c3 _1 _2)
	  = fsep [gpp "TTITLE", gppMaybe gpp _1, gpp _2]
	pp gpp (Sql_plus_statement_c4 _1) = fsep [gpp "BTITLE", gpp _1]
	pp gpp (Sql_plus_statement_c5 _1 _3)
	  = fsep [gpp "DEFINE", gpp _1, gpp "=", gpp _3]
	pp gpp (Sql_plus_statement_c6 _1 _3)
	  = fsep [gpp "DEFINE", gpp _1, gpp "=", gpp _3]
	pp gpp (Sql_plus_statement_c7 _1 _3)
	  = fsep [gpp "DEFINE", gpp _1, gpp "=", gpp _3]
	pp gpp (Sql_plus_statement_c8 _1) = fsep [gpp "UNDEFINE", gpp _1]
	pp gpp (Sql_plus_statement_c9 _1) = fsep [gpp "UNDEFINE", gpp _1]
	pp gpp (Sql_plus_statement_c10 _1) = fsep [gpp "SET", gpp _1]
	pp gpp (Sql_plus_statement_c11 _1) = fsep [gpp "CLEAR", gpp _1]
	pp gpp (Sql_plus_statement_c12 _2)
	  = fsep [gpp "WHENEVER", gpp "SQLERROR", gpp _2]
	pp gpp (Sql_plus_statement_c13 _1)
	  = fsep [gpp "START", gppList gpp _1]
	pp gpp (Sql_plus_statement_c14 _1 _2)
	  = fsep [gpp "COL", gpp _1, gpp _2]
	pp gpp (Sql_plus_statement_c15 _1 _2)
	  = fsep [gpp "COLUMN", gpp _1, gpp _2]
	pp gpp (Sql_plus_statement_c16 _1)
	  = fsep [gpp "BREAK", gppList gpp _1]
	pp gpp (Sql_plus_statement_c17 _1 _3)
	  = fsep [gpp "ACCEPT", gpp _1, gpp "PROMPT", gpp _3]
	pp gpp (Sql_plus_statement_c18 _1)
	  = fsep [gpp "COMPUTE", gppList gpp _1]
	pp gpp (Sql_plus_statement_c19 _2)
	  = fsep [gpp "OPTIONS", gpp "(", gpp _2, gpp ")"]
	pp gpp (Sql_plus_statement_c20) = fsep [gpp "SQL", gpp ">"]
	pp gpp (Sql_plus_statement_c21)
	  = fsep [gpp "SQL", gpp ">", gpp "/"]
	pp gpp (Sql_plus_statement_c22) = fsep [gpp "TEXT"]
	pp gpp (Sql_plus_statement_c23 _0)
	  = fsep [gpp _0, gpp "ROWS", gpp "SELECTED", gpp "."]
 
instance PP Sql_plus_value where
	pp gpp (Sql_plus_value _0) = fsep [gpp _0]
 
instance PP Sql_plus_value_or_text where
	pp gpp (Sql_plus_value_or_text_c1 _0) = fsep [gpp _0]
	pp gpp (Sql_plus_value_or_text_c2 _0) = fsep [gpp _0]
 
instance PP Sql_statement where
	pp gpp (Sql_statement_c1 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c2 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c3 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c4 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c5 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c6 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c7 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c8 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c9 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c10 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c11 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c12 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c13 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c14 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c15 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c16 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c17 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c18 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c19 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c20 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c21 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c22 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c23 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c24 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c25 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c26 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c27 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c28 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c29 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c30 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c31 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c32 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c33 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c34 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c35 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c36 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c37 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c38 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c39 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c40 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c41 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c42 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c43 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c44 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c45 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c46 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c47 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c48 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c49 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c50 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c51 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c52 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c53 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c54 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c55 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c56 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c57 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c58 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c59 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c60 _0) = fsep [gpp _0]
	pp gpp (Sql_statement_c61 _0) = fsep [gpp _0]
 
instance PP Start_date where
	pp gpp (Start_date _2) = fsep [gpp "START", gpp "WITH", gpp _2]
 
instance PP Statement_block where
	pp gpp (Statement_block _0 _2)
	  = fsep [gpp _0, gpp "END", gppMaybe gpp _2]
 
instance PP Statement_block_no_end where
	pp gpp (Statement_block_no_end _1 _2)
	  = fsep [gpp "BEGIN", gpp _1, gppMaybe gpp _2]
 
instance PP Storage_clause where
	pp gpp (Storage_clause _1) = fsep [gpp "STORAGE", gpp _1]
 
instance PP Storage_item where
	pp gpp (Storage_item_c1 _1) = fsep [gpp "INITIAL", gpp _1]
	pp gpp (Storage_item_c2 _0) = fsep [gpp _0]
	pp gpp (Storage_item_c3 _1) = fsep [gpp "MINEXTENTS", gpp _1]
	pp gpp (Storage_item_c4 _1) = fsep [gpp "MAXEXTENTS", gpp _1]
	pp gpp (Storage_item_c5 _1) = fsep [gpp "PCTINCREASE", gpp _1]
	pp gpp (Storage_item_c6 _1) = fsep [gpp "FREELISTS", gpp _1]
	pp gpp (Storage_item_c7 _2)
	  = fsep [gpp "FREELIST", gpp "GROUPS", gpp _2]
	pp gpp (Storage_item_c8 _1) = fsep [gpp "OPTIMAL", gpp _1]
 
instance PP Storage_items where
	pp gpp (Storage_items _0) = fsep [gppList gpp _0]
 
instance PP Storage_items_opt_brackets where
	pp gpp (Storage_items_opt_brackets_c1 _0) = fsep [gpp _0]
	pp gpp (Storage_items_opt_brackets_c2 _1)
	  = fsep [gpp "(", gpp _1, gpp ")"]
 
instance PP Subquery where
	pp gpp (Subquery _0) = fsep [gpp _0]
 
instance PP Subquery_brackets where
	pp gpp (Subquery_brackets _1) = fsep [gpp "(", gpp _1, gpp ")"]
 
instance PP Subquery_opt_brackets where
	pp gpp (Subquery_opt_brackets_c1 _1)
	  = fsep [gpp "(", gpp _1, gpp ")"]
	pp gpp (Subquery_opt_brackets_c2 _0) = fsep [gpp _0]
 
instance PP Symbol_dot where
	pp gpp (Symbol_dot) = fsep [gpp "."]
 
instance PP System_item where
	pp gpp (System_item_c1 _0)
	  = fsep [gpp _0, gpp "RESTRICTED", gpp "SESSION"]
	pp gpp (System_item_c2) = fsep [gpp "FLUSH", gpp "SHARED_POOL"]
	pp gpp (System_item_c3 _0 _1) = fsep [gpp _0, gppMaybe gpp _1]
	pp gpp (System_item_c4 _1) = fsep [gpp "SET", gpp _1]
	pp gpp (System_item_c5) = fsep [gpp "SWITCH", gpp "LOGFILE"]
	pp gpp (System_item_c6 _0)
	  = fsep [gpp _0, gpp "DISTRIBUTED", gpp "RECOVERY"]
	pp gpp (System_item_c7 _0) = fsep [gpp _0]
	pp gpp (System_item_c8 _3 _5)
	  = fsep
	      [gpp "KILL", gpp "SESSION", gpp "'", gpp _3, gpp ",", gpp _5,
	       gpp "'"]
 
instance PP System_items where
	pp gpp (System_items _0) = fsep [gppList gpp _0]
 
instance PP System_set_item where
	pp gpp (System_set_item_c1 _2)
	  = fsep [gpp "RESOURCE_LIMIT", gpp "=", gpp _2]
	pp gpp (System_set_item_c2 _2)
	  = fsep [gpp "GLOBAL_NAMES", gpp "=", gpp _2]
	pp gpp (System_set_item_c3 _2)
	  = fsep [gpp "SCAN_INSTANCES", gpp "=", gpp _2]
	pp gpp (System_set_item_c4 _2)
	  = fsep [gpp "CACHE_INSTANCES", gpp "=", gpp _2]
	pp gpp (System_set_item_c5 _2)
	  = fsep [gpp "MTS_SERVERS", gpp "=", gpp _2]
	pp gpp (System_set_item_c6 _3 _5)
	  = fsep
	      [gpp "MTS_DISPATCHERS", gpp "=", gpp "'", gpp _3, gpp ",", gpp _5,
	       gpp "'"]
	pp gpp (System_set_item_c7 _2)
	  = fsep [gpp "LICENSE_MAX_SESSIONS", gpp "=", gpp _2]
	pp gpp (System_set_item_c8 _2)
	  = fsep [gpp "LICENSE_SESSIONS_WARNING", gpp "=", gpp _2]
	pp gpp (System_set_item_c9 _2)
	  = fsep [gpp "LICENSE_MAX_USERS", gpp "=", gpp _2]
	pp gpp (System_set_item_c10 _2)
	  = fsep [gpp "REMOTE_DEPENDENCIES_MODE", gpp "=", gpp _2]
 
instance PP System_set_items where
	pp gpp (System_set_items _0) = fsep [gppList gpp _0]
 
instance PP Table_description where
	pp gpp (Table_description_c1 _0) = fsep [gpp _0]
	pp gpp (Table_description_c2 _0) = fsep [gpp _0]
 
instance PP Table_item where
	pp gpp (Table_item_c1 _0 _1 _2 _3)
	  = fsep [gpp _0, gpp _1, gppMaybe gpp _2, gppMaybe gpp _3]
	pp gpp (Table_item_c2 _0) = fsep [gpp _0]
 
instance PP Table_items where
	pp gpp (Table_items _0) = fsep [gppListSep gpp "," _0]
 
instance PP Table_items_opt_brackets where
	pp gpp (Table_items_opt_brackets_c1 _1)
	  = fsep [gpp "(", gpp _1, gpp ")"]
	pp gpp (Table_items_opt_brackets_c2 _0) = fsep [gpp _0]
 
instance PP Table_modify_item where
	pp gpp (Table_modify_item _0 _1 _2 _3)
	  = fsep [gpp _0, gppMaybe gpp _1, gppMaybe gpp _2, gppMaybe gpp _3]
 
instance PP Table_modify_items where
	pp gpp (Table_modify_items _0) = fsep [gppList gpp _0]
 
instance PP Table_type_declaration where
	pp gpp (Table_type_declaration _1 _5 _6)
	  = fsep
	      [gpp "TYPE", gpp _1, gpp "IS", gpp "TABLE", gpp "OF", gpp _5,
	       gppMaybe gpp _6, gpp "INDEX", gpp "BY", gpp "BINARY_INTEGER"]
 
instance PP Tablespace where
	pp gpp (Tablespace _0) = fsep [gpp _0]
 
instance PP Tablespace_clause where
	pp gpp (Tablespace_clause _1 _2)
	  = fsep [gpp "TABLESPACE", gpp _1, gppMaybe gpp _2]
 
instance PP Tablespace_item where
	pp gpp (Tablespace_item_c1 _2)
	  = fsep [gpp "ADD", gpp "DATAFILE", gpp _2]
	pp gpp (Tablespace_item_c2 _0) = fsep [gpp _0]
	pp gpp (Tablespace_item_c3) = fsep [gpp "COALESCE"]
	pp gpp (Tablespace_item_c4 _1) = fsep [gpp "DEFAULT", gpp _1]
	pp gpp (Tablespace_item_c5) = fsep [gpp "ONLINE"]
	pp gpp (Tablespace_item_c6 _1)
	  = fsep [gpp "OFFLINE", gppMaybe gpp _1]
	pp gpp (Tablespace_item_c7) = fsep [gpp "BEGIN", gpp "BACKUP"]
	pp gpp (Tablespace_item_c8) = fsep [gpp "END", gpp "BACKUP"]
	pp gpp (Tablespace_item_c9) = fsep [gpp "READ", gpp "ONLY"]
	pp gpp (Tablespace_item_c10) = fsep [gpp "READ", gpp "WRITE"]
	pp gpp (Tablespace_item_c11) = fsep [gpp "PERMANENT"]
	pp gpp (Tablespace_item_c12) = fsep [gpp "TEMPORARY"]
 
instance PP Tablespace_items where
	pp gpp (Tablespace_items _0) = fsep [gppList gpp _0]
 
instance PP Territory where
	pp gpp (Territory _0) = fsep [gpp _0]
 
instance PP Text where
	pp gpp (Text_c1 _0) = fsep [gpp _0]
	pp gpp (Text_c2) = fsep [gpp "ESCAPE"]
 
instance PP Then_branch where
	pp gpp (Then_branch _1) = fsep [gpp "THEN", gpp _1]
 
instance PP Thread_integer where
	pp gpp (Thread_integer _1) = fsep [gpp "THREAD", gpp _1]
 
instance PP To_location where
	pp gpp (To_location _1) = fsep [gpp "TO", gpp _1]
 
instance PP Top_expression where
	pp gpp (Top_expression_c1 _0) = fsep [gpp _0]
	pp gpp (Top_expression_c2 _0) = fsep [gpp _0]
 
instance PP Top_sort where
	pp gpp (Top_sort_c1 _0) = fsep [gpp _0]
	pp gpp (Top_sort_c2 _0) = fsep [gpp _0]
 
instance PP Top_sort_cobol_embedded_sql where
	pp gpp (Top_sort_cobol_embedded_sql _0 _1)
	  = fsep [gppListSep gpp "." _0, gppMaybe gpp _1]
 
instance PP Top_sort_pl_sql where
	pp gpp (Top_sort_pl_sql _0) = fsep [gppList gpp _0]
 
instance PP Trailing_clause where
	pp gpp (Trailing_clause) = fsep [gpp "TRAILING", gpp "NULLCOLS"]
 
instance PP Trailing_declarations where
	pp gpp (Trailing_declarations _0 _2)
	  = fsep [gppMaybe gpp _0, gpp "END", gppMaybe gpp _2]
 
instance PP Trigger_item where
	pp gpp (Trigger_item_c1) = fsep [gpp "DELETE"]
	pp gpp (Trigger_item_c2) = fsep [gpp "INSERT"]
	pp gpp (Trigger_item_c3 _1) = fsep [gpp "UPDATE", gppMaybe gpp _1]
 
instance PP Trigger_items where
	pp gpp (Trigger_items _0) = fsep [gppListSep gpp "OR" _0]
 
instance PP Truncate_clause where
	pp gpp (Truncate_clause_c1) = fsep [gpp "DROP", gpp "STORAGE"]
	pp gpp (Truncate_clause_c2) = fsep [gpp "REUSE"]
 
instance PP Truncate_statement where
	pp gpp (Truncate_statement _1 _2 _3)
	  = fsep [gpp "TRUNCATE", gpp _1, gpp _2, gppMaybe gpp _3]
 
instance PP Ttitle_date where
	pp gpp (Ttitle_date_c1) = fsep [gpp "DATE"]
	pp gpp (Ttitle_date_c2) = fsep [gpp "DATEVAR"]
 
instance PP Ttitle_item where
	pp gpp (Ttitle_item_c1) = fsep [gpp "ON"]
	pp gpp (Ttitle_item_c2) = fsep [gpp "OFF"]
	pp gpp (Ttitle_item_c3 _1) = fsep [gpp "CENTER", gppList gpp _1]
	pp gpp (Ttitle_item_c4 _1) = fsep [gpp "SKIP", gppList gpp _1]
	pp gpp (Ttitle_item_c5 _1 _2)
	  = fsep [gpp "LEFT", gppMaybe gpp _1, gppList gpp _2]
	pp gpp (Ttitle_item_c6 _1 _2)
	  = fsep [gpp "RIGHT", gppMaybe gpp _1, gppList gpp _2]
 
instance PP Ttitle_items where
	pp gpp (Ttitle_items _0) = fsep [gppList gpp _0]
 
instance PP Type_name where
	pp gpp (Type_name _0) = fsep [gpp _0]
 
instance PP Unlabelled_pl_statement where
	pp gpp (Unlabelled_pl_statement_c1 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c2 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c3 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c4 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c5 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c6 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c7 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c8 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c9 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c10 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c11 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c12 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c13 _0) = fsep [gpp _0]
	pp gpp (Unlabelled_pl_statement_c14 _0) = fsep [gpp _0]
 
instance PP Update_statement where
	pp gpp (Update_statement _1 _3 _4)
	  = fsep [gpp "UPDATE", gpp _1, gpp "SET", gpp _3, gppMaybe gpp _4]
 
instance PP User where
	pp gpp (User _0) = fsep [gpp _0]
 
instance PP User_identification where
	pp gpp (User_identification_c1 _1) = fsep [gpp "BY", gpp _1]
	pp gpp (User_identification_c2) = fsep [gpp "EXTERNALLY"]
 
instance PP User_item where
	pp gpp (User_item_c1 _1) = fsep [gpp "IDENTIFIED", gpp _1]
	pp gpp (User_item_c2 _2)
	  = fsep [gpp "DEFAULT", gpp "TABLESPACE", gpp _2]
	pp gpp (User_item_c3 _2)
	  = fsep [gpp "TEMPORARY", gpp "TABLESPACE", gpp _2]
	pp gpp (User_item_c4 _0) = fsep [gpp _0]
	pp gpp (User_item_c5 _1) = fsep [gpp "PROFILE", gpp _1]
	pp gpp (User_item_c6 _2) = fsep [gpp "DEFAULT", gpp "ROLE", gpp _2]
 
instance PP User_items where
	pp gpp (User_items _0) = fsep [gppList gpp _0]
 
instance PP Using_index_clause where
	pp gpp (Using_index_clause _2)
	  = fsep [gpp "USING", gpp "INDEX", gppMaybe gpp _2]
 
instance PP Using_index_item where
	pp gpp (Using_index_item_c1 _1) = fsep [gpp "INITRANS", gpp _1]
	pp gpp (Using_index_item_c2 _1) = fsep [gpp "MAXTRANS", gpp _1]
	pp gpp (Using_index_item_c3 _0) = fsep [gpp _0]
	pp gpp (Using_index_item_c4 _0) = fsep [gpp _0]
	pp gpp (Using_index_item_c5 _1) = fsep [gpp "PCTFREE", gpp _1]
	pp gpp (Using_index_item_c6 _1) = fsep [gpp "PCTUSED", gpp _1]
 
instance PP Values_description where
	pp gpp (Values_description_c1 _2)
	  = fsep [gpp "VALUES", gpp "(", gpp _2, gpp ")"]
	pp gpp (Values_description_c2 _0) = fsep [gpp _0]
	pp gpp (Values_description_c3 _0) = fsep [gpp _0]
 
instance PP Var_declaration where
	pp gpp (Var_declaration _0 _1 _2)
	  = fsep [gpp _0, gpp _1, gppMaybe gpp _2]
 
instance PP View_with_clause where
	pp gpp (View_with_clause_c1) = fsep [gpp "WITH"]
	pp gpp (View_with_clause_c2)
	  = fsep [gpp "WITH", gpp "READ", gpp "ONLY"]
	pp gpp (View_with_clause_c3 _3)
	  = fsep [gpp "WITH", gpp "CHECK", gpp "OPTION", gppMaybe gpp _3]
 
instance PP When_clause where
	pp gpp (When_clause _1) = fsep [gpp "WHEN", gpp _1]
 
instance PP Whenever_sqlerror_clause where
	pp gpp (Whenever_sqlerror_clause_c1) = fsep [gpp "CONTINUE"]
	pp gpp (Whenever_sqlerror_clause_c2)
	  = fsep [gpp "EXIT", gpp "ROLLBACK"]
	pp gpp (Whenever_sqlerror_clause_c3)
	  = fsep [gpp "EXIT", gpp "FAILURE"]
 
instance PP Whenever_sqlerror_statement where
	pp gpp (Whenever_sqlerror_statement_c1 _3)
	  = fsep [gpp "WHENEVER", gpp "SQLERROR", gpp "GOTO", gpp _3]
	pp gpp (Whenever_sqlerror_statement_c2)
	  = fsep [gpp "WHENEVER", gpp "SQLERROR", gpp "CONTINUE"]
 
instance PP Where_clause where
	pp gpp (Where_clause _1) = fsep [gpp "WHERE", gpp _1]
 
instance PP While_loop_statement where
	pp gpp (While_loop_statement _1 _2)
	  = fsep [gpp "WHILE", gpp _1, gpp _2]
 
instance PP Lex_name where
	pp gpp (Lex_identifier _0) = fsep [gpp _0]
	pp gpp (Lex_quoted_name _0) = fsep [gpp _0]
	pp gpp (Lex_ampersand_identifier _0) = fsep [gpp _0]
	pp gpp (REF) = fsep [gpp "REF"]
	pp gpp (Lex_name_kw _0) = fsep [gpp _0]
 
instance PP Lex_qualified_name where
	pp gpp (Lex_name _0) = fsep [gpp _0]
	pp gpp (Lex_name_Lex_dot_Lex_name _0 _1 _2)
	  = fsep [gpp _0, gpp _1, gpp _2]
	pp gpp (Lex_name_Lex_dot_Lex_name_Lex_dot_Lex_name _0 _1 _2 _3 _4)
	  = fsep [gpp _0, gpp _1, gpp _2, gpp _3, gpp _4]
	pp gpp (Lex_name_Lex_at_Lex_name _0 _1 _2)
	  = fsep [gpp _0, gpp _1, gpp _2]
	pp gpp (Lex_name_Lex_dot_Lex_name_Lex_at_Lex_name _0 _1 _2 _3 _4)
	  = fsep [gpp _0, gpp _1, gpp _2, gpp _3, gpp _4]
 
instance PP Lex_host_name where
	pp gpp (Lex_host_identifier _1) = fsep [gpp ":", gpp _1]
 
instance PP Lex_qualified_host_name where
	pp gpp (Lex_host_name_p _0) = fsep [gppList gpp _0]
	pp gpp (Lex_host_name_Lex_dot_Lex_host_identifier _0 _1 _2)
	  = fsep [gpp _0, gpp _1, gpp _2]
	pp gpp (Lex_host_name_Lex_host_identifier _0 _3)
	  = fsep [gpp _0, gpp "[", gpp ":", gpp _3, gpp "]"]
 
instance PP Lex_schema_qualifier where
	pp gpp (Lex_name_Lex_dot_star _0 _1) = fsep [gpp _0, gpp _1]
	pp gpp (Lex_name_Lex_dot_Lex_name_Lex_dot_star _0 _1 _2 _3)
	  = fsep [gpp _0, gpp _1, gpp _2, gpp _3]
 
instance PP Kw_k_m where
	pp gpp (K) = fsep [gpp "K"]
	pp gpp (M) = fsep [gpp "M"]
 
instance PP Lex_memory_size where
	pp gpp (Lex_memory_size_int_Kw_k_m_opt _0 _1)
	  = fsep [gpp _0, gppMaybe gpp _1]
 
instance PP Lex_sql_plus_macro where
	pp gpp (Lex_at_Lex_at_Lex_sql_plus_identifier _0 _1 _2)
	  = fsep [gpp _0, gpp _1, gpp _2]
 
uppPLSqlSyntax :: UPP
uppPLSqlSyntax gpp
  = (constTU empty) `adhocQ` (pp gpp :: MonoPP String) `adhocQ`
      (pp gpp :: MonoPP Lex_identifier)
      `adhocQ` (pp gpp :: MonoPP Alias)
      `adhocQ` (pp gpp :: MonoPP Aliased_select_item)
      `adhocQ` (pp gpp :: MonoPP Aliased_select_items)
      `adhocQ` (pp gpp :: MonoPP Aliased_select_items_or_all)
      `adhocQ` (pp gpp :: MonoPP Aliased_table_description)
      `adhocQ` (pp gpp :: MonoPP Aliased_table_descriptions)
      `adhocQ` (pp gpp :: MonoPP Allocate_extent_clause)
      `adhocQ` (pp gpp :: MonoPP Allocate_extent_item)
      `adhocQ` (pp gpp :: MonoPP Allocate_extent_items)
      `adhocQ` (pp gpp :: MonoPP Allocate_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_cluster_item)
      `adhocQ` (pp gpp :: MonoPP Alter_cluster_items)
      `adhocQ` (pp gpp :: MonoPP Alter_cluster_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_database_item)
      `adhocQ` (pp gpp :: MonoPP Alter_database_items)
      `adhocQ` (pp gpp :: MonoPP Alter_database_set_item)
      `adhocQ` (pp gpp :: MonoPP Alter_database_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_datafile_item)
      `adhocQ` (pp gpp :: MonoPP Alter_function_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_index_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_index_statement_item)
      `adhocQ` (pp gpp :: MonoPP Alter_index_statement_items)
      `adhocQ` (pp gpp :: MonoPP Alter_package_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_procedure_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_profile_item)
      `adhocQ` (pp gpp :: MonoPP Alter_profile_item_1)
      `adhocQ` (pp gpp :: MonoPP Alter_profile_items)
      `adhocQ` (pp gpp :: MonoPP Alter_profile_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_resource_cost_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_role_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_rollback_segment_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_sequence_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_session_clause)
      `adhocQ` (pp gpp :: MonoPP Alter_session_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_snapshot_log_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_snapshot_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_system_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_table_item)
      `adhocQ` (pp gpp :: MonoPP Alter_table_item_1)
      `adhocQ` (pp gpp :: MonoPP Alter_table_item_2)
      `adhocQ` (pp gpp :: MonoPP Alter_table_items_1)
      `adhocQ` (pp gpp :: MonoPP Alter_table_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_tablespace_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_trigger_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_user_statement)
      `adhocQ` (pp gpp :: MonoPP Alter_view_statement)
      `adhocQ` (pp gpp :: MonoPP Append_item)
      `adhocQ` (pp gpp :: MonoPP Append_items)
      `adhocQ` (pp gpp :: MonoPP Append_items_brackets)
      `adhocQ` (pp gpp :: MonoPP Archive_log_clause)
      `adhocQ` (pp gpp :: MonoPP Archive_log_item)
      `adhocQ` (pp gpp :: MonoPP Archive_log_item_located)
      `adhocQ` (pp gpp :: MonoPP As_subquery)
      `adhocQ` (pp gpp :: MonoPP Assignment_statement)
      `adhocQ` (pp gpp :: MonoPP Assignment_symbol)
      `adhocQ` (pp gpp :: MonoPP Atomic_boolean_expression)
      `adhocQ` (pp gpp :: MonoPP Atomic_expression)
      `adhocQ` (pp gpp :: MonoPP Autoextend)
      `adhocQ` (pp gpp :: MonoPP Autoextend_clause)
      `adhocQ` (pp gpp :: MonoPP Backup_controlfile_clause)
      `adhocQ` (pp gpp :: MonoPP Basic_loop_statement)
      `adhocQ` (pp gpp :: MonoPP Begin_declare_section_statement)
      `adhocQ` (pp gpp :: MonoPP Binary_or_name)
      `adhocQ` (pp gpp :: MonoPP Boolean_expression)
      `adhocQ` (pp gpp :: MonoPP Break_item)
      `adhocQ` (pp gpp :: MonoPP Bulk_collect)
      `adhocQ` (pp gpp :: MonoPP Call)
      `adhocQ` (pp gpp :: MonoPP Checkpoint)
      `adhocQ` (pp gpp :: MonoPP Close_statement)
      `adhocQ` (pp gpp :: MonoPP Col_item)
      `adhocQ` (pp gpp :: MonoPP Col_items)
      `adhocQ` (pp gpp :: MonoPP Column_name)
      `adhocQ` (pp gpp :: MonoPP Column_names)
      `adhocQ` (pp gpp :: MonoPP Columns_brackets)
      `adhocQ` (pp gpp :: MonoPP Comment)
      `adhocQ` (pp gpp :: MonoPP Comment_item)
      `adhocQ` (pp gpp :: MonoPP Comment_statement)
      `adhocQ` (pp gpp :: MonoPP Commit_statement)
      `adhocQ` (pp gpp :: MonoPP Connect_by_clause)
      `adhocQ` (pp gpp :: MonoPP Connect_by_clause_prefix)
      `adhocQ` (pp gpp :: MonoPP Connect_to_clause)
      `adhocQ` (pp gpp :: MonoPP Connect_using_clause)
      `adhocQ` (pp gpp :: MonoPP Constant_declaration)
      `adhocQ` (pp gpp :: MonoPP Constant_declaration_name)
      `adhocQ` (pp gpp :: MonoPP Constraint)
      `adhocQ` (pp gpp :: MonoPP Constraint_clause)
      `adhocQ` (pp gpp :: MonoPP Constraint_header)
      `adhocQ` (pp gpp :: MonoPP Constraint_item_1)
      `adhocQ` (pp gpp :: MonoPP Constraint_item_2)
      `adhocQ` (pp gpp :: MonoPP Constraints)
      `adhocQ` (pp gpp :: MonoPP Create_database_link_statement)
      `adhocQ` (pp gpp :: MonoPP Create_function_statement)
      `adhocQ` (pp gpp :: MonoPP Create_index_statement)
      `adhocQ` (pp gpp :: MonoPP Create_or_replace)
      `adhocQ` (pp gpp :: MonoPP Create_procedure_statement)
      `adhocQ` (pp gpp :: MonoPP Create_role_statement)
      `adhocQ` (pp gpp :: MonoPP Create_rollback_segment_item)
      `adhocQ` (pp gpp :: MonoPP Create_rollback_segment_items)
      `adhocQ` (pp gpp :: MonoPP Create_rollback_segment_statement)
      `adhocQ` (pp gpp :: MonoPP Create_sequence_statement)
      `adhocQ` (pp gpp :: MonoPP Create_synonym_statement)
      `adhocQ` (pp gpp :: MonoPP Create_table_clause_1)
      `adhocQ` (pp gpp :: MonoPP Create_table_clause_2)
      `adhocQ` (pp gpp :: MonoPP Create_table_statement)
      `adhocQ` (pp gpp :: MonoPP Create_tablespace_statement)
      `adhocQ` (pp gpp :: MonoPP Create_trigger_statement)
      `adhocQ` (pp gpp :: MonoPP Create_type_statement)
      `adhocQ` (pp gpp :: MonoPP Create_user_statement)
      `adhocQ` (pp gpp :: MonoPP Create_view_statement)
      `adhocQ` (pp gpp :: MonoPP Cursor)
      `adhocQ` (pp gpp :: MonoPP Cursor_attribute_attr)
      `adhocQ` (pp gpp :: MonoPP Cursor_declaration)
      `adhocQ` (pp gpp :: MonoPP Cursor_declaration_kernel)
      `adhocQ` (pp gpp :: MonoPP Cursor_name)
      `adhocQ` (pp gpp :: MonoPP Cursor_ref)
      `adhocQ` (pp gpp :: MonoPP Cursor_return)
      `adhocQ` (pp gpp :: MonoPP Datafile_description)
      `adhocQ` (pp gpp :: MonoPP Datafile_descriptions)
      `adhocQ` (pp gpp :: MonoPP Datatype)
      `adhocQ` (pp gpp :: MonoPP Date)
      `adhocQ` (pp gpp :: MonoPP Dblink)
      `adhocQ` (pp gpp :: MonoPP Declaration)
      `adhocQ` (pp gpp :: MonoPP Declaration_init_value)
      `adhocQ` (pp gpp :: MonoPP Declarations)
      `adhocQ` (pp gpp :: MonoPP Declarations_semicolon)
      `adhocQ` (pp gpp :: MonoPP Declare_cursor_statement)
      `adhocQ` (pp gpp :: MonoPP Declare_section)
      `adhocQ` (pp gpp :: MonoPP Default_expression)
      `adhocQ` (pp gpp :: MonoPP Delete_item)
      `adhocQ` (pp gpp :: MonoPP Delete_statement)
      `adhocQ` (pp gpp :: MonoPP Disable_clause)
      `adhocQ` (pp gpp :: MonoPP Disable_clause_item)
      `adhocQ` (pp gpp :: MonoPP Drop_database_link_statement)
      `adhocQ` (pp gpp :: MonoPP Drop_function_statement)
      `adhocQ` (pp gpp :: MonoPP Drop_index_statement)
      `adhocQ` (pp gpp :: MonoPP Drop_procedure_statement)
      `adhocQ` (pp gpp :: MonoPP Drop_sequence_statement)
      `adhocQ` (pp gpp :: MonoPP Drop_synonym_statement)
      `adhocQ` (pp gpp :: MonoPP Drop_table_statement)
      `adhocQ` (pp gpp :: MonoPP Drop_trigger_statement)
      `adhocQ` (pp gpp :: MonoPP Drop_view_statement)
      `adhocQ` (pp gpp :: MonoPP Dummy_top_sort)
      `adhocQ` (pp gpp :: MonoPP Else_branch)
      `adhocQ` (pp gpp :: MonoPP Elsif_branch)
      `adhocQ` (pp gpp :: MonoPP Emb_name)
      `adhocQ` (pp gpp :: MonoPP Embedded_sql_statement)
      `adhocQ` (pp gpp :: MonoPP Embedded_sql_statements)
      `adhocQ` (pp gpp :: MonoPP En_dis_able_clauses)
      `adhocQ` (pp gpp :: MonoPP Enable_clause)
      `adhocQ` (pp gpp :: MonoPP Enable_clause_item)
      `adhocQ` (pp gpp :: MonoPP Enable_clause_item_header)
      `adhocQ` (pp gpp :: MonoPP End_declare_section_statement)
      `adhocQ` (pp gpp :: MonoPP End_of_function)
      `adhocQ` (pp gpp :: MonoPP End_of_package)
      `adhocQ` (pp gpp :: MonoPP End_of_procedure)
      `adhocQ` (pp gpp :: MonoPP Exception_declaration)
      `adhocQ` (pp gpp :: MonoPP Exception_handler)
      `adhocQ` (pp gpp :: MonoPP Exception_handler_others)
      `adhocQ` (pp gpp :: MonoPP Exception_handlers)
      `adhocQ` (pp gpp :: MonoPP Exception_name)
      `adhocQ` (pp gpp :: MonoPP Exception_section)
      `adhocQ` (pp gpp :: MonoPP Exception_when_clause)
      `adhocQ` (pp gpp :: MonoPP Exceptions_into)
      `adhocQ` (pp gpp :: MonoPP Execute_parameter)
      `adhocQ` (pp gpp :: MonoPP Execute_statement)
      `adhocQ` (pp gpp :: MonoPP Execute_using)
      `adhocQ` (pp gpp :: MonoPP Exit_statement)
      `adhocQ` (pp gpp :: MonoPP Exit_when_clause)
      `adhocQ` (pp gpp :: MonoPP Explain_plan_statement)
      `adhocQ` (pp gpp :: MonoPP Expression)
      `adhocQ` (pp gpp :: MonoPP Expression_or_subquery_brackets)
      `adhocQ` (pp gpp :: MonoPP Expressions)
      `adhocQ` (pp gpp :: MonoPP Expressions_brackets)
      `adhocQ` (pp gpp :: MonoPP Expressions_or_subquery_brackets)
      `adhocQ` (pp gpp :: MonoPP Fetch_name)
      `adhocQ` (pp gpp :: MonoPP Fetch_names)
      `adhocQ` (pp gpp :: MonoPP Fetch_statement)
      `adhocQ` (pp gpp :: MonoPP Field_declaration)
      `adhocQ` (pp gpp :: MonoPP Field_declarations)
      `adhocQ` (pp gpp :: MonoPP File_rename_clause)
      `adhocQ` (pp gpp :: MonoPP Filename)
      `adhocQ` (pp gpp :: MonoPP Filename_with_extension)
      `adhocQ` (pp gpp :: MonoPP Filenames)
      `adhocQ` (pp gpp :: MonoPP Filenames_opt_brackets)
      `adhocQ` (pp gpp :: MonoPP Filespec)
      `adhocQ` (pp gpp :: MonoPP Filespecs)
      `adhocQ` (pp gpp :: MonoPP For_loop_statement)
      `adhocQ` (pp gpp :: MonoPP For_loop_statement_range)
      `adhocQ` (pp gpp :: MonoPP For_sql_statement)
      `adhocQ` (pp gpp :: MonoPP For_update_clause)
      `adhocQ` (pp gpp :: MonoPP For_update_item)
      `adhocQ` (pp gpp :: MonoPP For_update_items)
      `adhocQ` (pp gpp :: MonoPP Format)
      `adhocQ` (pp gpp :: MonoPP From_location)
      `adhocQ` (pp gpp :: MonoPP Function_body_declaration)
      `adhocQ` (pp gpp :: MonoPP Function_call)
      `adhocQ` (pp gpp :: MonoPP Function_header_declaration)
      `adhocQ` (pp gpp :: MonoPP Function_name)
      `adhocQ` (pp gpp :: MonoPP Goto_statement)
      `adhocQ` (pp gpp :: MonoPP Grant_item)
      `adhocQ` (pp gpp :: MonoPP Grant_statement)
      `adhocQ` (pp gpp :: MonoPP Grantee)
      `adhocQ` (pp gpp :: MonoPP Grantees)
      `adhocQ` (pp gpp :: MonoPP Group_by_clause)
      `adhocQ` (pp gpp :: MonoPP Group_integer)
      `adhocQ` (pp gpp :: MonoPP Group_or_filenames)
      `adhocQ` (pp gpp :: MonoPP Group_or_filenames_list)
      `adhocQ` (pp gpp :: MonoPP Grouped_filespec)
      `adhocQ` (pp gpp :: MonoPP Grouped_filespecs)
      `adhocQ` (pp gpp :: MonoPP Having_clause)
      `adhocQ` (pp gpp :: MonoPP Host_language_statement)
      `adhocQ` (pp gpp :: MonoPP Host_name)
      `adhocQ` (pp gpp :: MonoPP If_statement)
      `adhocQ` (pp gpp :: MonoPP Include_statement)
      `adhocQ` (pp gpp :: MonoPP Index_on_clause)
      `adhocQ` (pp gpp :: MonoPP Index_parameter)
      `adhocQ` (pp gpp :: MonoPP Index_parameters)
      `adhocQ` (pp gpp :: MonoPP Init_value)
      `adhocQ` (pp gpp :: MonoPP Insert_statement)
      `adhocQ` (pp gpp :: MonoPP SqlInteger)
      `adhocQ` (pp gpp :: MonoPP Integer_or_default)
      `adhocQ` (pp gpp :: MonoPP Integer_or_other)
      `adhocQ` (pp gpp :: MonoPP Integer_or_unlimited)
      `adhocQ` (pp gpp :: MonoPP Integer_text_var)
      `adhocQ` (pp gpp :: MonoPP Into)
      `adhocQ` (pp gpp :: MonoPP Into_qualified_name)
      `adhocQ` (pp gpp :: MonoPP Into_qualified_names)
      `adhocQ` (pp gpp :: MonoPP Isolation_level)
      `adhocQ` (pp gpp :: MonoPP Isolation_level_clause)
      `adhocQ` (pp gpp :: MonoPP Keep_clause)
      `adhocQ` (pp gpp :: MonoPP Keyword_except_to_on_from)
      `adhocQ` (pp gpp :: MonoPP Kw_all_triggers)
      `adhocQ` (pp gpp :: MonoPP Kw_any_some_all)
      `adhocQ` (pp gpp :: MonoPP Kw_append)
      `adhocQ` (pp gpp :: MonoPP Kw_as)
      `adhocQ` (pp gpp :: MonoPP Kw_assignment_default)
      `adhocQ` (pp gpp :: MonoPP Kw_automatic)
      `adhocQ` (pp gpp :: MonoPP Kw_before_after_instead_of)
      `adhocQ` (pp gpp :: MonoPP Kw_bitmap)
      `adhocQ` (pp gpp :: MonoPP Kw_cache_nocache)
      `adhocQ` (pp gpp :: MonoPP Kw_cascade)
      `adhocQ` (pp gpp :: MonoPP Kw_cascade_constraints)
      `adhocQ` (pp gpp :: MonoPP Kw_commit_rollback_nothing)
      `adhocQ` (pp gpp :: MonoPP Kw_default)
      `adhocQ` (pp gpp :: MonoPP Kw_desc_asc)
      `adhocQ` (pp gpp :: MonoPP Kw_distinct_all)
      `adhocQ` (pp gpp :: MonoPP Kw_drop)
      `adhocQ` (pp gpp :: MonoPP Kw_enable_disable)
      `adhocQ` (pp gpp :: MonoPP Kw_enable_disable_compile)
      `adhocQ` (pp gpp :: MonoPP Kw_exclusive_parallel)
      `adhocQ` (pp gpp :: MonoPP Kw_external)
      `adhocQ` (pp gpp :: MonoPP Kw_fast_complete_force)
      `adhocQ` (pp gpp :: MonoPP Kw_file_datafile)
      `adhocQ` (pp gpp :: MonoPP Kw_force_no_force)
      `adhocQ` (pp gpp :: MonoPP Kw_from)
      `adhocQ` (pp gpp :: MonoPP Kw_global_local)
      `adhocQ` (pp gpp :: MonoPP Kw_immediate)
      `adhocQ` (pp gpp :: MonoPP Kw_in_out_nocopy)
      `adhocQ` (pp gpp :: MonoPP Kw_into)
      `adhocQ` (pp gpp :: MonoPP Kw_is_as)
      `adhocQ` (pp gpp :: MonoPP Kw_local)
      `adhocQ` (pp gpp :: MonoPP Kw_no_resetlogs)
      `adhocQ` (pp gpp :: MonoPP Kw_nocache_cache)
      `adhocQ` (pp gpp :: MonoPP Kw_nologging)
      `adhocQ` (pp gpp :: MonoPP Kw_normal_temporary_immediate)
      `adhocQ` (pp gpp :: MonoPP Kw_not)
      `adhocQ` (pp gpp :: MonoPP Kw_not_null)
      `adhocQ` (pp gpp :: MonoPP Kw_nowait)
      `adhocQ` (pp gpp :: MonoPP Kw_old_new)
      `adhocQ` (pp gpp :: MonoPP Kw_on)
      `adhocQ` (pp gpp :: MonoPP Kw_on_delete_cascade)
      `adhocQ` (pp gpp :: MonoPP Kw_on_off)
      `adhocQ` (pp gpp :: MonoPP Kw_or_replace)
      `adhocQ` (pp gpp :: MonoPP Kw_package_body)
      `adhocQ` (pp gpp :: MonoPP Kw_public)
      `adhocQ` (pp gpp :: MonoPP Kw_reuse)
      `adhocQ` (pp gpp :: MonoPP Kw_reverse)
      `adhocQ` (pp gpp :: MonoPP Kw_savepoint)
      `adhocQ` (pp gpp :: MonoPP Kw_semicolon)
      `adhocQ` (pp gpp :: MonoPP Kw_sql_trace_global_names)
      `adhocQ` (pp gpp :: MonoPP Kw_standby_database)
      `adhocQ` (pp gpp :: MonoPP Kw_table_cluster)
      `adhocQ` (pp gpp :: MonoPP Kw_timestamp_signature)
      `adhocQ` (pp gpp :: MonoPP Kw_to)
      `adhocQ` (pp gpp :: MonoPP Kw_true_false)
      `adhocQ` (pp gpp :: MonoPP Kw_unarchived)
      `adhocQ` (pp gpp :: MonoPP Kw_unique)
      `adhocQ` (pp gpp :: MonoPP Kw_unique_primary_key)
      `adhocQ` (pp gpp :: MonoPP Kw_unrecoverable_datafile)
      `adhocQ` (pp gpp :: MonoPP Kw_with_admin_option)
      `adhocQ` (pp gpp :: MonoPP Kw_work)
      `adhocQ` (pp gpp :: MonoPP Label_header)
      `adhocQ` (pp gpp :: MonoPP Label_name)
      `adhocQ` (pp gpp :: MonoPP Language)
      `adhocQ` (pp gpp :: MonoPP Left_right)
      `adhocQ` (pp gpp :: MonoPP Lhs)
      `adhocQ` (pp gpp :: MonoPP Load_data_clause)
      `adhocQ` (pp gpp :: MonoPP Load_data_option)
      `adhocQ` (pp gpp :: MonoPP Load_data_options)
      `adhocQ` (pp gpp :: MonoPP Load_data_parameter)
      `adhocQ` (pp gpp :: MonoPP Load_data_parameter_value)
      `adhocQ` (pp gpp :: MonoPP Load_data_statement)
      `adhocQ` (pp gpp :: MonoPP Load_data_value)
      `adhocQ` (pp gpp :: MonoPP Lock_mode)
      `adhocQ` (pp gpp :: MonoPP Lock_table_statement)
      `adhocQ` (pp gpp :: MonoPP Logfile_member)
      `adhocQ` (pp gpp :: MonoPP Logfile_members)
      `adhocQ` (pp gpp :: MonoPP Loop_statement)
      `adhocQ` (pp gpp :: MonoPP Max_memory_size)
      `adhocQ` (pp gpp :: MonoPP Memory_size)
      `adhocQ` (pp gpp :: MonoPP Memory_size_clause)
      `adhocQ` (pp gpp :: MonoPP Memory_size_or_null)
      `adhocQ` (pp gpp :: MonoPP Memory_size_or_other)
      `adhocQ` (pp gpp :: MonoPP Memory_size_or_unlimited)
      `adhocQ` (pp gpp :: MonoPP Meta_parameter)
      `adhocQ` (pp gpp :: MonoPP Meta_parameter_value)
      `adhocQ` (pp gpp :: MonoPP Meta_parameter_values)
      `adhocQ` (pp gpp :: MonoPP Meta_statement)
      `adhocQ` (pp gpp :: MonoPP Meta_statement_semicolon)
      `adhocQ` (pp gpp :: MonoPP Name)
      `adhocQ` (pp gpp :: MonoPP Name_or_keyword)
      `adhocQ` (pp gpp :: MonoPP Names)
      `adhocQ` (pp gpp :: MonoPP Names_brackets)
      `adhocQ` (pp gpp :: MonoPP Next_date)
      `adhocQ` (pp gpp :: MonoPP Next_memory_size)
      `adhocQ` (pp gpp :: MonoPP Null_statement)
      `adhocQ` (pp gpp :: MonoPP Number)
      `adhocQ` (pp gpp :: MonoPP Of_columns)
      `adhocQ` (pp gpp :: MonoPP On_clause)
      `adhocQ` (pp gpp :: MonoPP Open_for_clause)
      `adhocQ` (pp gpp :: MonoPP Open_for_cursor)
      `adhocQ` (pp gpp :: MonoPP Open_for_statement)
      `adhocQ` (pp gpp :: MonoPP Open_statement)
      `adhocQ` (pp gpp :: MonoPP Or_exception_name)
      `adhocQ` (pp gpp :: MonoPP Order_by_clause)
      `adhocQ` (pp gpp :: MonoPP Order_item)
      `adhocQ` (pp gpp :: MonoPP Order_items)
      `adhocQ` (pp gpp :: MonoPP Ordered_column)
      `adhocQ` (pp gpp :: MonoPP Ordered_columns)
      `adhocQ` (pp gpp :: MonoPP Package_body)
      `adhocQ` (pp gpp :: MonoPP Package_body_prefix)
      `adhocQ` (pp gpp :: MonoPP Package_declaration)
      `adhocQ` (pp gpp :: MonoPP Package_header)
      `adhocQ` (pp gpp :: MonoPP Package_header_prefix)
      `adhocQ` (pp gpp :: MonoPP Package_name)
      `adhocQ` (pp gpp :: MonoPP Parallel_clause)
      `adhocQ` (pp gpp :: MonoPP Parallel_items)
      `adhocQ` (pp gpp :: MonoPP Parameter)
      `adhocQ` (pp gpp :: MonoPP Parameter_declaration)
      `adhocQ` (pp gpp :: MonoPP Parameter_declarations)
      `adhocQ` (pp gpp :: MonoPP Parameter_declarations_brackets)
      `adhocQ` (pp gpp :: MonoPP Parameters)
      `adhocQ` (pp gpp :: MonoPP Parameters_brackets)
      `adhocQ` (pp gpp :: MonoPP Password)
      `adhocQ` (pp gpp :: MonoPP Perc_found)
      `adhocQ` (pp gpp :: MonoPP Perc_isopen)
      `adhocQ` (pp gpp :: MonoPP Perc_notfound)
      `adhocQ` (pp gpp :: MonoPP Perc_rowcount)
      `adhocQ` (pp gpp :: MonoPP Perc_rowtype)
      `adhocQ` (pp gpp :: MonoPP Perc_type)
      `adhocQ` (pp gpp :: MonoPP Perc_typed_name)
      `adhocQ` (pp gpp :: MonoPP Permission)
      `adhocQ` (pp gpp :: MonoPP Pl_statement)
      `adhocQ` (pp gpp :: MonoPP Pl_statements)
      `adhocQ` (pp gpp :: MonoPP Plsql_block)
      `adhocQ` (pp gpp :: MonoPP Position)
      `adhocQ` (pp gpp :: MonoPP Pragma_declaration)
      `adhocQ` (pp gpp :: MonoPP Procedure_body_declaration)
      `adhocQ` (pp gpp :: MonoPP Procedure_header_declaration)
      `adhocQ` (pp gpp :: MonoPP Profile)
      `adhocQ` (pp gpp :: MonoPP Qualified_name)
      `adhocQ` (pp gpp :: MonoPP Qualified_names)
      `adhocQ` (pp gpp :: MonoPP Quota)
      `adhocQ` (pp gpp :: MonoPP Quotas)
      `adhocQ` (pp gpp :: MonoPP Raise_statement)
      `adhocQ` (pp gpp :: MonoPP Rebuild_item)
      `adhocQ` (pp gpp :: MonoPP Rebuild_items)
      `adhocQ` (pp gpp :: MonoPP Record_type_definition)
      `adhocQ` (pp gpp :: MonoPP Recover_clause)
      `adhocQ` (pp gpp :: MonoPP Recover_database_item)
      `adhocQ` (pp gpp :: MonoPP Recover_item)
      `adhocQ` (pp gpp :: MonoPP Ref_type_declaration)
      `adhocQ` (pp gpp :: MonoPP Referencing)
      `adhocQ` (pp gpp :: MonoPP Referencing_item)
      `adhocQ` (pp gpp :: MonoPP Referencing_items)
      `adhocQ` (pp gpp :: MonoPP Relational_operator)
      `adhocQ` (pp gpp :: MonoPP Rename_statement)
      `adhocQ` (pp gpp :: MonoPP Resource_cost_item)
      `adhocQ` (pp gpp :: MonoPP Resource_cost_items)
      `adhocQ` (pp gpp :: MonoPP Return_statement)
      `adhocQ` (pp gpp :: MonoPP Return_type)
      `adhocQ` (pp gpp :: MonoPP Reused_filename)
      `adhocQ` (pp gpp :: MonoPP Reused_filenames)
      `adhocQ` (pp gpp :: MonoPP Revoke_statement)
      `adhocQ` (pp gpp :: MonoPP Role)
      `adhocQ` (pp gpp :: MonoPP Role_description)
      `adhocQ` (pp gpp :: MonoPP Role_item)
      `adhocQ` (pp gpp :: MonoPP Roles)
      `adhocQ` (pp gpp :: MonoPP Rollback_segment_item)
      `adhocQ` (pp gpp :: MonoPP Rollback_statement)
      `adhocQ` (pp gpp :: MonoPP Row_clause)
      `adhocQ` (pp gpp :: MonoPP Savepoint)
      `adhocQ` (pp gpp :: MonoPP Savepoint_statement)
      `adhocQ` (pp gpp :: MonoPP Schema_qualifier)
      `adhocQ` (pp gpp :: MonoPP Select_clause)
      `adhocQ` (pp gpp :: MonoPP Select_clause_operator)
      `adhocQ` (pp gpp :: MonoPP Select_item)
      `adhocQ` (pp gpp :: MonoPP Select_statement)
      `adhocQ` (pp gpp :: MonoPP Select_statement_without_into)
      `adhocQ` (pp gpp :: MonoPP Sequence_item)
      `adhocQ` (pp gpp :: MonoPP Sequence_items)
      `adhocQ` (pp gpp :: MonoPP Session_set_item)
      `adhocQ` (pp gpp :: MonoPP Session_set_items)
      `adhocQ` (pp gpp :: MonoPP Set_flagger_item)
      `adhocQ` (pp gpp :: MonoPP Set_item)
      `adhocQ` (pp gpp :: MonoPP Set_items)
      `adhocQ` (pp gpp :: MonoPP Set_label_item)
      `adhocQ` (pp gpp :: MonoPP Set_optimizer_goal_item)
      `adhocQ` (pp gpp :: MonoPP Set_statement_id)
      `adhocQ` (pp gpp :: MonoPP Set_transaction_clause)
      `adhocQ` (pp gpp :: MonoPP Set_transaction_statement)
      `adhocQ` (pp gpp :: MonoPP Snap_shot_item)
      `adhocQ` (pp gpp :: MonoPP Snap_shot_items)
      `adhocQ` (pp gpp :: MonoPP Snapshot_refresh)
      `adhocQ` (pp gpp :: MonoPP Sql_plus_identifier)
      `adhocQ` (pp gpp :: MonoPP Sql_plus_macro)
      `adhocQ` (pp gpp :: MonoPP Sql_plus_statement)
      `adhocQ` (pp gpp :: MonoPP Sql_plus_value)
      `adhocQ` (pp gpp :: MonoPP Sql_plus_value_or_text)
      `adhocQ` (pp gpp :: MonoPP Sql_statement)
      `adhocQ` (pp gpp :: MonoPP Start_date)
      `adhocQ` (pp gpp :: MonoPP Statement_block)
      `adhocQ` (pp gpp :: MonoPP Statement_block_no_end)
      `adhocQ` (pp gpp :: MonoPP Storage_clause)
      `adhocQ` (pp gpp :: MonoPP Storage_item)
      `adhocQ` (pp gpp :: MonoPP Storage_items)
      `adhocQ` (pp gpp :: MonoPP Storage_items_opt_brackets)
      `adhocQ` (pp gpp :: MonoPP Subquery)
      `adhocQ` (pp gpp :: MonoPP Subquery_brackets)
      `adhocQ` (pp gpp :: MonoPP Subquery_opt_brackets)
      `adhocQ` (pp gpp :: MonoPP Symbol_dot)
      `adhocQ` (pp gpp :: MonoPP System_item)
      `adhocQ` (pp gpp :: MonoPP System_items)
      `adhocQ` (pp gpp :: MonoPP System_set_item)
      `adhocQ` (pp gpp :: MonoPP System_set_items)
      `adhocQ` (pp gpp :: MonoPP Table_description)
      `adhocQ` (pp gpp :: MonoPP Table_item)
      `adhocQ` (pp gpp :: MonoPP Table_items)
      `adhocQ` (pp gpp :: MonoPP Table_items_opt_brackets)
      `adhocQ` (pp gpp :: MonoPP Table_modify_item)
      `adhocQ` (pp gpp :: MonoPP Table_modify_items)
      `adhocQ` (pp gpp :: MonoPP Table_type_declaration)
      `adhocQ` (pp gpp :: MonoPP Tablespace)
      `adhocQ` (pp gpp :: MonoPP Tablespace_clause)
      `adhocQ` (pp gpp :: MonoPP Tablespace_item)
      `adhocQ` (pp gpp :: MonoPP Tablespace_items)
      `adhocQ` (pp gpp :: MonoPP Territory)
      `adhocQ` (pp gpp :: MonoPP Text)
      `adhocQ` (pp gpp :: MonoPP Then_branch)
      `adhocQ` (pp gpp :: MonoPP Thread_integer)
      `adhocQ` (pp gpp :: MonoPP To_location)
      `adhocQ` (pp gpp :: MonoPP Top_expression)
      `adhocQ` (pp gpp :: MonoPP Top_sort)
      `adhocQ` (pp gpp :: MonoPP Top_sort_cobol_embedded_sql)
      `adhocQ` (pp gpp :: MonoPP Top_sort_pl_sql)
      `adhocQ` (pp gpp :: MonoPP Trailing_clause)
      `adhocQ` (pp gpp :: MonoPP Trailing_declarations)
      `adhocQ` (pp gpp :: MonoPP Trigger_item)
      `adhocQ` (pp gpp :: MonoPP Trigger_items)
      `adhocQ` (pp gpp :: MonoPP Truncate_clause)
      `adhocQ` (pp gpp :: MonoPP Truncate_statement)
      `adhocQ` (pp gpp :: MonoPP Ttitle_date)
      `adhocQ` (pp gpp :: MonoPP Ttitle_item)
      `adhocQ` (pp gpp :: MonoPP Ttitle_items)
      `adhocQ` (pp gpp :: MonoPP Type_name)
      `adhocQ` (pp gpp :: MonoPP Unlabelled_pl_statement)
      `adhocQ` (pp gpp :: MonoPP Update_statement)
      `adhocQ` (pp gpp :: MonoPP User)
      `adhocQ` (pp gpp :: MonoPP User_identification)
      `adhocQ` (pp gpp :: MonoPP User_item)
      `adhocQ` (pp gpp :: MonoPP User_items)
      `adhocQ` (pp gpp :: MonoPP Using_index_clause)
      `adhocQ` (pp gpp :: MonoPP Using_index_item)
      `adhocQ` (pp gpp :: MonoPP Values_description)
      `adhocQ` (pp gpp :: MonoPP Var_declaration)
      `adhocQ` (pp gpp :: MonoPP View_with_clause)
      `adhocQ` (pp gpp :: MonoPP When_clause)
      `adhocQ` (pp gpp :: MonoPP Whenever_sqlerror_clause)
      `adhocQ` (pp gpp :: MonoPP Whenever_sqlerror_statement)
      `adhocQ` (pp gpp :: MonoPP Where_clause)
      `adhocQ` (pp gpp :: MonoPP While_loop_statement)
      `adhocQ` (pp gpp :: MonoPP Lex_name)
      `adhocQ` (pp gpp :: MonoPP Lex_qualified_name)
      `adhocQ` (pp gpp :: MonoPP Lex_host_name)
      `adhocQ` (pp gpp :: MonoPP Lex_qualified_host_name)
      `adhocQ` (pp gpp :: MonoPP Lex_schema_qualifier)
      `adhocQ` (pp gpp :: MonoPP Kw_k_m)
      `adhocQ` (pp gpp :: MonoPP Lex_memory_size)
      `adhocQ` (pp gpp :: MonoPP Lex_keyword_except_to_on_from)
      `adhocQ` (pp gpp :: MonoPP Lex_sql_plus_macro)
