module Language.Pointfree.Parser where

import Language.Pointfree.Syntax
import Language.Haskell.Syntax

hs2pf (HsParen e) = hs2pf e
hs2pf (HsVar (UnQual (HsIdent "id"))) = return ID
hs2pf (HsVar (UnQual (HsIdent "fst"))) = return FST
hs2pf (HsVar (UnQual (HsIdent "snd"))) = return SND
hs2pf (HsVar (UnQual (HsIdent "inl"))) = return INL
hs2pf (HsVar (UnQual (HsIdent "inr"))) = return INR
hs2pf (HsCon (UnQual (HsIdent "Left"))) = return INL
hs2pf (HsCon (UnQual (HsIdent "Right"))) = return INR
hs2pf (HsVar (UnQual (HsIdent "app"))) = return AP
hs2pf (HsVar (UnQual (HsIdent "bang"))) = return BANG
hs2pf (HsVar (UnQual (HsIdent "inn"))) = return IN
hs2pf (HsVar (UnQual (HsIdent "out"))) = return OUT
hs2pf (HsVar (UnQual (HsIdent str))) = return $ Macro str []
hs2pf (HsInfixApp e1 (HsQVarOp (UnQual (HsSymbol "."))) e2)
    = do t1 <- hs2pf e1
         t2 <- hs2pf e2
         return $ t1 :.: t2
hs2pf (HsInfixApp e1 (HsQVarOp (UnQual (HsSymbol "/\\"))) e2)
    = do t1 <- hs2pf e1
         t2 <- hs2pf e2
         return $ t1 :/\: t2
--    hs2pf (HsApp (HsApp (HsVar (UnQual (HsIdent "either"))) e1) e2)
hs2pf (HsInfixApp e1 (HsQVarOp (UnQual (HsSymbol "\\/"))) e2)
    = do t1 <- hs2pf e1
         t2 <- hs2pf e2
         return $ t1 :\/: t2
hs2pf (HsInfixApp e1 (HsQVarOp (UnQual (HsSymbol sym))) e2)
    = do t1 <- hs2pf e1
         t2 <- hs2pf e2
         return $ Macro ('(':sym++")") [t1,t2]
hs2pf (HsApp (HsVar (UnQual (HsIdent "curry"))) e) 
    = hs2pf e >>= return . Curry
hs2pf (HsApp (HsApp (HsApp (HsVar (UnQual (HsIdent "hylo")))
              (HsParen (HsExpTypeSig _ (HsVar (UnQual (HsIdent "_L")))
               (HsQualType [] typ)))) e1) e2)
    = do typ' <- hs2type typ
         t1 <- hs2pf e1
         t2 <- hs2pf e2
         return $ Hylo typ' t1 t2
---- when "Point String" becomes "Point Pointwise.Term":
--hs2pf (HsApp (HsVar (UnQual (HsIdent "pnt"))) pw) 
--    = hs2pw pw >>= return . Point
hs2pf (HsApp x y) -- has to be a parametrized macro
    = do term1 <- hs2pf x
         term2 <- hs2pf y
         case term1 of (Macro v lst) -> return (Macro v (lst++[term2]))
                       x -> fail "macro expected"
hs2pf x = fail "not a valid pf term"



hs2type (HsTyCon (UnQual (HsIdent "One"))) = return One
hs2type (HsTyTuple [e1,e2])
  = do t1 <- hs2type e1
       t2 <- hs2type e2
       return $ t1 :*: t2
hs2type (HsTyApp (HsTyApp (HsTyCon (UnQual (HsIdent "Either"))) e1) e2)
  = do t1 <- hs2type e1
       t2 <- hs2type e2
       return $ t1 :+: t2
hs2type (HsTyFun e1 e2)
  = do t1 <- hs2type e1
       t2 <- hs2type e2
       return $ t1 :-> t2
hs2type (HsTyVar (HsIdent v)) = return $ Base v
hs2type (HsTyApp (HsTyCon (UnQual (HsIdent "Mu"))) e)
  = hs2func e >>= return . Mu
 where
  hs2func (HsTyCon (UnQual (HsIdent "Id"))) = return Id
  hs2func (HsTyApp (HsTyCon (UnQual (HsIdent "Const"))) e)
     = hs2type e >>= return . Const
  -- grammar not rich enough to allow infix constructors
  hs2func (HsTyApp (HsTyApp (HsTyCon (UnQual (HsSymbol ":*:"))) e1) e2)
     = do t1 <- hs2func e1
          t2 <- hs2func e2
          return $ t1 :**: t2
  hs2func (HsTyApp (HsTyApp (HsTyCon (UnQual (HsSymbol ":+:"))) e1) e2)
     = do t1 <- hs2func e1
          t2 <- hs2func e2
          return $ t1 :++: t2
  hs2func _ = fail "not a valid type"

hs2type _ = fail "not a valid type"
