module Language.Pointwise.Pretty where

import Language.Pointwise.Syntax
import Language.Haskell.Syntax

pw2hs :: Term -> HsExp
pw2hs (Var str)     = mkVar str
pw2hs Unit          = mkVar "_L"
pw2hs (Const str)   = mkVar str
pw2hs (t1 :&: t2)   = HsTuple [pw2hs t1,pw2hs t2]
pw2hs (Fst t)       = HsApp (mkVar "fst") (mbParen$ pw2hs t)
pw2hs (Snd t)       = HsApp (mkVar "snd") (mbParen$ pw2hs t)
pw2hs (Case t1 (Lam str2 t2) (Lam str3 t3)) =
      HsCase (pw2hs t1)
          [HsAlt mkLoc (HsPApp (UnQual$ HsIdent "Left") [mkPVar str2])
           (HsUnGuardedAlt$ mbParen$ pw2hs t2) [],
           HsAlt mkLoc (HsPApp (UnQual$ HsIdent "Right") [mkPVar str3])
           (HsUnGuardedAlt$ mbParen$ pw2hs t3) []]
pw2hs (Inl t)       = HsApp (mkCon "Left") (mbParen$ pw2hs t)
pw2hs (Inr t)       = HsApp (mkCon "Right") (mbParen$ pw2hs t)
pw2hs (Lam str t)   = HsLambda mkLoc [mkPVar str] (mbParen (pw2hs t))
pw2hs (t1 :@: t2)   = HsApp (mbParen$ pw2hs t1) (mbParen$ pw2hs t2)
pw2hs (In term)     = HsApp (mkVar "inn") (mbParen$ pw2hs term)
pw2hs (Out term)    = HsApp (mkVar "out") (mbParen$ pw2hs term)
pw2hs (Fix term)    = HsApp (mkVar "fix")  (mbParen$ pw2hs term)
pw2hs (Match t alts)= HsCase (pw2hs t) (map pw2alt alts)
  where
    pw2alt (t1,t2)   = HsAlt mkLoc (pw2pat t1) (HsUnGuardedAlt $ pw2hs t2) []
    pw2pat (Var s)   = mkPVar s
    pw2pat Unit      = HsPWildCard
    pw2pat (Const s) = mkPVar s
    pw2pat (t1 :&: t2) = HsPTuple [pw2pat t1,pw2pat t2]
    pw2pat (Inl t)   = HsPApp (UnQual $ HsIdent "Left") [mbPParen$ pw2pat t]
    pw2pat (Inr t)   = HsPApp (UnQual $ HsIdent "Right") [mbPParen$ pw2pat t]
    pw2pat ((Var str) :@: t2) =
                       HsPApp (UnQual $ HsIdent str) [mbPParen$ pw2pat t2]
    pw2pat t = error $ "not a valid pattern - " ++ show t


mkLoc = SrcLoc "" 0 0
mkCon  = HsCon . UnQual . HsIdent
mkVar  = HsVar . UnQual . HsIdent
mkPVar  = HsPVar . HsIdent
-- places parentisis only if it is necessary
mbParen :: HsExp -> HsExp
mbParen e@(HsApp _ _) = HsParen e
mbParen e@(HsInfixApp _ _ _) = HsParen e
mbParen e@(HsCase _ _) = HsParen e
mbParen e@(HsLambda _ _ _) = HsParen e
mbParen e@(HsExpTypeSig _ _ _) = HsParen e
mbParen x = x
mbPParen :: HsPat -> HsPat
mbPParen p@(HsPApp _ _) = HsPParen p
mbPParen p = p

