
import OOHaskell hiding ((:*:),(:+:),Id)
import Data.Map as Map
import Data.Set as Set

data Fun = Fun String [Term]
data Term = Term Fun

data FunT;     fun   = proxy :: Proxy FunT
data TermT;    term  = proxy :: Proxy TermT

string = proxy :: Proxy String

--

d :: Record (HCons (Proxy FunT, (Proxy String, [Proxy TermT]))
                   (HCons (Proxy TermT, Proxy FunT) HNil))
d =   fun  .=. (string,[term])
  .*. term .=. fun
  .*. emptyRecord

--

newtype Ref a = Ref Int deriving (Ord,Eq,Show)

class Vars v
instance Vars HNil
instance Vars (HCons (Proxy v) HNil)

class FreeVars t v | t -> v
instance FreeVars (Proxy v) (Proxy v)
instance (
    FreeVars a v, 
    FreeVars b w,
    HAppend v w vw
 ) => FreeVars (a,b) vw
instance (
    FreeVars a v, 
    FreeVars b w,
    HAppend v w vw
 ) => FreeVars (Either a b) vw
instance (
    FreeVars a v
 ) => FreeVars [a] v
instance (
    FreeVars a v, 
    FreeVars b w,
    HAppend v w vw
 ) => FreeVars (Map a b) vw

class Fix x fx mf | x fx -> mf
instance Fix HNil HNil HNil
instance (
    Fix xs fxs xsfxs
 ) => Fix (HCons (Proxy x) xs) (HCons fx fxs) (HCons (Map Int fx) xsfxs)
  
class GetAlt x mf fx | x mf -> fx where
 getAlt :: x -> mf -> fx
instance GetAlt (Ref x) (HCons (Map (Ref x) fx) mf) fx where
 getAlt ref (HCons mf _) 
   = maybe (error "OOPS!") id $ Map.lookup ref mf
instance (
    GetAlt x mf fx
 ) => GetAlt x (HCons m mf) fx where
 getAlt x (HCons _ mf) 
   = getAlt x mf 

class Subst t mf t' | t mf -> t' where
 subst :: t -> mf -> t'
instance (
    GetAlt (Ref v) mf t'
 ) => Subst (Ref v) mf t' where
 subst ref mf = getAlt ref mf
instance (
  Subst a mf a',
  Subst b mf b'
 ) => Subst (a,b) mf (a',b') where
 subst (a,b) mf
  = (subst a mf, subst b mf)
instance (
  Subst a mf a',
  Subst b mf b'
 ) => Subst (Either a b) mf (Either a' b') where
 subst (Left a) mf = Left (subst a mf)
 subst (Right a) mf = Right (subst a mf)
instance (
  Subst a mf a'
 ) => Subst [a] mf [a'] where
 subst as mf = Prelude.map (\a->subst a mf) as
instance (
  Subst b mf b'
 ) => Subst (Map a b) mf (Map a b') where
 subst m mf 
   = Map.map (\b -> subst b mf) m 

-- Now we can represent some funs and terms

fRef i = Ref i :: Ref FunT
tRef i = Ref i :: Ref TermT

fun1 = Fun "a" []
frp1 = (fun,("a",[]::[Ref TermT]))

trm1 = Term fun1
trp1 = (term,fRef 0)

fun2 = Fun "f" [trm1,trm1]
frp2 = (fun,("f",[tRef 0,tRef 1 ]))

--

{-
data Lft a s = Lft a
data Rgt a s = Rgt s

class IsSum s
instance IsSum (Lft a ())
instance IsSum s => IsSum (Rgt a s)
instance IsSum s => IsSum (Lft a s)

class Inject a s where
  inject :: a -> s
  select :: s -> a
instance Inject a (Lft a s) where
  inject a = Lft a
  select (Lft a) = a
instance Inject b s => Inject b (Rgt b s) where
  inject b = Rgt (inject b)
  select (Rgt s) = select s


---

-- Sum type is list of (Proxy,Type)
-- But that list holds exactly one element.

class HasAlt l s v | s l -> v where
  getAlt :: l -> s -> v
instance HasAlt
-}

----

class Select n xs x | n xs -> x where
 select :: n -> xs -> x
 inject :: n -> x -> xs
instance Select HZero (Either x xs) x where
 select _ (Left x) = x
 select _ (Right y) = error "Wrong summand"
 inject _ x = Left x
instance (HNat n, Select n xs x) => Select (HSucc n) (Either y xs) x where
 select sn (Right xs) = select (hPred sn) xs
 select sn (Left _) = error "Wrong summand"
 inject sn x = Right (inject (hPred sn) x)

newtype ID n xs = Id {unId :: Select n xs x => x}
newtype K b a       = Const {unConst :: b}       deriving Eq
data (g :+: h) a    = Inl (g a) | Inr (h a)      deriving Eq
data (g :*: h) a    = Pair (g a) (h a)           deriving Eq
newtype (g :@: h) a = Comp {unComp :: g (h a)}   deriving Eq

-- | A type-safe representation of types.
data Type a where

    -- Without fixpoints
    Int :: Type Int
    Bool :: Type Bool
    Char :: Type Char
    String :: Type String
    One :: Type ()
    List :: Type a -> Type [a]
    Set :: Type a -> Type (Set a)
    Map :: Type a -> Type b -> Type (Map a b)
    Either :: Type a -> Type b -> Type (Either a b)
    Prod :: Type a -> Type b -> Type (a,b)
    Tag :: String -> Type a -> Type a

    Mu :: Dist f => Fctr f -> Type (Mu f)
    
    X :: Type X   -- Only for printing!
    
data X

data Fctr f where
    ID    :: n -> Fctr (ID n)
    K     :: Type a -> Fctr (K a)
    (:+:) :: Fctr g -> Fctr h -> Fctr (g:+:h)
    (:*:) :: Fctr g -> Fctr h -> Fctr (g:*:h)
    (:@:) :: Functor g => (forall a . Type a -> Type (g a)) -> Fctr h -> Fctr (g:@:h)
    
class Functor f => Dist f where
    dist :: Monad m => f (m a) -> m (f a)

-- | Explicit fixpoint operator
newtype Mu f = In {out :: f (Mu f)}

instance Eq (f (Mu f)) => Eq (Mu f) where
  (In fmf) == (In gmg) = fmf==gmg

-- | Catamorphism or fold
cata :: Functor f => (f a -> a) -> Mu f -> a
cata g = g . fmap (cata g) . out

-- | Anamorphism or unfold
ana :: Functor f => (a -> f a) -> a -> Mu f
ana g = In . fmap (ana g) . g

-- | Unlift a functor while applying it to a type.
--   Previously: removefunctors.
applyF :: Fctr f -> Type a -> Maybe (View (Type (f a)))
-- applyF (ID n) x    = return (View (Rep {to = (inject n) . unId, from = Id . (select n)}) x)
applyF (K a) x     = return (View (Rep {to = unConst, from = Const}) a)
applyF (g:*:h) x   = do
  (View (Rep g2gx gx2g) gx) <- applyF g x
  (View (Rep h2hx hx2h) hx) <- applyF h x
  let rep = Rep {to = \(Pair x y) -> (g2gx x,h2hx y), from = \(x,y) -> Pair (gx2g x) (hx2h y)}
  return (View rep (Prod gx hx))
applyF (g:+:h) x = do
  (View (Rep g2gx gx2g) gx) <- applyF g x
  (View (Rep h2hx hx2h) hx) <- applyF h x
  let rep = Rep {to = (g2gx-|-h2hx) . aux, from = xua . (gx2g-|-hx2h)}
  return (View rep (Either gx hx))
      where 
        aux (Inl x) = Left x
        aux (Inr x) = Right x
        xua (Left x)  = Inl x
        xua (Right x) = Inr x
applyF (g:@:h) x   = do 
  (View (Rep to from) hx) <- applyF h x
  let rep = Rep (fmap to . unComp) (Comp . fmap from)
  return (View rep (g hx))

-- | Pairs of bi-directional value-level data transformation functions.
data Rep a b = Rep {to :: a -> b, from :: b -> a}

-- | The View constructor expreses that a type a can be represented by
--   a type b, as witnessed by value-level functions that convert 
--   between these types. Note that the target type does not escape, 
--   which means that it is implicitly existentially qualified.
data View a where
    View :: Rep a b -> Type b -> View (Type a)

infix 5 ><
infix 4 /\
infix 3 -|-
infix 2 \/

(f >< g) (x,y) = (f x, g y)
(/\) f g x = (f x, g x)
(f -|- g) (Left x) = Left (f x)
(f -|- g) (Right x) = Right (g x)
(\/) f g (Left x) = f x
(\/) f g (Right x) = g x



