import Data.STRef


data Types a where
  TypesNil :: Types ()
  TypesCons :: Label l => l -> Type a -> Types as -> Types (l,a,as)
  
class DefinedLabels as ls | as -> ls where
  definedLabels :: Types as -> ls
instance DefinedLabels () () where
  definedLabels TypesNil = ()
instance DefinedLabels as ls
  => DefinedLabels (l,a,as) (l,ls) where
  definedLabels (TypesCons l a as)
    = (l,definedLabels as)
    
class Lookup as l a | as l -> a where
  lookupType :: Types as -> l -> a
instance Lookup (l,a,as) l (Type a) where
  lookupType (TypesCons l a as) _ = a
instance Lookup as l a
  => Lookup (k,b,as) l a where
  lookupType (TypesCons _ _ as) l = lookupType as l

data Type a where
  Int :: Type Int
  Prod :: Type a -> Type b -> Type (a,b)
  Either :: Type a -> Type b -> Type (Either a b)
  Tag :: String -> Type a -> Type a
  
  ID :: Label lbl => lbl -> Type a -> Type a
  Ref :: Label lbl => lbl -> Type a -> Type lbl
  
  REF :: Label lbl => CLS lbl a -> Type lbl
  REFF :: Label lbl => CLS lbl a -> Type (lbl,a')
  
  STRef :: STRef s (Type a) -> Type (STRef s a)
  
testSTRef = do
  let myType = Int
  myTypeRef <- newSTRef myType
  let t = STRef myTypeRef
  return ()
  
data Equals a b where
  Eq :: Equals a a
  
data CLS lbl a where
  CLS :: Label lbl => lbl -> Type a -> CLS lbl a

data DynType where
  DynType :: Type a -> DynType
  
class Label lbl where
  showLabel :: lbl -> String
  
data Typed a = a :| Type a
data Classed a where
  (:||) :: a -> CLS lbl a -> Classed  a
  
data A = A deriving Show
data B = B deriving Show

instance Label A where
    showLabel = show

instance Label B where
  showLabel = show

a = ID A (Either (Tag "A1" (Ref B b)) (Tag "A2" Int))
b = ID B (Tag "B1" (Ref A a))


a' = CLS A (Either (Tag "A1" (REF b')) (Tag "A2" Int))
b' = CLS B (Tag "B1" (REF a'))

av1 = Right 1 :|| a'
av2 = Left B :|| a'

a'' = CLS A (Either (Tag "A1" (REFF b'')) (Tag "A2" Int))
b'' = CLS B (Tag "B1" (REFF a''))

av3 = Left (B,Right 1) :|| a''

instance Show (Type a) where
  show Int = "Int"
  show (Prod a b) = "(Prod "++show a++" "++show b++")"
  show (Either a b) = "(Either "++show a++" "++show b++")"
  show (Tag s a) = "(Tag "++show s++" "++show a++")"
  show (ID lbl a) = "(ID "++showLabel lbl++" "++show a++")"
  show (Ref _ (ID lbl a)) =  "(Ref ... "++"(ID "++showLabel lbl++"...))"
  show (Ref lbl a) = "(Ref "++"lbl"++")"
--  show (Ref lbl a) = "(Ref "++showLabel lbl++" "+show a++")"
  show (REF cls@(CLS lbl a)) = "(REF "++show cls++")"

instance Label lbl => Show (CLS lbl a) where
  show (CLS lbl a) = "(CLS "++showLabel lbl++")"
  
printCLS (CLS lbl a) = print a

{-
a = Tag "A" (Either (Tag "A1" b) (Tag "A2" Int))
b = Tag "B" (Tag "B1" (Ref (DynType a)))

instance Show (Type a) where
  show Int = "Int"
  show (Prod a b) = "(Prod "++show a++" "++show b++")"
  show (Either a b) = "(Either "++show a++" "++show b++")"
  show (Tag s a) = "(Tag "++show s++" "++show a++")"
  show (Ref dt) = "(Ref "++show dt++")"

instance Show DynType where
  show (DynType t) = show t
  
  
showType :: Type a -> String
showType t = aux [] t
 where
   aux :: [String] -> Type a -> String
   aux done Int = "Int"
   aux done (Prod a b) = "(Prod "++aux done a++" "++aux done b++")"
   aux done (Either a b) = "(Either "++aux done a++" "++aux done b++")"
   aux done (Tag s a)
     | s `elem` done = "(Tag "++show s++" ...)"
     | otherwise = "(Tag "++show s++" "++aux (s:done) a++")"
   aux done (Ref dt) = "(Ref "++showDT done dt++")"
   
   showDT done (DynType t) = aux done t
-}